

#include <xview/xview.h>
#include <xview/panel.h>
#include "snafu_impl.h"

Attr_attribute INSTANCE = 0;	/* may need to define this in main() */
static Attr_attribute SNAFU_KEY = 0;

static void	snafu_reset_proc ( Panel_item item, Event *event );
static void	snafu_apply_proc ( Panel_item item, Event *event );
static void	snafu_set_value ( Snafu_values *value, Snafu_ui *ui );
static void	snafu_accept_value ( Snafu_values *value, Snafu_ui *ui );
static void	snafu_create_ui ( Snafu_private *private );


/*
 * xv_create() method
 */
Pkg_private int
snafu_init ( Xv_opaque owner, Snafu_public *public, Attr_avlist avlist )
{
    Snafu_private *private = xv_alloc ( Snafu_private );

    public->private_data = (Xv_opaque) private;
    private->public_self = (Xv_opaque) public;

    /* 
     * setup default values and private data
     * Note:  good thing to use resources for...
     */
    private->value		= xv_alloc ( Snafu_values );
    strcpy ( private->value->name, "Unnamed Snafu" );
    private->value->severity	= 50;
    private->value->type 	= SNAFU_NUCLEAR;

    private->apply_proc		= (void (*)())NULL;
    private->owner		= owner;
    strcpy ( private->label, "" );

    /*
     * Note:  to aid in invocation performance, we don't create
     * the UI objects here.  instead we wait until XV_SHOW is
     * set to TRUE.
     */

    return XV_OK;
} /* snafu_init() */




/*
 * xv_set() method
 */
Pkg_private Xv_opaque
snafu_set ( Snafu public, Attr_avlist avlist )
{
    Snafu_private *private = SNAFU_PRIVATE(public);
    Attr_avlist attrs;

    for (attrs=avlist; *attrs; attrs=attr_next(attrs)) {
	switch ( (int) attrs[0] ) {
	case SNAFU_TYPE:
	    private->value->type = (Snafu_type) attrs[1];
	    if ( private->ui )
		xv_set ( private->ui->type_choice, PANEL_VALUE, private->value->type, NULL );
	    ATTR_CONSUME(attrs[0]);
	    break;

	case SNAFU_NAME:
	    strcpy ( private->value->name, (char *)attrs[1] );
	    if ( private->ui )
		xv_set ( private->ui->name_text, PANEL_VALUE, private->value->name, NULL );
	    ATTR_CONSUME(attrs[0]);
	    break;

	case SNAFU_SEVERITY:
	    private->value->severity = (int) attrs[1];
	    if ( private->ui )
		xv_set ( private->ui->severity_slider, PANEL_VALUE, private->ui->severity_slider, NULL );
	    ATTR_CONSUME(attrs[0]);
	    break;

	case SNAFU_APPLY_PROC:
	    private->apply_proc = (void (*)()) attrs[1];
	    ATTR_CONSUME(attrs[0]);
	    break;

	case XV_LABEL:
	    strcpy ( private->label, (char *)attrs[1] );
	    if ( private->ui )
		xv_set ( private->ui->popup, XV_LABEL, private->label, NULL );
	    ATTR_CONSUME(attrs[0]);
	    break;

	case XV_SHOW:
	    /* create UI objects when they need to be mapped! */
	    if ( ((int)attrs[1] != 0) && !private->ui )
		snafu_create_ui ( private );
	    xv_set ( private->ui->popup, XV_SHOW, (int)attrs[1], NULL );
	    ATTR_CONSUME(attrs[0]);
	    break;

	case XV_END_CREATE:
	    break;

	default:
	    xv_check_bad_attr ( SNAFU, attrs[0] );
	    break;
	} /* switch() */
    } /* for() */

    return XV_OK;
} /* snafu_set() */



/*
 * xv_get() method
 */
Pkg_private Xv_opaque
snafu_get ( Snafu_public   *public, 
	   int             *status,
	   Attr_attribute  attr,
	   Attr_avlist     args 
	   )
{
    Snafu_private *private = SNAFU_PRIVATE(public);

    /*
     * Note:  always "get" the validated value, not the
     * unvalidated popup contents for Snafu attributes.  
     * the Panel package should do this also, but....
     */
    switch ( (int) attr ) {
    case SNAFU_TYPE:
	return (Xv_opaque) private->value->type;

    case SNAFU_NAME:
	return (Xv_opaque) private->value->name;

    case SNAFU_SEVERITY:
	return (Xv_opaque) private->value->severity;

    case SNAFU_APPLY_PROC:
	return (Xv_opaque) private->apply_proc;

    case XV_LABEL:
	return (Xv_opaque) private->label;

    case XV_OWNER:
	return (Xv_opaque) private->owner;

    case XV_SHOW:
	if ( private->ui )
	    return (Xv_opaque) xv_get ( private->ui->popup, XV_SHOW );
	else
	    return (Xv_opaque) FALSE;

    default :
	*status = xv_check_bad_attr ( SNAFU, attr );
	return (Xv_opaque)XV_OK;
    } /* switch */

} /* snafu_get() */



/*
 * xv_destroy() method
 */
Pkg_private int
snafu_destroy ( Snafu_public *public, Destroy_status status )
{
    Snafu_private *private = SNAFU_PRIVATE(public);

    if (status == DESTROY_CLEANUP) {
	if ( private->ui )
	    xv_free ( private->ui );
	xv_free ( private->value );
        xv_free ( private );
    }
    return XV_OK;
} /* snafu_destroy() */


/*******************************************************************************/



/*
 * We want to create the UI portion of the Snafu seprately
 * from the rest of the object.  because of mulitple calls
 * to xv_create() can be expensive, we want to delay creating
 * the UI until XV_SHOW gets set to TRUE.  hopefully, this will
 * help invocation performance.
 */
static void
snafu_create_ui ( Snafu_private *private )
{

    /* used to attach private data to popup objects */
    if ( SNAFU_KEY == 0 )
	SNAFU_KEY = xv_unique_key();

    /* DevGuide code requires this */
    if ( INSTANCE == 0 )
	INSTANCE = xv_unique_key();
    
    private->ui = snafu_popup_objects_initialize ( NULL, private->owner );
    
    /* Post DevGuide Setup */
    xv_set ( private->ui->popup,
	    XV_LABEL,	private->label,
	    NULL );
    xv_set ( private->ui->controls, 
	    PANEL_DEFAULT_ITEM,	private->ui->apply_button,
	    NULL );
    xv_set ( private->ui->apply_button,
	    XV_KEY_DATA,	SNAFU_KEY,	private,
	    PANEL_NOTIFY_PROC,	snafu_apply_proc,
	    NULL );
    xv_set ( private->ui->reset_button,
	    XV_KEY_DATA,	SNAFU_KEY,	private,
	    PANEL_NOTIFY_PROC,	snafu_reset_proc,
	    NULL );

    /* make sure popup reflects our default values! */
    snafu_set_value ( private->value, private->ui );

} /* snafu_create_ui() */



/*
 * Restore the popup to the state of it's last validation
 * implemented as callback to "Reset" button.
 */
static void
snafu_reset_proc ( Panel_item item, Event *event )
{
    Snafu_private *private = SNAFU_FROM_OBJ(item);
    snafu_set_value ( private->value, private->ui );
} /* snafu_reset_proc() */


/*
 * Make the popup data current and notify the app
 * tha the "Apply" button was pressed.
 */
static void
snafu_apply_proc ( Panel_item item, Event *event )
{
    Snafu_private *private = SNAFU_FROM_OBJ(item);

    snafu_accept_value ( private->value, private->ui );

    /* maybe pass back a structure? */
    if ( private->apply_proc )
	(private->apply_proc) ( SNAFU_PUBLIC(private),
			       private->value->name,
			       private->value->severity,
			       private->value->type
			       );
} /* snafu_reset_proc() */



/*
 * set popup contents from value struct
 */
static void
snafu_set_value ( Snafu_values *value, Snafu_ui *ui )
{
    if ( !ui )
	return;

    xv_set ( ui->name_text, PANEL_VALUE, value->name, NULL );
    xv_set ( ui->severity_slider, PANEL_VALUE, value->severity, NULL );
    xv_set ( ui->type_choice, PANEL_VALUE, value->type, NULL );
} /* snafu_set_value() */


/*
 * set values struct from popup contents
 */
static void
snafu_accept_value ( Snafu_values *value, Snafu_ui *ui )
{
    if ( !ui )
	return;

    strcpy ( value->name, (char *) xv_get(ui->name_text, PANEL_VALUE) );
    value->severity = (int) xv_get ( ui->severity_slider, PANEL_VALUE );
    value->type = (Snafu_type) xv_get ( ui->type_choice, PANEL_VALUE );
} /* snafu_accept_value() */
