/* frame.busy.c -- Making a base frame and it's subframes "busy"
 * Author:  Kenneth Bibb
 * May 1992 -- Arastar Consulting
 *--------------------------------------
 */

#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/canvas.h>
#include <xview/notify.h>

Attr_attribute FRAME_BASE_KEY;
Attr_attribute FRAME_1_KEY;
Attr_attribute FRAME_2_KEY;
Attr_attribute FRAME_3_KEY;
Attr_attribute FRAME_SUB1_KEY;

/* the following routine is from Heller/Van Raalte's XVPM */
void xv_sleep(seconds)
unsigned seconds;
{
	int	oldmask, mask;
	struct timeval tv;

	tv.tv_sec = seconds;
	tv.tv_usec = 0;
	
	mask = sigmask(SIGIO);
	mask |= sigmask(SIGALRM);
	oldmask = sigblock(mask);

	if((select(0,0,0,0,&tv)) == -1)
	{
		perror("select");
	}
	sigsetmask(oldmask);
}

void sites(button, event)
Panel_item button;
Event *event;
{
	Frame base = (Frame)xv_get(button,XV_KEY_DATA,FRAME_BASE_KEY);
	Frame sub;
	int   i;



/* an example of a hardcoded walk through the frame tree
 * I count backwards to force the windows into the pattern you'll
 * see when you run the program.
 */
	for(i=3; i; i--)
	{
		sub = (Frame)xv_get(base,FRAME_NTH_SUBFRAME,i);

		/* I've hardcoded the subsubframe here--an example of
		 * dynamically modifying it is in push_me_proc()
		 */
		if(i == 1)
		{
			xv_set((Frame)xv_get(sub,FRAME_NTH_SUBFRAME,1),
				XV_SHOW, TRUE,
				NULL);
		}

		xv_set(sub,
			XV_SHOW, TRUE,
			NULL);
	}
}

void push_me_proc(button,event)
Panel_item button;
Event *event;
{
	char s[64];
	Frame base=(Frame)xv_get(button,XV_KEY_DATA,FRAME_BASE_KEY);
	Frame sub, sub2;
	int i,j;




	xv_set(base,
		FRAME_LEFT_FOOTER,"Button: [Make Frames Busy for 10 Seconds]\n",
		NULL);
/* make the base frame busy:  the subframe walk won't get this
 */
	xv_set(base,
		FRAME_BUSY, TRUE,
		NULL);



	for(i=1; ; i++)		/* start walking through the subframes */
	{

		/* this is the key statement:  the FRAME_NTH_SUBFRAME
		 * attribute will return a subframe handle.  It returns
		 * them in the order created, so you could hardcode control
		 * of a certain frame by using DEFINEs or whatever
		 */
		sub = (Frame)xv_get(base,FRAME_NTH_SUBFRAME,i);

		if(sub)		/* when you run out of subframes, sub == NULL */
		{
			xv_set(sub,
				FRAME_BUSY, TRUE,
				NULL);
			for(j=1; ; j++)	/* walk through subsubframes */
			{
				sub2 = (Frame)xv_get(sub,FRAME_NTH_SUBFRAME,j);
				if(sub2)
				{
					xv_set(sub2,
						FRAME_BUSY, TRUE,
						NULL);
				}
				else
					break;
			}
		}
		else
			break;
	}




	xv_sleep(5);		/* wait for a sec so you can see it worked */



	xv_set(base,		/* reset base frame */
		FRAME_BUSY, FALSE,
		NULL);



	for(i=1; ; i++)
	{
		sub = (Frame)xv_get(base,FRAME_NTH_SUBFRAME,i);
		if(sub)
		{
			xv_set(sub,
				FRAME_BUSY, FALSE,
				NULL);
			for(j=1; ; j++)
			{
				sub2 = (Frame)xv_get(sub,FRAME_NTH_SUBFRAME,j);
				if(sub2)
				{
					xv_set(sub2,
						FRAME_BUSY, FALSE,
						NULL);
				}
				else
					break;
			}
		}
		else
			break;
	}



	sprintf(s,"Button: [%s]\n", (char *)xv_get(button, PANEL_LABEL_STRING));
	xv_set(base,FRAME_LEFT_FOOTER,s, NULL);
}

int main(argc, argv)
int argc;
char **argv;
{
	Frame		frame;
	Frame		frame1;		/* subframes */
	Frame		frame1_1;	/* subframe of frame1 */
	Frame		frame2;
	Frame		frame3;
	Canvas		canvas;
	Panel		panel;
	Panel		p1;
	Panel		p2;
	Panel		p3;
	Panel		p1_1;




	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);

	FRAME_BASE_KEY = xv_unique_key();
	FRAME_1_KEY = xv_unique_key();
	FRAME_2_KEY = xv_unique_key();
	FRAME_3_KEY = xv_unique_key();
	FRAME_SUB1_KEY = xv_unique_key();




	/* create the frames
	 */
	frame = (Frame)xv_create(NULL, FRAME,
		FRAME_LABEL,		argv[0],
		FRAME_SHOW_FOOTER,	TRUE,
		XV_X,			300,
		XV_Y,			175,
		NULL);
	frame1 = (Frame)xv_create(frame, FRAME,
		FRAME_LABEL,	"Subframe1",
		XV_X,		350,
		XV_Y,		300,
		NULL);
	frame2 = (Frame)xv_create(frame, FRAME,
		FRAME_LABEL,	"Subframe2",
		XV_X,		375,
		XV_Y,		325,
		NULL);
	frame3 = (Frame)xv_create(frame, FRAME,
		FRAME_LABEL,	"Subframe3",
		XV_X,		400,
		XV_Y,		350,
		NULL);
	frame1_1 = (Frame)xv_create(frame1, FRAME_CMD,
		FRAME_LABEL,	"SubSubframe",
		XV_X,		425,
		XV_Y,		375,
		NULL);




	/* panel creation stage
	 */
	panel = (Panel)xv_create(frame, PANEL, NULL);
	p1    = (Panel)xv_create(frame1, PANEL, NULL);
	p2    = (Panel)xv_create(frame2, PANEL, NULL);
	p3    = (Panel)xv_create(frame3, PANEL, NULL);
	p1_1  = (Panel)xv_get(frame1_1, FRAME_CMD_PANEL);

	(void)xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"View Subframes...",
		PANEL_NOTIFY_PROC,	sites,
		XV_KEY_DATA,		FRAME_BASE_KEY, frame,
		NULL);



	
	/* panel item creation stage
	 */
	(void)xv_create(p1,PANEL_BUTTON,
		PANEL_LABEL_STRING,	"Push Me",
		PANEL_NOTIFY_PROC,	push_me_proc,
		XV_KEY_DATA,		FRAME_BASE_KEY, frame,
		NULL);
	(void)xv_create(p2,PANEL_MESSAGE,
		PANEL_LABEL_STRING,	"Sub Frame 2",
		NULL);
	(void)xv_create(p3,PANEL_MESSAGE,
		PANEL_LABEL_STRING,	"Sub Frame 3",
		NULL);
	(void)xv_create(p1_1,PANEL_MESSAGE,
		PANEL_LABEL_STRING,	"Sub SubFrame",
		NULL);



	/* make everything look nice
	 */
	window_fit(p1_1);
	window_fit(p3);
	window_fit(p2);
	window_fit(p1);
	window_fit(frame1_1);
	window_fit(frame3);
	window_fit(frame2);
	window_fit(frame1);
	window_fit_height(panel);



	canvas = (Canvas)xv_create(frame, CANVAS, 
		XV_X,			0,
		XV_WIDTH,		WIN_EXTEND_TO_EDGE,
		XV_HEIGHT,		WIN_EXTEND_TO_EDGE,
		/* CANVAS_X_PAINT_WINDOW,	TRUE,*/
		NULL);

	window_fit(frame);



	xv_main_loop(frame);
	return(0);
}

