/* help.c -- demonstration of different kinds of help
 * Author:  Kenneth Bibb
 * March 1992 -- Arastar Consulting
 *-------------------------------------
 *
 * This is the button.c program with help attached to it.
 *
 */

#include <stdlib.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/canvas.h>
#include <xview/xv_xrect.h>
#include <xview/openmenu.h>

/* set up key data variables
 */
Attr_attribute BASE_FRAME_KEY=NULL;
Attr_attribute POPUP_WINDOW_KEY=NULL;
Attr_attribute POPUP_PANEL_KEY=NULL;
Attr_attribute BUTTON_HI_KEY=NULL;
Attr_attribute BUTTON_IN_KEY=NULL;
Attr_attribute BUTTON_BU_KEY=NULL;
Attr_attribute BUTTON_CA_KEY=NULL;
Attr_attribute BUTTON_ME_KEY=NULL;


/* test menu stub
 */
void menu_stub(menu, menu_item)
Menu menu;
Menu_item menu_item;
{
	char s[64];
	Frame frame = (Frame)xv_get(menu, XV_KEY_DATA, BASE_FRAME_KEY, NULL);

	sprintf(s,"Menu: [%s]\n", xv_get(menu_item, MENU_STRING));
	(void)xv_set(frame, FRAME_LEFT_FOOTER, s, NULL);
}

/* highlight a button
 */
void highlight(button, event)
Panel_item button;
Event *event;
{
}

/* inactivate all buttons except the "inactivate" button
 */
void inactive(button, event)
Panel_item button;
Event *event;
{
	Panel Pbutton = (Panel)xv_get(button, XV_KEY_DATA, POPUP_PANEL_KEY);
	Panel_item ph = (Panel_item)xv_get(button, XV_KEY_DATA, BUTTON_HI_KEY);
	Panel_item pi = (Panel_item)xv_get(button, XV_KEY_DATA, BUTTON_IN_KEY);
	Panel_item pb = (Panel_item)xv_get(button, XV_KEY_DATA, BUTTON_BU_KEY);
	Panel_item pc = (Panel_item)xv_get(button, XV_KEY_DATA, BUTTON_CA_KEY);
	Panel_item pm = (Panel_item)xv_get(button, XV_KEY_DATA, BUTTON_ME_KEY);
	Panel_item i  = NULL;

	/* if the highlight button is inactive
	 *	activate all panel items (in this case, buttons)
	 * else
	 *	inactivate all panel items (in this case, buttons)
	 *	activate the inactivate button
	 */
	if(xv_get(ph, PANEL_INACTIVE) == TRUE)
	{
		xv_set(ph,
			PANEL_INACTIVE, FALSE,
			XV_HELP_DATA,	"help:Highlight",
			NULL);
		xv_set(pb,
			PANEL_INACTIVE, FALSE,
			XV_HELP_DATA,	"help:BusyButton",
			NULL);
		xv_set(pc,
			PANEL_INACTIVE, FALSE,
			XV_HELP_DATA,	"help:Cancel",
			NULL);
		xv_set(pm,
			PANEL_INACTIVE, FALSE,
			XV_HELP_DATA,	"help:MenuRight",
			NULL);
	}
	else
	{
		PANEL_EACH_ITEM(Pbutton, i)
			xv_set(i,
				PANEL_INACTIVE, TRUE,
				XV_HELP_DATA,	"help:InactiveButton",
				NULL);
		PANEL_END_EACH
		xv_set(button,
			PANEL_INACTIVE, FALSE,
			XV_HELP_DATA,		"help:inactivebutton",
			NULL);
	}
}

/* busy all buttons except the "Busy" button
 */
void busy(button, event)
Panel_item button;
Event *event;
{
	Panel Pbutton = (Panel)xv_get(button, XV_KEY_DATA, POPUP_PANEL_KEY);
	Panel_item ph = (Panel_item)xv_get(button, XV_KEY_DATA, BUTTON_HI_KEY);
	Panel_item pi = (Panel_item)xv_get(button, XV_KEY_DATA, BUTTON_IN_KEY);
	Panel_item pb = (Panel_item)xv_get(button, XV_KEY_DATA, BUTTON_BU_KEY);
	Panel_item pc = (Panel_item)xv_get(button, XV_KEY_DATA, BUTTON_CA_KEY);
	Panel_item pm = (Panel_item)xv_get(button, XV_KEY_DATA, BUTTON_ME_KEY);
	Panel_item i  = NULL;

	/* if the highlight button is busy
	 *	turn busy off on all panel items
	 * else
	 *	turn busy on for all panel items
	 *	turn busy off on busy button
	 */
	if(xv_get(ph, PANEL_BUSY) == TRUE)
	{
		xv_set(ph,
			PANEL_BUSY, FALSE,
			XV_HELP_DATA,	"help:Highlight",
			NULL);
		xv_set(pi,
			PANEL_BUSY, FALSE,
			XV_HELP_DATA,	"help:BusyButton",
			NULL);
		xv_set(pc,
			PANEL_BUSY, FALSE,
			XV_HELP_DATA,	"help:Cancel",
			NULL);
		xv_set(pm,
			PANEL_BUSY, FALSE,
			XV_HELP_DATA,	"help:MenuRight",
			NULL);
	}
	else
	{
		PANEL_EACH_ITEM(Pbutton, i)
			xv_set(i,
				PANEL_BUSY, TRUE,
				XV_HELP_DATA, "help:BusyButtons",
				NULL);
		PANEL_END_EACH
		xv_set(button, PANEL_BUSY, FALSE, NULL);
	}
}

/* "Button Window..." callback:  display button window
 */
void button_window(button, event)
Panel_item button;
Event *event;
{
	xv_set(xv_get(button, XV_KEY_DATA, POPUP_WINDOW_KEY),
		XV_SHOW, TRUE, NULL);
}

/* "Cancel" button callback
 */
void cancel(button, event)
Panel_item button;
Event *event;
{
	Frame frame = (Frame)xv_get(button, XV_KEY_DATA, POPUP_WINDOW_KEY);

	/* pull the pushpin and unmap the window
	 */
	(void)xv_set(frame,
		FRAME_CMD_PIN_STATE,	FALSE,
		XV_SHOW,		FALSE,
		NULL);
}

/* display the popup window
 */
void show_it(window, event)
Xv_Window window;
Event *event;
{
	if(event_action(event) == ACTION_MENU && event_is_down(event))
	{
		Menu m = (Menu)xv_get(window, WIN_CLIENT_DATA);
		menu_show(m, window, event, NULL);
	}
	else if(event_action(event) == ACTION_HELP)
		xv_help_show(window, "help:general", event);
}

/* repaint the canvas
 */
void repaint(canvas, win, dpy, xwin, xrect)
Canvas canvas;
Xv_Window win;
Display *dpy;
Window xwin;
Xv_xrectlist *xrect;
{
	GC gc;
	int width, height;

	gc = DefaultGC(dpy, DefaultScreen(dpy));
	width = (int)xv_get(win, XV_WIDTH);
	height = (int)xv_get(win, XV_HEIGHT);
}

int main(argc, argv)
int argc;
char **argv;
{
	Frame		frame;
	Frame		Fbutton;
	Panel		Pbutton;
	Panel_item	ph, pi, pb, pc, mb;
	Canvas		canvas;
	Panel		panel;
	Menu		m, m_undo;



	/* initialization
	 */
	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);
	BASE_FRAME_KEY	 = xv_unique_key();
	POPUP_WINDOW_KEY = xv_unique_key();
	POPUP_PANEL_KEY	 = xv_unique_key();
	BUTTON_HI_KEY	 = xv_unique_key();
	BUTTON_IN_KEY	 = xv_unique_key();
	BUTTON_BU_KEY	 = xv_unique_key();
	BUTTON_CA_KEY	 = xv_unique_key();
	BUTTON_ME_KEY	 = xv_unique_key();

	/* create base frame
	 */
	frame = (Frame)xv_create(XV_NULL, FRAME,
		FRAME_LABEL,	argv[0],
		FRAME_SHOW_FOOTER, TRUE,
		XV_X,		300,
		XV_Y,		175,
		NULL);

	/* create button subframe
	 */
	Fbutton = (Frame)xv_create(frame, FRAME_CMD,
		FRAME_LABEL,	"Button Examples",
		XV_X,		400,
		XV_Y,		200,
		NULL);

	/* retrieve subframe's panel
	 */
	Pbutton = (Panel)xv_get(Fbutton, FRAME_CMD_PANEL);
	xv_set(Pbutton,
		PANEL_LAYOUT, PANEL_VERTICAL,
		XV_HELP_DATA,	"help:buttonpanel",
		NULL);

	panel = (Panel)xv_create(frame, PANEL,
		XV_HELP_DATA,	"help:general",
		NULL);

	/* create file menu
	 */
	m = (Menu)xv_create(XV_NULL, MENU,
		MENU_ITEM,
			MENU_STRING,	"New...",
			NULL,
		MENU_ITEM,
			MENU_STRING, "Save...",
			NULL,
		MENU_ITEM,
			MENU_STRING, "Print...",
			NULL,
		MENU_NOTIFY_PROC, menu_stub,
		XV_KEY_DATA, BASE_FRAME_KEY, frame,
		NULL);
	(void)xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"File",
		PANEL_ITEM_MENU,	m,	
		NULL);

	/* command window button
	 *	display "Button Window"
	 */
	(void)xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"Button Window...",
		PANEL_NOTIFY_PROC,	button_window,
		XV_KEY_DATA,		POPUP_WINDOW_KEY, Fbutton,
		NULL);

/*-------------B u t t o n    W i n d o w-------------------*/

	/* create the "Highlight button
	 */
	ph = xv_create(Pbutton,		PANEL_BUTTON,
		XV_HELP_DATA,		"help:Highlight",
		PANEL_LABEL_STRING,	"Highlight",
		PANEL_NOTIFY_PROC,	highlight,
		NULL);

	/* create the "Inactive" button
	 */
	pi = xv_create(Pbutton,		PANEL_BUTTON,
		XV_HELP_DATA,		"help:inactivebutton",
		PANEL_LABEL_STRING,	"Inactive",
		PANEL_NOTIFY_PROC,	inactive,
		NULL);

	/* create the Busy signal
	 */
	pb = xv_create(Pbutton,		PANEL_BUTTON,
		XV_HELP_DATA,		"help:BusyButton",
		PANEL_LABEL_STRING,	"Busy",
		PANEL_NOTIFY_PROC,	busy,
		NULL);

	/* create the "Cancel" button
	 */
	pc = (Panel_item)xv_create(Pbutton, PANEL_BUTTON,
		XV_HELP_DATA,		"help:Cancel",
		PANEL_LABEL_STRING,	"Cancel",
		PANEL_NOTIFY_PROC,	cancel,
		NULL);

	/* create the "Menu Right" button
	 */
	m = (Menu)xv_create(XV_NULL, MENU,
		MENU_ITEM,
			MENU_STRING, "Marillion",
			XV_HELP_DATA, "help:Marillion",
			NULL,
		MENU_ITEM,
			MENU_STRING, "Genesis",
			XV_HELP_DATA, "help:Genesis",
			NULL,
		MENU_ITEM,
			MENU_STRING, "IQ",
			XV_HELP_DATA, "help:IQ",
			NULL,
		MENU_ITEM,
			MENU_STRING, "Twelfth Night",
			XV_HELP_DATA, "help:TwelfthNight",
			NULL,
		MENU_NOTIFY_PROC, menu_stub,
		XV_KEY_DATA, BASE_FRAME_KEY, frame,
		NULL);
	mb = (Panel_item)xv_create(Pbutton, PANEL_BUTTON,
		XV_HELP_DATA,		"help:MenuRight",
		PANEL_LABEL_STRING,	"Menu Right",
		PANEL_ITEM_MENU,	m,
		NULL);
	/* set the key data on the buttons
	 */
	(void)xv_set(ph,
		XV_KEY_DATA,		BUTTON_HI_KEY, ph,
		XV_KEY_DATA,		BUTTON_IN_KEY, pi,
		XV_KEY_DATA,		BUTTON_BU_KEY, pb,
		XV_KEY_DATA,		BUTTON_CA_KEY, pc,
		XV_KEY_DATA,		BUTTON_ME_KEY, mb,
		XV_KEY_DATA,		POPUP_WINDOW_KEY, Fbutton,
		XV_KEY_DATA,		POPUP_PANEL_KEY, Pbutton,
		NULL);
	(void)xv_set(pi,
		XV_KEY_DATA,		BUTTON_HI_KEY, ph,
		XV_KEY_DATA,		BUTTON_IN_KEY, pi,
		XV_KEY_DATA,		BUTTON_BU_KEY, pb,
		XV_KEY_DATA,		BUTTON_CA_KEY, pc,
		XV_KEY_DATA,		BUTTON_ME_KEY, mb,
		XV_KEY_DATA,		POPUP_WINDOW_KEY, Fbutton,
		XV_KEY_DATA,		POPUP_PANEL_KEY, Pbutton,
		NULL);
	(void)xv_set(pb,
		XV_KEY_DATA,		BUTTON_HI_KEY, ph,
		XV_KEY_DATA,		BUTTON_IN_KEY, pi,
		XV_KEY_DATA,		BUTTON_BU_KEY, pb,
		XV_KEY_DATA,		BUTTON_CA_KEY, pc,
		XV_KEY_DATA,		BUTTON_ME_KEY, mb,
		XV_KEY_DATA,		POPUP_WINDOW_KEY, Fbutton,
		XV_KEY_DATA,		POPUP_PANEL_KEY, Pbutton,
		NULL);
	(void)xv_set(pc,
		XV_KEY_DATA,		BUTTON_HI_KEY, ph,
		XV_KEY_DATA,		BUTTON_IN_KEY, pi,
		XV_KEY_DATA,		BUTTON_BU_KEY, pb,
		XV_KEY_DATA,		BUTTON_CA_KEY, pc,
		XV_KEY_DATA,		BUTTON_ME_KEY, mb,
		XV_KEY_DATA,		POPUP_WINDOW_KEY, Fbutton,
		XV_KEY_DATA,		POPUP_PANEL_KEY, Pbutton,
		NULL);
	(void)xv_set(mb,
		XV_KEY_DATA,		BUTTON_HI_KEY, ph,
		XV_KEY_DATA,		BUTTON_IN_KEY, pi,
		XV_KEY_DATA,		BUTTON_BU_KEY, pb,
		XV_KEY_DATA,		BUTTON_CA_KEY, pc,
		XV_KEY_DATA,		BUTTON_ME_KEY, mb,
		XV_KEY_DATA,		POPUP_WINDOW_KEY, Fbutton,
		XV_KEY_DATA,		POPUP_PANEL_KEY, Pbutton,
		NULL);
	/* make "Cancel" the default button
	 */
	(void)xv_set(Pbutton, PANEL_DEFAULT_ITEM, pc, NULL);

	/* make everything nice
	 */
	window_fit_height(Pbutton);
	window_fit(Fbutton);
	window_fit_height(panel);

	/* set up the canvas
	 */
	canvas = (Canvas)xv_create(frame, CANVAS, 
		XV_X,			0,
		XV_WIDTH,		WIN_EXTEND_TO_EDGE,
		XV_HEIGHT,		WIN_EXTEND_TO_EDGE,
		CANVAS_REPAINT_PROC,	repaint,
		CANVAS_X_PAINT_WINDOW,	TRUE,
		NULL);
	window_fit(frame);

	/* create the popup */
	m = (Menu)xv_create(XV_NULL,	MENU,
		XV_HELP_DATA,		"help:popup",
		MENU_GEN_PIN_WINDOW,	frame, "Popup Menu",
		MENU_ACTION_ITEM,	"Quit", exit,
		NULL);
	xv_set(canvas_paint_window(canvas),
		WIN_CONSUME_EVENTS,	WIN_MOUSE_BUTTONS,
					ACTION_HELP,
					NULL,
		WIN_EVENT_PROC,		show_it,
		WIN_CLIENT_DATA,	m,
		NULL);

	xv_main_loop(frame);
	return(0);
}

