/* icon.c -- Icon test program
 * Author:  Kenneth Bibb
 * April 1992 -- Arastar Consulting
 *--------------------------------------
 *
 * o Icon State animation
 *	animation based on some state
 * o Icon Time animation
 *	animation based on a timer
 */

#include <stdlib.h>
#include <sys/param.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/xv_xrect.h>
#include <xview/openmenu.h>
#include <xview/svrimage.h>
#include <xview/icon.h>
#include <xview/notify.h>

/* external bitmaps for the state animation
 * are state0_image and state1_image
 */
short state0_image[]={
#include "state0.icon"
};

short state1_image[]={
#include "state1.icon"
};


/* external bitmaps for the time animation
 * are anime[0-8]_image
 */
short anime0_image[]={
#include "a0.icon"
};

short anime1_image[]={
#include "a1.icon"
};

short anime2_image[]={
#include "a2.icon"
};

short anime3_image[]={
#include "a3.icon"
};

short anime4_image[]={
#include "a4.icon"
};

short anime5_image[]={
#include "a5.icon"
};

short anime6_image[]={
#include "a6.icon"
};

short anime7_image[]={
#include "a7.icon"
};

short anime8_image[]={
#include "a8.icon"
};


Attr_attribute	BASE_FRAME_KEY;		/* handle of base frame */
Attr_attribute	STATE_FRAME_KEY;	/* handle of state frame */
Attr_attribute	ANI_FRAME_KEY;		/* handle of animate frame */
Attr_attribute	S0_ICON_KEY;		/* handle of state icon */

Server_image	s0image;		/* server images for the bitmaps */
Server_image	s1image;
Server_image	a0image;
Server_image	a1image;
Server_image	a2image;
Server_image	a3image;
Server_image	a4image;
Server_image	a5image;
Server_image	a6image;
Server_image	a7image;
Server_image	a8image;

Icon	animate0;		/* the icon for the time animation window  */
int current_state=0;		/* which state icon image is used          */
int animate_state=0;		/* which part of the animation are we in   */
int anime_toggle=FALSE;		/* start/pause the animation               */
int slider_val=60;		/* frame time delay slider controlled      */
struct itimerval t;		/* timer structure used by timing function */





/*================================================================
 * State Animation callbacks
 *----------------------------------------------------------------
 */

/* the "State..." menu item's callback
 * this function displays the state animation frame
 */
void state(m,mi)
Menu m;
Menu_item mi;
{
	xv_set((Frame)xv_get(mi,XV_KEY_DATA,STATE_FRAME_KEY),
		XV_SHOW, TRUE,
		NULL);
}




/* the "Change State" button's callback
 * this is where the image is changed
 *
 *	current_state toggles between 0 and 1 displaying alternate
 *	images.
 */
void change_state_icon(item,event)
Panel_item	item;
Event		*event;
{
	Icon state0 = (Icon)xv_get(item,XV_KEY_DATA,S0_ICON_KEY);

	if(current_state)
	{
		xv_set(state0,ICON_IMAGE,s0image,NULL);
		current_state=0;
	}
	else
	{
		xv_set(state0,ICON_IMAGE,s1image,NULL);
		current_state=1;
	}
}







/*================================================================
 * Time Animation callbacks
 *----------------------------------------------------------------
 */

/* The "Animate..." menu item callback
 * this function displays the animate animation frame
 *
 *	animate_state determines which frame to show.  0 sets this
 *		to the beginning of the range (see animate_state_icon()
 *		for more details)
 *	then we show the Fanime frame
 */
void anime(m,mi)
Menu m;
Menu_item mi;
{
	animate_state=0;
	xv_set((Frame)xv_get(mi,XV_KEY_DATA, ANI_FRAME_KEY),
		XV_SHOW, TRUE,
		NULL);
}

/* the workhorse routine.  This routine is responsible for deciding
 * which frame to display next.  While the counter animate_state is below
 * 500, it will display a0image except for every 50th frame where it will
 * show a8image.  This makes the cat flick it's tail.
 *
 * Then we have a whirlwind of activity between 500 and 504 where
 * the ball comes flying and smacks the cat in the head.
 *
 * From 505 to 599 the corpse twitches in a comically gruesome manner.
 *
 * From 600 to 1099 the corpse lies there dead
 * At 1100, the animate_state is reset to 0 resurrecting the little beastie
 */
void animate_state_icon(item,event)
Panel_item	item;
Event		*event;
{
	if(++animate_state < 300)
		xv_set(animate0,ICON_IMAGE,
			(animate_state%50) ? a0image :a8image,NULL);
	else if(animate_state == 300)
		xv_set(animate0,ICON_IMAGE,a1image,NULL);
	else if(animate_state == 301)
		xv_set(animate0,ICON_IMAGE,a2image,NULL);
	else if(animate_state == 302)
		xv_set(animate0,ICON_IMAGE,a3image,NULL);
	else if(animate_state == 303)
		xv_set(animate0,ICON_IMAGE,a4image,NULL);
	else if(animate_state == 304)
		xv_set(animate0,ICON_IMAGE,a5image,NULL);
	else if(animate_state > 304 && animate_state < 400)
		xv_set(animate0,ICON_IMAGE,
			(animate_state%10) ? a6image :a7image,NULL);
	else if(animate_state > 399 && animate_state < 600)
		xv_set(animate0,ICON_IMAGE, a6image, NULL);
	else
	{
		xv_set(animate0,ICON_IMAGE, a6image, NULL);
		animate_state = 0;
	}
}

/* the "Animate" button callback
 *
 *	anime_toggle reflects whether you should pause (FALSE) or
 *		go (TRUE)
 *	t.it_interval.tv_usec and t.it_value.tv_usec are used to
 *		determine the delay between frames.  Smaller numbers
 *		mean faster animation
 *	notify_set_itimer_func() actually does the animation
 *		by calling animate_state_icon()
 */
void toggle_animation(item,event)
Panel_item	item;
Event		*event;
{
	Frame frame = (Frame)xv_get(item,XV_KEY_DATA, BASE_FRAME_KEY);
	int value = slider_val;

	if(!anime_toggle)
	{
		t.it_interval.tv_usec = t.it_value.tv_usec =
			(value+40) * 1000;
		notify_set_itimer_func(frame,animate_state_icon,
			ITIMER_REAL,&t,NULL);
		anime_toggle=TRUE;
	}
	else
	{
		notify_set_itimer_func(frame,NOTIFY_FUNC_NULL,
			ITIMER_REAL, NULL, NULL);
		anime_toggle=FALSE;
	}
}

/* the slider's callback
 *
 *	slider_val is scaled then used as the frame delay.
 *		small values make the animation faster
 *		while large values slow it down
 */
void movie(item,value,event)
Panel_item item;
int value;
Event *event;
{
	slider_val=value;
}

/*--------------------------------------------------------------------*/

main(argc, argv)
int argc;
char **argv;
{
	Frame		frame;
	Frame		Fstate;
	Frame		Fanime;
	Icon		state0;
	Panel		panel;
	Panel		Pstate;			/* site panel */
	Panel		Panime;			/* site panel */
	Menu		m;

	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);
	BASE_FRAME_KEY = xv_unique_key();		/* assign keys */
	STATE_FRAME_KEY = xv_unique_key();
	ANI_FRAME_KEY = xv_unique_key();
	S0_ICON_KEY = xv_unique_key();

	s0image = (Server_image)xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,	64,
		XV_HEIGHT,	64,
		SERVER_IMAGE_BITS, state0_image,
		NULL);
	s1image = (Server_image)xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,	64,
		XV_HEIGHT,	64,
		SERVER_IMAGE_BITS, state1_image,
		NULL);
	
	a0image = (Server_image)xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,	64,
		XV_HEIGHT,	64,
		SERVER_IMAGE_BITS,  anime0_image,
		NULL);
	a1image = (Server_image)xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,	64,
		XV_HEIGHT,	64,
		SERVER_IMAGE_BITS,  anime1_image,
		NULL);
	a2image = (Server_image)xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,	64,
		XV_HEIGHT,	64,
		SERVER_IMAGE_BITS,  anime2_image,
		NULL);
	a3image = (Server_image)xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,	64,
		XV_HEIGHT,	64,
		SERVER_IMAGE_BITS,  anime3_image,
		NULL);
	a4image = (Server_image)xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,	64,
		XV_HEIGHT,	64,
		SERVER_IMAGE_BITS,  anime4_image,
		NULL);
	a5image = (Server_image)xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,	64,
		XV_HEIGHT,	64,
		SERVER_IMAGE_BITS,  anime5_image,
		NULL);
	a6image = (Server_image)xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,	64,
		XV_HEIGHT,	64,
		SERVER_IMAGE_BITS,  anime6_image,
		NULL);
	a7image = (Server_image)xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,	64,
		XV_HEIGHT,	64,
		SERVER_IMAGE_BITS,  anime7_image,
		NULL);
	a8image = (Server_image)xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,	64,
		XV_HEIGHT,	64,
		SERVER_IMAGE_BITS,  anime8_image,
		NULL);
	state0 = (Icon)xv_create(XV_NULL,ICON,
			ICON_IMAGE, s0image,
			ICON_LABEL, "state",
			NULL);
	animate0 = (Icon)xv_create(XV_NULL,ICON,
			ICON_IMAGE, a0image,
			ICON_LABEL, "time",
			NULL);
	frame = (Frame)xv_create(NULL, FRAME,
		FRAME_LABEL,	argv[0],
		XV_X,		300,
		XV_Y,		175,
		NULL);
	Fstate = (Frame)xv_create(frame, FRAME,
		FRAME_LABEL,	"State",
		FRAME_ICON,	state0,
		FRAME_CLOSED,	TRUE,
		XV_X,		350,
		XV_Y,		300,
		NULL);
	Fanime = (Frame)xv_create(frame, FRAME,
		FRAME_LABEL,	"Animate",
		FRAME_ICON,	animate0,
		FRAME_CLOSED,	TRUE,
		XV_X,		400,
		XV_Y,		300,
		NULL);
	panel = (Panel)xv_create(frame, PANEL, NULL);
	Pstate= (Panel)xv_create(Fstate, PANEL, NULL);
	Panime= (Panel)xv_create(Fanime, PANEL, NULL);

	m = (Menu)xv_create(NULL, MENU,
		MENU_ITEM,
			MENU_STRING,	"State...",
			MENU_NOTIFY_PROC, state,
			XV_KEY_DATA,	STATE_FRAME_KEY, Fstate,
			NULL,
		MENU_ITEM,
			MENU_STRING,	"Animated...",
			MENU_NOTIFY_PROC,anime,
			XV_KEY_DATA,	ANI_FRAME_KEY, Fanime,
			NULL,
		NULL);
	(void)xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"View",
		PANEL_ITEM_MENU,	m,
		NULL);
	
	(void)xv_create(panel,PANEL_BUTTON,
		PANEL_LABEL_STRING,	"Change State",
		PANEL_NOTIFY_PROC,	change_state_icon,
		XV_KEY_DATA,		S0_ICON_KEY, state0,
		NULL);
	(void)xv_create(panel,PANEL_BUTTON,
		PANEL_LABEL_STRING,	"Animate",
		PANEL_NOTIFY_PROC,	toggle_animation,
		XV_KEY_DATA,		BASE_FRAME_KEY, frame,
		NULL);
	(void)xv_create(panel,PANEL_SLIDER,
		PANEL_NEXT_ROW,		-1,
		PANEL_LABEL_STRING,	"Frame Delay:",
		PANEL_VALUE,		10,
		PANEL_MIN_VALUE,	0,
		PANEL_MAX_VALUE,	100,
		PANEL_NOTIFY_PROC,	movie,
		NULL);

	window_fit_height(panel);
	window_fit(frame);

	xv_main_loop(frame);
}

