/* frame.busy.c -- Making a base frame and it's subframes "busy"
 * Author:  Kenneth Bibb
 * May 1992 -- Arastar Consulting
 *--------------------------------------
 */

#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/canvas.h>
#include <xview/notify.h>

Attr_attribute FRAME_BASE_KEY;
void subtree_walk();



/* the following routine is from Heller/Van Raalte's XVPM */
void xv_sleep(seconds)
unsigned seconds;
{
	int	oldmask, mask;
	struct timeval tv;

	tv.tv_sec = seconds;
	tv.tv_usec = 0;
	
	mask = sigmask(SIGIO);
	mask |= sigmask(SIGALRM);
	oldmask = sigblock(mask);

	if((select(0,0,0,0,&tv)) == -1)
	{
		perror("select");
	}
	sigsetmask(oldmask);
}

int stw(base,attrib,val,i)
Frame base;
Attr_attribute attrib;
int val;
int i;
{
	Frame sub = (Frame)xv_get(base,FRAME_NTH_SUBFRAME,i);

	if(sub)
	{
		subtree_walk(sub,attrib,val);
		stw(base,attrib,val,i+1);
	}
	else
		return;
}

void subtree_walk(base,attrib,val)
Frame base;
Attr_attribute attrib;
int val;
{
	xv_set(base,attrib,val,NULL);
	stw(base,attrib,val,1);
}

void sites(button, event)
Panel_item button;
Event *event;
{
	Frame base = (Frame)xv_get(button,XV_KEY_DATA,FRAME_BASE_KEY);
	Frame sub, sub2, sub3;
	int   i;

	subtree_walk(base,XV_SHOW,TRUE);
	/* I want the following frame on "top" */
	xv_set((Frame)xv_get(base,FRAME_NTH_SUBFRAME,1),XV_SHOW,TRUE,NULL);
}

void push_me_proc(button,event)
Panel_item button;
Event *event;
{
	char s[64];
	Frame base=(Frame)xv_get(button,XV_KEY_DATA,FRAME_BASE_KEY);
	Frame sub, sub2;
	int i,j;




	xv_set(base,
		FRAME_LEFT_FOOTER,"Button: [Make Frames Busy for 10 Seconds]\n",
		NULL);


	subtree_walk(base,FRAME_BUSY,TRUE);
	xv_sleep(5);		/* wait for a sec so you can see it worked */
	subtree_walk(base,FRAME_BUSY,FALSE);


	sprintf(s,"Button: [%s]\n", (char *)xv_get(button, PANEL_LABEL_STRING));
	xv_set(base,FRAME_LEFT_FOOTER,s, NULL);
}

int main(argc, argv)
int argc;
char **argv;
{
	Frame		base;
	Frame		frame1;		/* subframes */
	Frame		frame1_1;	/* subframe of frame1 */
	Frame		frame1_1_1;	/* subframe of frame1 */
	Frame		frame1_1_2;	/* subframe of frame1 */
	Frame		frame2;
	Frame		frame3;
	Canvas		canvas;
	Panel		panel;
	Panel		p1;
	Panel		p2;
	Panel		p3;
	Panel		p1_1;
	Panel		p1_1_1;
	Panel		p1_1_2;




	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);
	FRAME_BASE_KEY = xv_unique_key();




	/* create the frames
	 */
	base = (Frame)xv_create(NULL, FRAME,
		FRAME_LABEL,		argv[0],
		FRAME_SHOW_FOOTER,	TRUE,
		XV_X,			300,
		XV_Y,			175,
		NULL);
	panel = (Panel)xv_create(base, PANEL, NULL);
	(void)xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"View Subframes...",
		PANEL_NOTIFY_PROC,	sites,
		XV_KEY_DATA,		FRAME_BASE_KEY, base,
		NULL);
	window_fit_height(panel);
	canvas = (Canvas)xv_create(base, CANVAS, 
		XV_X,			0,
		XV_WIDTH,		WIN_EXTEND_TO_EDGE,
		XV_HEIGHT,		WIN_EXTEND_TO_EDGE,
		NULL);
	window_fit(base);

	frame1 = (Frame)xv_create(base, FRAME,
		FRAME_LABEL,	"Subframe1",
		XV_X,		350,
		XV_Y,		300,
		NULL);
	frame2 = (Frame)xv_create(base, FRAME,
		FRAME_LABEL,	"Subframe2",
		XV_X,		375,
		XV_Y,		325,
		NULL);
	frame3 = (Frame)xv_create(base, FRAME,
		FRAME_LABEL,	"Subframe3",
		XV_X,		400,
		XV_Y,		350,
		NULL);
	frame1_1 = (Frame)xv_create(frame3, FRAME,
		FRAME_LABEL,	"SubSubframe 3.1",
		XV_X,		425,
		XV_Y,		375,
		NULL);
	frame1_1_1 = (Frame)xv_create(frame1_1, FRAME,
		FRAME_LABEL,	"SubSubSubframe 3.1.1",
		XV_X,		450,
		XV_Y,		400,
		NULL);
	frame1_1_2 = (Frame)xv_create(frame1_1, FRAME_CMD,
		FRAME_LABEL,	"SubSubSubframe 3.1.2",
		XV_X,		475,
		XV_Y,		425,
		NULL);




	/* panel creation stage
	 */
	p1    = (Panel)xv_create(frame1, PANEL, NULL);
	p2    = (Panel)xv_create(frame2, PANEL, NULL);
	p3    = (Panel)xv_create(frame3, PANEL, NULL);
	p1_1  = (Panel)xv_create(frame1_1, PANEL, NULL);
	p1_1_1  = (Panel)xv_create(frame1_1_1, PANEL, NULL);
	p1_1_2  = (Panel)xv_get(frame1_1_2, FRAME_CMD_PANEL);

	
	/* panel item creation stage
	 */
	(void)xv_create(p1,PANEL_BUTTON,
		PANEL_LABEL_STRING,	"Push Me",
		PANEL_NOTIFY_PROC,	push_me_proc,
		XV_KEY_DATA,		FRAME_BASE_KEY, base,
		NULL);
	(void)xv_create(p2,PANEL_MESSAGE,
		PANEL_LABEL_STRING,	"Sub Frame 2",
		NULL);
	(void)xv_create(p3,PANEL_MESSAGE,
		PANEL_LABEL_STRING,	"Sub Frame 3",
		NULL);
	(void)xv_create(p1_1,PANEL_MESSAGE,
		PANEL_LABEL_STRING,	"Sub SubFrame",
		NULL);
	(void)xv_create(p1_1_1,PANEL_MESSAGE,
		PANEL_LABEL_STRING,	"Sub Sub SubFrame",
		NULL);
	(void)xv_create(p1_1_2,PANEL_MESSAGE,
		PANEL_LABEL_STRING,	"Sub Sub SubFrame",
		NULL);

	/* make everything look nice
	 */
	window_fit(p1_1_2);
	window_fit(p1_1_1);
	window_fit(p1_1);
	window_fit(p3);
	window_fit(p2);
	window_fit(p1);
	window_fit(frame1_1_2);
	window_fit(frame1_1_1);
	window_fit(frame1_1);
	window_fit(frame3);
	window_fit(frame2);
	window_fit(frame1);

	xv_main_loop(base);
	return(0);
}

