/* notice.c -- Open Look basic notice types
 * Author:  Kenneth Bibb
 * March 1992 -- Arastar Consulting
 *-----------
 */

#include <stdlib.h>
#include <sys/param.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/openmenu.h>
#include <xview/xv_xrect.h>
#include <xview/notice.h>
#include <xview/server.h>

Attr_attribute FRAME_BASE_KEY;
Attr_attribute PANEL_BASE_KEY;
Attr_attribute SERVER_KEY;

/* this routine shows an open look compliant notice
 * complete with input queue flushing, default buttons,
 * and it doesn't block all input to the window.
 * it does NOT have the shadow showing where it
 * came from, however.  A future attribute?
 */
void show_notice(item, event)
Panel_item item;
Event *event;
{
	Xv_Server server = xv_get(item, XV_KEY_DATA, SERVER_KEY);
	Frame frame	 = xv_get(item, XV_KEY_DATA, FRAME_BASE_KEY);
	Panel panel	 = xv_get(item, XV_KEY_DATA, PANEL_BASE_KEY);

	/* flush input buffer */
	xv_set(server,
		SERVER_SYNC_AND_PROCESS_EVENTS,
		NULL);

	/* create notify popup
	 *-----------------------
	 * The weird use of buttons lets us define
	 * a default button which is not the first button.
	 * we are also able to find out which button was pressed
	 * this way.  
	 */
	(void)xv_create(panel, NOTICE,
		XV_HELP_DATA,		"help:notice",
		NOTICE_BLOCK_THREAD,	TRUE,
		NOTICE_LOCK_SCREEN,	FALSE,
		NOTICE_MESSAGE_STRINGS,	"You are about to ignite",
					"your supervisor's desk on fire",
					NULL,
		NOTICE_BUTTON_NO,	"Set Fire",
		NOTICE_BUTTON_YES,	"Wimp Out",
		NOTICE_BUTTON,	"Use Bomb", 202,
		NOTICE_BUSY_FRAMES,	frame, NULL,
		XV_SHOW,		TRUE,
		NULL);
}

/* this is a standard Xview notice.
 * it is missing some of the things suggested by the OLAG
 * but it gets the job done
 */
void show_xview(item, event)
Panel_item item;
Event *event;
{
	Panel panel = xv_get(item, XV_KEY_DATA, PANEL_BASE_KEY);

	(void)xv_create(panel, NOTICE,
		NOTICE_MESSAGE_STRINGS,	"Standard XView Notice",
					NULL,
		XV_SHOW,	TRUE,
		NULL);
}

/* this is not a useful notice.
 * just thought you might like to see what the defaults are
 */
void show_dxv(item, event)
Panel_item item;
Event *event;
{
	Panel panel = xv_get(item, XV_KEY_DATA, PANEL_BASE_KEY);

	(void)xv_create(panel, NOTICE,
		XV_SHOW,	TRUE,
		NULL);
}

/* This is a blocking notice.
 * It is not Open Look compliant, but it looks nice.
 */
void show_block(item, event)
Panel_item item;
Event *event;
{
	Panel panel = xv_get(item, XV_KEY_DATA, PANEL_BASE_KEY);

	(void)xv_create(panel, NOTICE,
		NOTICE_MESSAGE_STRINGS,	"This is not compliant",
					"Full screen blocking is bad",
					NULL,
		NOTICE_LOCK_SCREEN,	TRUE,
		XV_SHOW,		TRUE,
		NULL);
}




int main(argc, argv)
int argc;
char **argv;
{
	Frame		frame;
	Panel		panel;
	Panel_item	ph, pi, pj;

	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);
	FRAME_BASE_KEY = xv_unique_key();
	PANEL_BASE_KEY = xv_unique_key();
	SERVER_KEY = xv_unique_key();

	frame = (Frame)xv_create(NULL, FRAME,
		FRAME_LABEL,	argv[0],
		XV_X,		300,
		XV_Y,		175,
		NULL);
	panel = (Panel)xv_create(frame, PANEL,
		NULL);


	ph = xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"OL Std Notice",
		PANEL_NOTIFY_PROC,	show_notice,
		XV_KEY_DATA,		FRAME_BASE_KEY, frame,
		XV_KEY_DATA,		PANEL_BASE_KEY, panel,
		XV_KEY_DATA,		SERVER_KEY,
			xv_get(xv_get(frame,XV_SCREEN), SCREEN_SERVER),
		NULL);
	xv_set(panel,PANEL_DEFAULT_ITEM,ph);
	ph = xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"Std XView Notice",
		PANEL_NOTIFY_PROC,	show_xview,
		XV_KEY_DATA,		PANEL_BASE_KEY, panel,
		NULL);
	ph = xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"Default Std XView Notice",
		PANEL_NOTIFY_PROC,	show_dxv,
		XV_KEY_DATA,		PANEL_BASE_KEY, panel,
		NULL);
	ph = xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"Blocking Notice",
		PANEL_NOTIFY_PROC,	show_block,
		XV_KEY_DATA,		PANEL_BASE_KEY, panel,
		NULL);

	window_fit(panel);
	window_fit(frame);

	xv_main_loop(frame);
}

