/* panel.c -- Open Look panels
 * Author:  Kenneth Bibb
 * January 1992 -- Arastar Consulting
 *-----------------------------------
 * align_items(int,Panel_items ...) GROUP_LEFT, GROUP_CENTER, GROUP_RIGHT
 *				    GROUP_TOP,  GROUP_BOTTOM,
 */

/* #include <stdarg.h> */
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/canvas.h>
#include <xview/xv_xrect.h>
#include <xview/openmenu.h>
#include <xview/icon.h>
#include <xview/svrimage.h>
#include "group.h"

Attr_attribute PIX;
Attr_attribute PJX;
Attr_attribute PKX;
Attr_attribute PLX;
Attr_attribute PMX;
Attr_attribute PANEL_KEY;

int distribute_items(alignment,  va_alist)
va_dcl
{
	va_list 	 panel_items;
	Rect		*r;
	Panel_item	 pitem;
	int		 margin_a, margin_b;
	int		 n=0;	/* number of items */
	int		 span=0; /* distance between centers */
	int		 i;
	int		 width=0;

	margin_a = 100000;		/* left/top margin */
	margin_b = 0;			/* right/bottom margin */
	va_start(panel_items);

	if(alignment == PANEL_HORIZONTAL)
	{
		while(pitem = va_arg(panel_items,Panel_item))
		{
			++n;
			r = (Rect *)xv_get(pitem,XV_RECT);
			if(xv_get(pitem,XV_X) < margin_a)
				margin_a = xv_get(pitem,XV_X)
						+ (xv_get(pitem,XV_WIDTH)/2);
			if(rect_right(r) > margin_b)
				margin_b = rect_right(r)
						- (xv_get(pitem,XV_WIDTH)/2)+1;
		}
	}
	else
	{
		while(pitem = va_arg(panel_items,Panel_item))
		{
			++n;
			r = (Rect *)xv_get(pitem,XV_RECT);
			if(xv_get(pitem,XV_Y) < margin_a)
				margin_a = xv_get(pitem,XV_Y)
						+ (xv_get(pitem,XV_HEIGHT)/2);
			if(rect_bottom(r) > margin_b)
				margin_b = rect_bottom(r)
						- (xv_get(pitem,XV_HEIGHT)/2);
		}
	}
	va_end(panel_items);

	va_start(panel_items);
	if(alignment == PANEL_HORIZONTAL)
	{
		/* do first item */
		if(pitem = va_arg(panel_items,Panel_item))
		{
			span = (margin_b - margin_a)/(n-1);
			width = xv_get(pitem,XV_WIDTH)/2;
			(void)xv_set(pitem,
				XV_X, (margin_a - width),
				NULL);
		}
		else
			return(XV_ERROR);
		for(i=1; i<(n-1); i++)
		{
			pitem = va_arg(panel_items,Panel_item);
			width = xv_get(pitem,XV_WIDTH)/2;
			(void)xv_set(pitem, XV_X, margin_a + (span*i) - width,
				NULL);
		}
	}
	else
	{
		/* do first item */
		if(pitem = va_arg(panel_items,Panel_item))
		{
			printf("hi: %d\nlo: %d\n",margin_b,margin_a);
			span = (margin_b - margin_a)/(n-1);
			printf("span=%d\n",span);
			width = xv_get(pitem,XV_HEIGHT)/2;
			(void)xv_set(pitem,
				XV_Y, (margin_a - width),
				NULL);
		}
		else
			return(XV_ERROR);
		for(i=1; i<(n-1); i++)
		{
			pitem = va_arg(panel_items,Panel_item);
			width = xv_get(pitem, XV_HEIGHT)/2;
			(void)xv_set(pitem, XV_Y, margin_a + (span*i) - width,
				NULL);
		}
	}
	va_end(panel_items);
	return(XV_OK);
}

int align_items(alignment,  va_alist)
va_dcl
int alignment;
{
	va_list 	 panel_items;
	Rect		*r;
	Panel_item	 pitem;
	int		 old_margin, x;

	va_start(panel_items);
	if(alignment != GROUP_CENTER && alignment != GROUP_MIDDLE)
	{
		if(alignment == GROUP_LEFT || alignment == GROUP_TOP)
			old_margin = 100000;
		else
			old_margin = 0;
		while(pitem = va_arg(panel_items,Panel_item))
		{
			r = (Rect *)xv_get(pitem,XV_RECT);
			switch(alignment)
			{
			case GROUP_LEFT:   /* align left edge of rects */
				if(xv_get(pitem,XV_X) < old_margin)
					old_margin = xv_get(pitem,XV_X);
				break;
			case GROUP_CENTER: /* align (left+right)/2 */
				/* center on first object */
				break;
			case GROUP_RIGHT:  /* align right edge of rects */
				if(rect_right(r) > old_margin)
					old_margin = rect_right(r);
				break;
			case GROUP_TOP:	   /* align top edge of rects */
				if(xv_get(pitem,XV_Y) < old_margin)
					old_margin = xv_get(pitem,XV_Y);
				break;
			case GROUP_MIDDLE: /* align (top+bottom)/2 */
				/* center on first object */
				break;
			case GROUP_BOTTOM: /* align bottom edge of rects */
				if(rect_bottom(r) > old_margin)
					old_margin = rect_bottom(r);
				break;
			default: /* bad alignment--set XV_ERROR */
			return(XV_ERROR);
			}
		}
	}
	else
	{
		pitem = va_arg(panel_items,Panel_item);
		if(alignment==GROUP_CENTER)
		{
			old_margin = xv_get(pitem,XV_X) +
				(int)(xv_get(pitem,XV_WIDTH)/2);
		}
		else
			old_margin = xv_get(pitem,XV_Y) +
				(int)(xv_get(pitem,XV_HEIGHT)/2);
	}
	va_end(panel_items);

	va_start(panel_items);
	while(pitem = va_arg(panel_items,Panel_item))
	{
		switch(alignment)
		{
		case GROUP_LEFT:   /* align left edge of rects */
			(void)xv_set(pitem,XV_X,old_margin,NULL);
			break;
		case GROUP_CENTER: /* align (left+right)/2 */
			x = (int)xv_get(pitem,XV_X);
			(void)xv_set(pitem,XV_X,
				(old_margin-((int)xv_get(pitem,XV_WIDTH)/2)),
				NULL);
			break;
		case GROUP_RIGHT:  /* align right edge of rects */
			(void)xv_set(pitem,XV_X,
				(old_margin-xv_get(pitem,XV_WIDTH)),
				NULL);
			break;
		case GROUP_TOP:	   /* align top edge of rects */
			(void)xv_set(pitem,XV_Y, old_margin, NULL);
			break;
		case GROUP_MIDDLE: /* align (top+bottom)/2 */
			(void)xv_set(pitem,XV_Y,
				(old_margin-((int)xv_get(pitem,XV_HEIGHT/2))),
				NULL);
			break;
		case GROUP_BOTTOM: /* align bottom edge of rects */
			(void)xv_set(pitem,XV_Y, old_margin, NULL);
			break;
		default: /* bad alignment--set XV_ERROR */
			return(XV_ERROR);
		}
	}
	va_end(panel_items);
	return(XV_OK);
}

void left_proc(button, event)
Panel_item  button;
Event      *event;
{
	Panel      panel = (Panel)  xv_get(button,XV_KEY_DATA, PANEL_KEY);
	Panel_item pi = (Panel_item)xv_get(button,XV_KEY_DATA, PIX);
	Panel_item pj = (Panel_item)xv_get(button,XV_KEY_DATA, PJX);
	Panel_item pk = (Panel_item)xv_get(button,XV_KEY_DATA, PKX);
	Panel_item pl = (Panel_item)xv_get(button,XV_KEY_DATA, PLX);
	Panel_item pm = (Panel_item)xv_get(button,XV_KEY_DATA, PMX);

	align_items(GROUP_LEFT,pi,pj,pk,pl,pm,NULL);
	panel_paint(panel, PANEL_CLEAR);
}

void center_proc(button, event)
Panel_item  button;
Event	   *event;
{
	Panel      panel = (Panel)  xv_get(button,XV_KEY_DATA, PANEL_KEY);
	Panel_item pi = (Panel_item)xv_get(button,XV_KEY_DATA, PIX);
	Panel_item pj = (Panel_item)xv_get(button,XV_KEY_DATA, PJX);
	Panel_item pk = (Panel_item)xv_get(button,XV_KEY_DATA, PKX);
	Panel_item pl = (Panel_item)xv_get(button,XV_KEY_DATA, PLX);
	Panel_item pm = (Panel_item)xv_get(button,XV_KEY_DATA, PMX);

	align_items(GROUP_CENTER,pi,pj,pk,pl,pm,NULL);
	panel_paint(panel, PANEL_CLEAR);
}

void right_proc(button, event)
Panel_item  button;
Event	   *event;
{
	Panel      panel = (Panel)  xv_get(button,XV_KEY_DATA, PANEL_KEY);
	Panel_item pi = (Panel_item)xv_get(button,XV_KEY_DATA, PIX);
	Panel_item pj = (Panel_item)xv_get(button,XV_KEY_DATA, PJX);
	Panel_item pk = (Panel_item)xv_get(button,XV_KEY_DATA, PKX);
	Panel_item pl = (Panel_item)xv_get(button,XV_KEY_DATA, PLX);
	Panel_item pm = (Panel_item)xv_get(button,XV_KEY_DATA, PMX);

	align_items(GROUP_RIGHT,pi,pj,pk,pl,pm,NULL);
	panel_paint(panel, PANEL_CLEAR);
}

void top_proc(button, event)
Panel_item  button;
Event	   *event;
{
	Panel      panel = (Panel)  xv_get(button,XV_KEY_DATA, PANEL_KEY);
	Panel_item pi = (Panel_item)xv_get(button,XV_KEY_DATA, PIX);
	Panel_item pj = (Panel_item)xv_get(button,XV_KEY_DATA, PJX);
	Panel_item pk = (Panel_item)xv_get(button,XV_KEY_DATA, PKX);
	Panel_item pl = (Panel_item)xv_get(button,XV_KEY_DATA, PLX);
	Panel_item pm = (Panel_item)xv_get(button,XV_KEY_DATA, PMX);

	align_items(GROUP_TOP,pi,pj,pk,pl,pm,NULL);
	panel_paint(panel, PANEL_CLEAR);
}

void middle_proc(button, event)
Panel_item  button;
Event	   *event;
{
	Panel      panel = (Panel)  xv_get(button,XV_KEY_DATA, PANEL_KEY);
	Panel_item pi = (Panel_item)xv_get(button,XV_KEY_DATA, PIX);
	Panel_item pj = (Panel_item)xv_get(button,XV_KEY_DATA, PJX);
	Panel_item pk = (Panel_item)xv_get(button,XV_KEY_DATA, PKX);
	Panel_item pl = (Panel_item)xv_get(button,XV_KEY_DATA, PLX);
	Panel_item pm = (Panel_item)xv_get(button,XV_KEY_DATA, PMX);

	align_items(GROUP_MIDDLE,pi,pj,pk,pl,pm,NULL);
	panel_paint(panel, PANEL_CLEAR);
}

void bottom_proc(button, event)
Panel_item  button;
Event	   *event;
{
	Panel      panel = (Panel)  xv_get(button,XV_KEY_DATA, PANEL_KEY);
	Panel_item pi = (Panel_item)xv_get(button,XV_KEY_DATA, PIX);
	Panel_item pj = (Panel_item)xv_get(button,XV_KEY_DATA, PJX);
	Panel_item pk = (Panel_item)xv_get(button,XV_KEY_DATA, PKX);
	Panel_item pl = (Panel_item)xv_get(button,XV_KEY_DATA, PLX);
	Panel_item pm = (Panel_item)xv_get(button,XV_KEY_DATA, PMX);

	align_items(GROUP_BOTTOM,pi,pj,pk,pl,pm,NULL);
	panel_paint(panel, PANEL_CLEAR);
}

void horiz_proc(button, event)
Panel_item  button;
Event	   *event;
{
	Panel      panel = (Panel)  xv_get(button,XV_KEY_DATA, PANEL_KEY);
	Panel_item pi = (Panel_item)xv_get(button,XV_KEY_DATA, PIX);
	Panel_item pj = (Panel_item)xv_get(button,XV_KEY_DATA, PJX);
	Panel_item pk = (Panel_item)xv_get(button,XV_KEY_DATA, PKX);
	Panel_item pl = (Panel_item)xv_get(button,XV_KEY_DATA, PLX);
	Panel_item pm = (Panel_item)xv_get(button,XV_KEY_DATA, PMX);

	if(distribute_items(PANEL_HORIZONTAL,pi,pj,pk,pl,pm,NULL) == XV_ERROR)
		printf("horiz error!\n");
	panel_paint(panel, PANEL_CLEAR);
}

void vert_proc(button, event)
Panel_item  button;
Event	   *event;
{
	Panel      panel = (Panel)  xv_get(button,XV_KEY_DATA, PANEL_KEY);
	Panel_item pi = (Panel_item)xv_get(button,XV_KEY_DATA, PIX);
	Panel_item pj = (Panel_item)xv_get(button,XV_KEY_DATA, PJX);
	Panel_item pk = (Panel_item)xv_get(button,XV_KEY_DATA, PKX);
	Panel_item pl = (Panel_item)xv_get(button,XV_KEY_DATA, PLX);
	Panel_item pm = (Panel_item)xv_get(button,XV_KEY_DATA, PMX);

	if(distribute_items(PANEL_VERTICAL,pi,pj,pk,pl,pm,NULL) == XV_ERROR)
		printf("vert error!\n");
	panel_paint(panel, PANEL_CLEAR);
}

void display_proc(button, event)
Panel_item  button;
Event	   *event;
{
	Panel_item pi = (Panel_item)xv_get(button,XV_KEY_DATA, PIX);
	Panel_item pj = (Panel_item)xv_get(button,XV_KEY_DATA, PJX);
	Panel_item pk = (Panel_item)xv_get(button,XV_KEY_DATA, PKX);
	Panel_item pl = (Panel_item)xv_get(button,XV_KEY_DATA, PLX);
	Panel_item pm = (Panel_item)xv_get(button,XV_KEY_DATA, PMX);
	int direction;

	printf("-----------------------------\n");
	printf("pi (%d,%d)\t%d\t[%s]\n",
		xv_get(pi,XV_X), xv_get(pi,XV_Y),
		xv_get(pi,XV_X)+xv_get(pi,XV_WIDTH)/2,
		xv_get(pi,PANEL_LABEL_STRING)
		);
	printf("pj (%d,%d)\t%d\t[%s]\n",
		xv_get(pj,XV_X), xv_get(pj,XV_Y),
		xv_get(pj,XV_X)+xv_get(pj,XV_WIDTH)/2,
		xv_get(pj,PANEL_LABEL_STRING)
		);
	printf("pk (%d,%d)\t%d\t[%s]\n",
		xv_get(pk,XV_X), xv_get(pk,XV_Y),
		xv_get(pk,XV_X)+xv_get(pk,XV_WIDTH)/2,
		xv_get(pk,PANEL_LABEL_STRING)
		);
	printf("pl (%d,%d)\t%d\t[%s]\n",
		xv_get(pl,XV_X), xv_get(pl,XV_Y),
		xv_get(pl,XV_X)+xv_get(pl,XV_WIDTH)/2,
		xv_get(pl,PANEL_LABEL_STRING)
		);
	printf("pm (%d,%d)\t%d\t[%s]\n",
		xv_get(pm,XV_X), xv_get(pm,XV_Y),
		xv_get(pm,XV_X)+xv_get(pm,XV_WIDTH)/2,
		xv_get(pm,PANEL_LABEL_STRING)
		);
}

void reset_proc(button, event)
Panel_item  button;
Event	   *event;
{
	Panel      panel = (Panel)  xv_get(button,XV_KEY_DATA, PANEL_KEY);
	Panel_item pi = (Panel_item)xv_get(button,XV_KEY_DATA, PIX);
	Panel_item pj = (Panel_item)xv_get(button,XV_KEY_DATA, PJX);
	Panel_item pk = (Panel_item)xv_get(button,XV_KEY_DATA, PKX);
	Panel_item pl = (Panel_item)xv_get(button,XV_KEY_DATA, PLX);
	Panel_item pm = (Panel_item)xv_get(button,XV_KEY_DATA, PMX);

	(void)xv_set(pi, XV_X,  50, XV_Y, 200, NULL);
	(void)xv_set(pj, XV_X, 150, XV_Y, 250, NULL);
	(void)xv_set(pk, XV_X, 250, XV_Y, 300, NULL);
	(void)xv_set(pl, XV_X, 350, XV_Y, 220, NULL);
	(void)xv_set(pm, XV_X, 450, XV_Y, 350, NULL);
	panel_paint(panel,PANEL_CLEAR);
}

int main(argc, argv)
int argc;
char **argv;
{
	Frame		frame;
	Canvas		canvas;
	Panel		panel;
	Panel		p2;
	Panel_item	pi, pj, pk, pl, pm;
	Panel_item	bi, bj, bk, bl, bm, bn, bo, bp, bq, br;

	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);
	PIX = xv_unique_key();
	PJX = xv_unique_key();
	PKX = xv_unique_key();
	PLX = xv_unique_key();
	PMX = xv_unique_key();
	PANEL_KEY = xv_unique_key();
	frame = (Frame)xv_create(NULL, FRAME,
		FRAME_LABEL,	argv[0],
		XV_X,		200,
		XV_Y,		175,
		XV_WIDTH,	600,
		NULL);
	panel = (Panel)xv_create(frame, PANEL,
			PANEL_LAYOUT, PANEL_HORIZONTAL,
			NULL);
	bi = (Panel_item)xv_create(panel, PANEL_BUTTON,
			PANEL_LABEL_STRING,	"Top",
			PANEL_NOTIFY_PROC,	top_proc,
			NULL);
	bp = (Panel_item)xv_create(panel, PANEL_BUTTON,
			PANEL_LABEL_STRING,	"Horizontal",
			PANEL_NOTIFY_PROC,	horiz_proc,
			NULL);
	bj = (Panel_item)xv_create(panel, PANEL_BUTTON,
			PANEL_NEXT_ROW,		-1,
			PANEL_LABEL_STRING,	"Left",
			PANEL_NOTIFY_PROC,	left_proc,
			NULL);
	bk = (Panel_item)xv_create(panel, PANEL_BUTTON,
			PANEL_LABEL_STRING,	"Center",
			PANEL_NOTIFY_PROC,	center_proc,
			NULL);
	bl = (Panel_item)xv_create(panel, PANEL_BUTTON,
			PANEL_LABEL_STRING,	"Right",
			PANEL_NOTIFY_PROC,	right_proc,
			NULL);
	bm = (Panel_item)xv_create(panel, PANEL_BUTTON,
			XV_X,			xv_get(panel,PANEL_ITEM_X)+
						xv_get(bl,XV_WIDTH)+50,
			PANEL_LABEL_STRING,	"Middle",
			PANEL_NOTIFY_PROC,	middle_proc,
			NULL);
	bq = (Panel_item)xv_create(panel, PANEL_BUTTON,
			XV_X,			xv_get(panel,PANEL_ITEM_X)+
						xv_get(bm,XV_WIDTH)+50,
			PANEL_LABEL_STRING,	"Vertical",
			PANEL_NOTIFY_PROC,	vert_proc,
			NULL);
	(void)xv_set(bi, XV_X, xv_get(bm, XV_X),NULL);
	(void)xv_set(bp, XV_X, xv_get(bq, XV_X),NULL);
	bn = (Panel_item)xv_create(panel, PANEL_BUTTON,
			PANEL_NEXT_ROW,		-1,
			XV_X,			xv_get(bm, XV_X),
			PANEL_LABEL_STRING,	"Bottom",
			PANEL_NOTIFY_PROC,	bottom_proc,
			NULL);

	align_items(GROUP_CENTER,bi,bm,bn,NULL);

	bo = (Panel_item)xv_create(panel, PANEL_BUTTON,
			PANEL_NEXT_ROW,		-1,
			PANEL_LABEL_STRING,	"Reset",
			PANEL_NOTIFY_PROC,	reset_proc,
			NULL);
	br = (Panel_item)xv_create(panel, PANEL_BUTTON,
			PANEL_LABEL_STRING,	"Display",
			PANEL_NOTIFY_PROC,	display_proc,
			NULL);
	(void)xv_create(panel, PANEL_MESSAGE,
			PANEL_NEXT_ROW,		-1,
			PANEL_LABEL_STRING,	"____________________________",
			NULL);
	pi = (Panel_item)xv_create(panel, PANEL_BUTTON,
			PANEL_LABEL_STRING,	"T r i t o n",
			XV_X,		50,
			XV_Y,		200,
			NULL);
	pj = (Panel_item)xv_create(panel, PANEL_BUTTON,
			PANEL_LABEL_STRING,	"Merman",
			XV_X,		150,
			XV_Y,		250,
			NULL);
	pk = (Panel_item)xv_create(panel, PANEL_BUTTON,
			PANEL_LABEL_STRING,	"Naiad",
			XV_X,		250,
			XV_Y,		300,
			NULL);
	pl = (Panel_item)xv_create(panel, PANEL_BUTTON,
			PANEL_LABEL_STRING,	"Fishman",
			XV_X,		350,
			XV_Y,		220,
			NULL);
	pm = (Panel_item)xv_create(panel, PANEL_BUTTON,
			PANEL_LABEL_STRING,	"li",
			XV_X,		450,
			XV_Y,		350,
			NULL);
	(void)xv_set(bi,
		XV_KEY_DATA, PIX,	pi,
		XV_KEY_DATA, PJX,	pj,
		XV_KEY_DATA, PKX,	pk,
		XV_KEY_DATA, PLX,	pl,
		XV_KEY_DATA, PMX,	pm,
		XV_KEY_DATA, PANEL_KEY, panel,
		NULL);
	(void)xv_set(bj,
		XV_KEY_DATA, PIX,	pi,
		XV_KEY_DATA, PJX,	pj,
		XV_KEY_DATA, PKX,	pk,
		XV_KEY_DATA, PLX,	pl,
		XV_KEY_DATA, PMX,	pm,
		XV_KEY_DATA, PANEL_KEY, panel,
		NULL);
	(void)xv_set(bk,
		XV_KEY_DATA, PIX,	pi,
		XV_KEY_DATA, PJX,	pj,
		XV_KEY_DATA, PKX,	pk,
		XV_KEY_DATA, PLX,	pl,
		XV_KEY_DATA, PMX,	pm,
		XV_KEY_DATA, PANEL_KEY, panel,
		NULL);
	(void)xv_set(bl,
		XV_KEY_DATA, PIX,	pi,
		XV_KEY_DATA, PJX,	pj,
		XV_KEY_DATA, PKX,	pk,
		XV_KEY_DATA, PLX,	pl,
		XV_KEY_DATA, PMX,	pm,
		XV_KEY_DATA, PANEL_KEY, panel,
		NULL);
	(void)xv_set(bm,
		XV_KEY_DATA, PIX,	pi,
		XV_KEY_DATA, PJX,	pj,
		XV_KEY_DATA, PKX,	pk,
		XV_KEY_DATA, PLX,	pl,
		XV_KEY_DATA, PMX,	pm,
		XV_KEY_DATA, PANEL_KEY, panel,
		NULL);
	(void)xv_set(bn,
		XV_KEY_DATA, PIX,	pi,
		XV_KEY_DATA, PJX,	pj,
		XV_KEY_DATA, PKX,	pk,
		XV_KEY_DATA, PLX,	pl,
		XV_KEY_DATA, PMX,	pm,
		XV_KEY_DATA, PANEL_KEY, panel,
		NULL);
	(void)xv_set(bo,
		XV_KEY_DATA, PIX,	pi,
		XV_KEY_DATA, PJX,	pj,
		XV_KEY_DATA, PKX,	pk,
		XV_KEY_DATA, PLX,	pl,
		XV_KEY_DATA, PMX,	pm,
		XV_KEY_DATA, PANEL_KEY, panel,
		NULL);
	(void)xv_set(bp,
		XV_KEY_DATA, PIX,	pi,
		XV_KEY_DATA, PJX,	pj,
		XV_KEY_DATA, PKX,	pk,
		XV_KEY_DATA, PLX,	pl,
		XV_KEY_DATA, PMX,	pm,
		XV_KEY_DATA, PANEL_KEY, panel,
		NULL);
	(void)xv_set(bq,
		XV_KEY_DATA, PIX,	pi,
		XV_KEY_DATA, PJX,	pj,
		XV_KEY_DATA, PKX,	pk,
		XV_KEY_DATA, PLX,	pl,
		XV_KEY_DATA, PMX,	pm,
		XV_KEY_DATA, PANEL_KEY, panel,
		NULL);
	(void)xv_set(br,
		XV_KEY_DATA, PIX,	pi,
		XV_KEY_DATA, PJX,	pj,
		XV_KEY_DATA, PKX,	pk,
		XV_KEY_DATA, PLX,	pl,
		XV_KEY_DATA, PMX,	pm,
		XV_KEY_DATA, PANEL_KEY, panel,
		NULL);
	window_fit(frame);

	xv_main_loop(frame);
}

