/* text.c -- demonstration of text panel items
* Author: Kenneth Bibb
*---------------------------------------------
* demonstrate PANEL_NONE, PANEL_NONPRINTABLE, PANEL_ALL, PANEL_SPECIFIED
* for separate items.
*
* by field check: state abbs
* by char  check: province abbs
* using panel_none (only checked with "validate"): integer
*/

#include <stdio.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/notify.h>

char *valid_states[] = {
#include "states.h"
};

/* valid province codes according to the Canada Postal Code Directory:
 *	AB	Alberta
 *	BC	British Columbia
 *	MB	Manitoba
 *	NB	New Brunswick
 *	NF	Newfoundland
 *	NS	Nova Scotia
 *	NT	Northwest Territories
 *	ON	Ontario
 *	PE	Prince Edward Island
 *	PQ	Quebec
 *	SK	Saskatchewan
 *	YT	Yukon Territory
 */

static Notify_value 	base_item_event_handler();
static Notify_value 	decimal_numbers_only();
static Notify_value 	no_periods();
static Notify_value 	no_lower_case();
static void		post_panel_item_events();
static void		register_panel_items();


Attr_attribute FRAME_BASE_KEY;
Attr_attribute TEXT_STATE_KEY;
Attr_attribute TEXT_PROVINCE_KEY;
Attr_attribute TEXT_INTGR_KEY;
Attr_attribute EVENT_PROC_KEY;


#define NUM_STATES	50		/* shouldn't change any time soon */

#define isdot(chr)	((chr) == '.')


Panel_setting state_check(text,event)
Panel_item text;
Event *event;
{
	Frame base = (Frame)xv_get(text, XV_KEY_DATA, FRAME_BASE_KEY);
	char *value = (char *)xv_get(text, PANEL_VALUE);
	int   i, found=FALSE;

	for(i=0; i<NUM_STATES; i++)
	{
		if(strcmp(value,valid_states[i]) == 0)
		{
			found = TRUE;
			break;
		}
	}
	if(found)
	{
		(void)xv_set(base,
			FRAME_LEFT_FOOTER,	" ",
			NULL);
		return(PANEL_NEXT);
	}
	else
	{
		(void)xv_set(base,
			FRAME_LEFT_FOOTER,	"Not a valid state",
			NULL);
		window_bell((Panel)xv_get((Panel_item)text,XV_OWNER));
		return(PANEL_NONE);
	}
}

Panel_setting province_check(c,text)
int   c;
Panel_item text;
{
	char *string = (char *)xv_get(text, PANEL_VALUE);

	if(string[0] == NULL)
	{
		switch(c)
		{
		case 'A':
		case 'B':
		case 'M':
		case 'N':
		case 'O':
		case 'P':
		case 'S':
		case 'Y': return(PANEL_INSERT);
		default:
			window_bell((Panel)xv_get((Panel_item)text,XV_OWNER));
			return(PANEL_NONE);
		}
	}
	else if(string[1] == NULL)
	{
		switch(string[0])
		{
		case 'A':
			if(c == 'B')
				return(PANEL_INSERT);
			break;
		case 'B':
			if(c == 'C')
				return(PANEL_INSERT);
			break;
		case 'M':
			if(c == 'B')
				return(PANEL_INSERT);
			break;
		case 'N':
			switch(c)
			{
			case 'B':
			case 'F':
			case 'S':
			case 'T':
				return(PANEL_INSERT);
			default:	break;
			}
		case 'O':
			if(c == 'N')
				return(PANEL_INSERT);
			break;
		case 'P':
			if(c == 'E' || c == 'Q')
				return(PANEL_INSERT);
			break;
		case 'S':
			if(c == 'K')
				return(PANEL_INSERT);
			break;
		case 'Y':
			if(c == 'T')
				return(PANEL_INSERT);
			break;
		}
		window_bell((Panel)xv_get((Panel_item)text,XV_OWNER));
		return(PANEL_NONE);
	}
	return(PANEL_INSERT);
}

Panel_setting province_notify(text,event)
Panel_item text;
Event *event;
{
	char *string = (char *)xv_get(text, PANEL_VALUE);
	int   c = event_action((Event *)event);
	extern Panel_setting panel_text_notify();

	if(string[0] == NULL)
		return(province_check(c,text));
	else if(!isprint(c))
	{
		switch(c)
		{
		case '\n':
		case '\t':
		case '\r':	return(PANEL_NEXT);
		default:	return(panel_text_notify(text,event));
		}
	}
	else
	{
		if(string[1] == NULL)
			return(province_check(c,text));
		else
		{
			window_bell((Panel)xv_get((Panel_item)text,XV_OWNER));
			return(PANEL_NONE);
		}
	}
}

Panel_setting integer_check(text, event)
Panel_item text;
Event *event;
{
	Frame base = (Frame)xv_get(text, XV_KEY_DATA, FRAME_BASE_KEY);
	char *string = (char *)xv_get(text, PANEL_VALUE);
	int   len = strlen(string);
	int   i,found=FALSE;

	for(i=0; i<len; i++)
		if(!isdigit(string[i]))
			found = TRUE;
	if(found)
	{
		(void)xv_set(base,
			FRAME_LEFT_FOOTER, "Only decimal digits are allowed",
			NULL);
		window_bell((Panel)xv_get((Panel_item)text,XV_OWNER));
		return(PANEL_NONE);
	}
	else
	{
		(void)xv_set(base,
			FRAME_LEFT_FOOTER, " ",
			NULL);
		return(PANEL_NEXT);
	}
}

void bval(button, event)
Panel_item button;
Event *event;
{
	Frame          base=xv_get(button,XV_KEY_DATA,FRAME_BASE_KEY);
	Panel_item    state=xv_get(button,XV_KEY_DATA,TEXT_STATE_KEY);
	Panel_item province=xv_get(button,XV_KEY_DATA,TEXT_PROVINCE_KEY);
	Panel_item    intgr=xv_get(button,XV_KEY_DATA,TEXT_INTGR_KEY);
	char *province_string = (char *)xv_get(province, PANEL_VALUE);
	char  sbuf[80];
	char *s=sbuf;

	if((state_check(state,event) == PANEL_NONE) ||
	   (province_notify(province,event) == NULL) ||
	   (integer_check(intgr,event) == PANEL_NONE))
		return;

	sprintf(s,"State: %s Province: %s",
		(char *)xv_get(state, PANEL_VALUE),
		province_string);
	(void)xv_set(base,
		FRAME_LEFT_FOOTER, s,
		NULL);
}

int main(argc,argv)
int argc;
char **argv;
{
	Frame base;
	Panel panel;
	Panel_text_item state;
	Panel_text_item province;
	Panel_text_item decint;


	(void)xv_init( XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL );
	EVENT_PROC_KEY = xv_unique_key();
	FRAME_BASE_KEY = xv_unique_key();
	TEXT_STATE_KEY = xv_unique_key();
	TEXT_PROVINCE_KEY = xv_unique_key();
	TEXT_INTGR_KEY = xv_unique_key();

	base = (Frame)xv_create(XV_NULL, FRAME,
		XV_LABEL,	argv[0],
		FRAME_SHOW_FOOTER,	TRUE,
		NULL );

	panel = (Panel_item)xv_create(base, PANEL, 
		PANEL_LAYOUT,	PANEL_VERTICAL,
		NULL );

	state = (Panel_item)xv_create(panel, PANEL_TEXT, 
		PANEL_LABEL_STRING,		"U.S. State:",
		PANEL_VALUE_DISPLAY_LENGTH,	2,
		PANEL_VALUE_STORED_LENGTH,	2,
		PANEL_NOTIFY_PROC,		state_check,
		XV_KEY_DATA,			FRAME_BASE_KEY, base,
		NULL);

	province = (Panel_item)xv_create(panel, PANEL_TEXT, 
		PANEL_LABEL_STRING,		"Canadian Province:",
		PANEL_VALUE_DISPLAY_LENGTH,	2,
		PANEL_VALUE_STORED_LENGTH,	2,
		PANEL_NOTIFY_LEVEL,		PANEL_ALL,
		WIN_IGNORE_EVENTS,		WIN_UP_EVENTS, NULL,
		PANEL_NOTIFY_PROC,		province_notify,
		XV_KEY_DATA,			FRAME_BASE_KEY, base,
		NULL );
	
	decint = (Panel_item)xv_create(panel, PANEL_TEXT,
		PANEL_LABEL_STRING,	"Decimal Integer:",
		PANEL_VALUE_DISPLAY_LENGTH,	8,
		PANEL_VALUE_STORED_LENGTH,	8,
		XV_KEY_DATA,		FRAME_BASE_KEY, base,
		PANEL_NOTIFY_LEVEL,		PANEL_NONE,
		PANEL_NOTIFY_PROC,		integer_check,
		NULL);
	
	(void)xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"Validate",
		PANEL_NOTIFY_PROC,	bval,
		XV_KEY_DATA,		FRAME_BASE_KEY, base,
		XV_KEY_DATA,		TEXT_STATE_KEY, state,
		XV_KEY_DATA,		TEXT_PROVINCE_KEY, province,
		XV_KEY_DATA,		TEXT_INTGR_KEY, decint,
		NULL);

	register_panel_items(panel);
	notify_interpose_event_func(state,no_lower_case,NOTIFY_SAFE);
	notify_interpose_event_func(province,no_lower_case,NOTIFY_SAFE);

	window_fit ( panel );
	window_fit ( base );
	xv_main_loop ( base );
	return( 0 );
}


/****************************************************************/
/* These routines are generic to the panel items.		*/
/****************************************************************/


/*
* Traverse each item in the given Panel and register our base event handler
* for it with the Notifier.  also, replace the default event handler with
* one that posts events via the Notifier and attch the old one onto the
* item with key-data.  Note:  once this is done, our Panel_item's are
* functionally no different than when we started, except that they can
* now be interposed on.
*/
static void register_panel_items(panel)
Panel panel;
{
	Panel_item item;
	void (*items_event_proc)();

	PANEL_EACH_ITEM(panel, item)
	items_event_proc = (void(*)())xv_get(item,PANEL_EVENT_PROC);
	xv_set(item, 
		PANEL_EVENT_PROC,	post_panel_item_events,
		XV_KEY_DATA,		EVENT_PROC_KEY, items_event_proc, 
		NULL );
	notify_set_event_func(item,base_item_event_handler,NOTIFY_SAFE);
	PANEL_END_EACH
}

static void
post_panel_item_events(item,event)
Panel_item item;
Event *event;
{
	notify_post_event((Notify_client)item,(Notify_event)event,NOTIFY_SAFE);
}



/*
* The generic base handler for panel items.  this makes
* a direct call to the items default event handler, just
* as the Panel would normally do.  Note:  a base handler
* is always the LAST handler to see the event.
*/
static Notify_value base_item_event_handler(client, event, arg, type)
Notify_client client;
Notify_event event;
Notify_arg arg;
Notify_event_type type; 
{
	void (*items_event_proc)() = (void (*)())xv_get((Panel_item)client,
		XV_KEY_DATA, EVENT_PROC_KEY);

	if(items_event_proc)
		(*items_event_proc)((Panel_item)client,(Event *)event);
	return NOTIFY_DONE;
}

/*
* Force all characters to be upper case
*/
static Notify_value 
no_lower_case ( client, event, arg, type )
Notify_client client;
Notify_event event;
Notify_arg arg;
Notify_event_type type; 
{
	int id = event_action((Event *)event);

	if(!(isalpha(id) && isupper(id)))
		event_set_action((Event *)event,toupper(id));
	return(notify_next_event_func(client,event,arg,type));
}

