/* slider.c -- slider demo
 * Author:  Kenneth Bibb
 * January 1992 -- Arastar Consulting
 *--------------------------------------
 * o Change the slider's layout by pressing the "Flip Direction" button
 * o Toggle attributes by (un)commenting them and compiling
 *
 */

#include <xview/xview.h>
#include <xview/panel.h>


Attr_attribute SLIDER_ITEM_KEY;		/* slider handle */
Attr_attribute FRAME_KEY;               /* base frame handle */


/* callback for the "Flip Direction" button
 */
void button_flip_proc(item, event)
Panel_item item;
Event *event;
{
	/* retrieve the handle of the slider which we're going
	 * to manipulate in the following xv_set()
	 */
	Panel_item sl = (Panel_item)xv_get(item,XV_KEY_DATA,
					SLIDER_ITEM_KEY);

	/* retrieve the current PANEL_DIRECTION with the xv_get()
	 * then toggle it
	 */
	xv_set(sl, PANEL_DIRECTION,
			((xv_get(sl,PANEL_DIRECTION)==PANEL_VERTICAL)
				? PANEL_HORIZONTAL
				: PANEL_VERTICAL),
		NULL);
}

/* callback for the slider -- written by Rick Goldstein
 */
void slider_proc(item, val, event)
Panel_item item;
int val;
Event *event;
{
	char buf[32];
	Frame frame = (Frame)xv_get(item, XV_KEY_DATA, FRAME_KEY, NULL);

	sprintf ( buf, "You stopped on %d", val);
	(void) xv_set (frame, FRAME_LEFT_FOOTER, buf, NULL);
}

main(argc, argv)
int argc;
char **argv;
{
	Frame		frame;		/* base frame */
	Panel		panel;		/* base panel */
	Panel_item	sl;		/* slider handle */



	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);
	SLIDER_ITEM_KEY = xv_unique_key();
	FRAME_KEY = xv_unique_key();


	frame = (Frame)xv_create(NULL, FRAME,
		FRAME_LABEL,			argv[0],
		FRAME_SHOW_RESIZE_CORNER,	FALSE,
		FRAME_SHOW_FOOTER,		TRUE,
		XV_X,				300,
		XV_Y,				175,
		NULL);
	panel = (Panel)xv_create(frame, PANEL, NULL);


	/* create the slider
	 * many of these attributes use defaults
	 */
	sl = xv_create(panel,PANEL_SLIDER,
		PANEL_LABEL_STRING,	"Test Slider:",
		PANEL_DIRECTION,	PANEL_HORIZONTAL,
		PANEL_VALUE,		0,		/* default value     */
		PANEL_MIN_VALUE,	-100,		/* low end of range  */
#ifdef XVIEW_VERSION
		PANEL_MIN_TICK_STRING,	"-100",		/* XView 3.0 */
		PANEL_MIN_VALUE_STRING,	"min",		/* XView 3.0 */
#endif
		PANEL_MAX_VALUE,	100,		/* high end of range */
#ifdef XVIEW_VERSION
/*		PANEL_MAX_VALUE_STRING,	"max",		/* XView 3.0 */
		PANEL_MAX_TICK_STRING,	"100",		/* XView 3.0 */
#endif
		PANEL_SLIDER_END_BOXES,	TRUE,		/* show boxes         */
		PANEL_SLIDER_WIDTH,	300,		/* width in pixels    */
		PANEL_TICKS,		5,		/* number of ticks    */
		PANEL_SHOW_VALUE,	TRUE,		/* text field         */
		PANEL_SHOW_RANGE,	TRUE,		/* show endpoint vals */
		PANEL_NOTIFY_PROC,	slider_proc,
		PANEL_NOTIFY_LEVEL,	PANEL_DONE,	/* only on stops      */
		XV_KEY_DATA,		FRAME_KEY,	frame,
		NULL);
	/* create the button
	 * the XV_Y is used to force the button onto the same row
	 * as the slider
	 */
	(void)xv_create(panel,PANEL_BUTTON,
		XV_Y,			xv_get(sl,XV_Y),
		PANEL_LABEL_STRING,	"Flip Direction",
		XV_KEY_DATA,		SLIDER_ITEM_KEY,	sl,
		PANEL_NOTIFY_PROC,	button_flip_proc,
		NULL);

	xv_main_loop(frame);
}

