/*
 * alertwin.c - Notify and event callback function stubs.
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <X11/Xutil.h>
#include "alertwin_ui.h"
#include "msgboard.h"
#include "msgboard_xv.h"

/*
 *  Puts up an alert popup announcing that "id" logged in.
 */
void
watch_alert(id)
char *id;
{
#ifdef SCALABLE
    int xpos, ypos;
#endif
    alertwin_frame_objects	*ip;
    Panel_item			glyph;
    char buf[50];
    Window xid;
    XSizeHints *hints;

    ip = alertwin_frame_objects_initialize(NULL, base_frame);

    /*
     *  Create glyph manually so we can re-use the program's icon
     *  server image.
     */
    glyph = (Panel_item)xv_create(ip->panel, PANEL_MESSAGE,
	XV_KEY_DATA, INSTANCE, ip,
	XV_X, 16,
	XV_Y, 16,
	XV_WIDTH, 64,
	XV_HEIGHT, 64,
	PANEL_LABEL_IMAGE, (Server_image)xv_get(
				(Icon)xv_get(base_frame, FRAME_ICON),
				ICON_IMAGE),
	NULL);

    sprintf(buf, "%s checked in.", get_fname(id));
    xv_set(ip->text, PANEL_LABEL_STRING, buf, NULL);

#ifdef SCALABLE
    xv_set(glyph, XV_X, wgrid, XV_Y, wgrid, NULL);
    xpos = wgrid * 2 + (int)xv_get(glyph, XV_WIDTH);
    ypos = wgrid + (int)xv_get(glyph, XV_HEIGHT) / 2 -
	(int)xv_get(ip->text, XV_HEIGHT) / 2;
    xv_set(ip->text, XV_X, xpos, XV_Y, ypos, NULL);
    window_fit(ip->panel);
    window_fit(ip->frame);
#endif


    xid = (Window)xv_get(ip->frame, XV_XID);
    hints = XAllocSizeHints();

	/* This particular window doesn't allow resizing */
/*    hints->flags = PMinSize | PMaxSize;
    hints->min_height = hints->max_height = (int)xv_get(ip->frame, XV_HEIGHT);
    hints->min_width = hints->max_width = (int)xv_get(ip->frame, XV_WIDTH);
 */
    hints->flags = PBaseSize;
    hints->base_height = (int)xv_get(ip->frame, XV_HEIGHT);
    hints->base_width = (int)xv_get(ip->frame, XV_WIDTH);
    XSetWMNormalHints(dpy, xid, hints);
    XFree(hints);
    xv_set(ip->frame, XV_SHOW, TRUE, NULL);
    window_bell(ip->frame);
}

/*
 * Done callback function for `frame'.
 */
void
alertwin_done(frame)
Frame frame;
{
    xv_set(frame, XV_SHOW, FALSE, 0);

    free( (char *)xv_get(frame, XV_KEY_DATA, INSTANCE) );
    if ( xv_destroy_safe(frame) != XV_OK )
	pr_msg(NULL, TRUE, "Destroy of popup failed!");
}

