/*
 * amwin.c - Support routines for Message Board add/modify window
 */

#include <stdio.h>
#include <string.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include "amwin_ui.h"
#include "msgboard.h"
#include "msgboard_xv.h"
#include <pwd.h>

amwin_am_frame_objects		*am_ip;
amwin_names_frame_objects	*nameswin_p;

int
    am_window_created = FALSE,		/* is A/M window created?	*/
    names_window_created = FALSE,	/* is name chooser window created? */
    am_mode = IDLE;			/* What's the A/M window being
					   used for?			*/

void	names_panel_event();

void
create_am_window()
{
#ifdef SCALABLE
    int valx, ypos;
#endif

    am_ip = amwin_am_frame_objects_initialize(NULL, base_frame);

    xv_set(am_ip->pagernum_item, PANEL_NOTIFY_LEVEL, PANEL_ALL, NULL);

#ifdef SCALABLE
    /*
     *  sortby has the longest label, so we position it properly from the
     *  left edge of the panel.
     */
    xv_set(am_ip->sortby_item, XV_X, wgrid * 2, NULL);
    valx = (int)xv_get(am_ip->sortby_item, PANEL_VALUE_X);

    ypos = wgrid * 2;
    align_values(am_ip->logid_item, valx, ypos);
    xv_set(am_ip->names_item,
	XV_X,	(int)xv_get(am_ip->logid_item, XV_X) +
		(int)xv_get(am_ip->logid_item, XV_WIDTH) + wgrid,
	XV_Y,	ypos,
	NULL);

    ypos += wgrid + (int)xv_get(am_ip->logid_item, XV_HEIGHT);
    align_values(am_ip->fname_item, valx, ypos);

    ypos += wgrid + (int)xv_get(am_ip->fname_item, XV_HEIGHT);
    align_values(am_ip->status_item, valx, ypos);

    ypos += wgrid + (int)xv_get(am_ip->status_item, XV_HEIGHT);
    align_values(am_ip->until_item, valx, ypos);

    ypos += wgrid + (int)xv_get(am_ip->until_item, XV_HEIGHT);
    align_values(am_ip->comment_item, valx, ypos);

    ypos += wgrid * 2 + (int)xv_get(am_ip->comment_item, XV_HEIGHT);
    align_values(am_ip->cat_item, valx, ypos);

    ypos += wgrid + (int)xv_get(am_ip->cat_item, XV_HEIGHT);
    xv_set(am_ip->sortby_item, XV_Y, ypos, NULL);

    ypos += wgrid + (int)xv_get(am_ip->sortby_item, XV_HEIGHT);
    align_values(am_ip->ext_item, valx, ypos);

    ypos += wgrid + (int)xv_get(am_ip->ext_item, XV_HEIGHT);
    align_values(am_ip->pagernum_item, valx, ypos);
    xv_set(am_ip->pagertype_item,
	XV_X,	(int)xv_get(am_ip->pagernum_item, XV_X) +
		(int)xv_get(am_ip->pagernum_item, XV_WIDTH) + wgrid,
	XV_Y,	ypos,
	NULL);

    ypos += wgrid * 2 + (int)xv_get(am_ip->pagertype_item, XV_HEIGHT);
    align_values(am_ip->plan_item, valx, ypos);

    ypos += wgrid / 2 + (int)xv_get(am_ip->plan_item, XV_HEIGHT);
    align_values(am_ip->preserve_item, valx, ypos);

    ypos += wgrid / 2 + (int)xv_get(am_ip->preserve_item, XV_HEIGHT);
    align_values(am_ip->via_mail_item, valx, ypos);

    ypos += wgrid / 2 + (int)xv_get(am_ip->via_mail_item, XV_HEIGHT);
    align_values(am_ip->via_mb_item, valx, ypos);

    ypos += wgrid / 2 + (int)xv_get(am_ip->via_mb_item, XV_HEIGHT);
    align_values(am_ip->admin_item, valx, ypos);

    ypos += wgrid * 2 + (int)xv_get(am_ip->admin_item, XV_HEIGHT);

    xv_set(am_ip->apply_item, XV_Y, ypos, NULL);
    xv_set(am_ip->reset_item, XV_Y, ypos, NULL);

    window_fit(am_ip->am_panel);    
    window_fit(am_ip->am_frame);

    position_bot_buttons(am_ip->am_panel, am_ip->apply_item, am_ip->reset_item);
#endif

    am_window_created = TRUE;
}

/*
 *  Prepare A/M window for Add mode
 */
void
prep_add_mode()
{
/*
 *@  Check to see if we're not already doing something with the window!
 */
   if (!am_window_created) create_am_window();

    xv_set(am_ip->logid_item,
	PANEL_VALUE,		"",
	PANEL_READ_ONLY,	FALSE,
	NULL);
    xv_set(am_ip->names_item, XV_SHOW, TRUE, NULL);
    xv_set(am_ip->fname_item, PANEL_VALUE, "", NULL);
    xv_set(am_ip->status_item, PANEL_VALUE, MB_UNK, NULL);
    xv_set(am_ip->until_item, PANEL_VALUE, "", NULL);
    xv_set(am_ip->comment_item, PANEL_VALUE, "", NULL);
    xv_set(am_ip->cat_item, PANEL_VALUE, "", XV_SHOW, TRUE, NULL);
    xv_set(am_ip->sortby_item,
	PANEL_VALUE,	"",
	XV_SHOW,	TRUE,
	NULL);
    xv_set(am_ip->ext_item, PANEL_VALUE, "", XV_SHOW, TRUE, NULL);
    xv_set(am_ip->pagernum_item, PANEL_VALUE, "", XV_SHOW, TRUE, NULL);
    xv_set(am_ip->pagertype_item,
	PANEL_VALUE, 	globrec.pagertype,
	XV_SHOW,	TRUE,
	PANEL_INACTIVE, TRUE,
	NULL);
    xv_set(am_ip->plan_item,
	PANEL_VALUE,	(globrec.flags & MB_G_CHG_PLAN) != 0,
	NULL);
    xv_set(am_ip->via_mail_item,
	PANEL_VALUE,	(globrec.flags & MB_G_VIA_MAIL) != 0,
	NULL);
    xv_set(am_ip->via_mb_item,
	PANEL_VALUE,	(globrec.flags & MB_G_VIA_MB) != 0,
	NULL);
    xv_set(am_ip->preserve_item,
	PANEL_VALUE,	(globrec.flags & MB_G_PRESERVE) != 0,
	NULL);
    xv_set(am_ip->admin_item,
	PANEL_VALUE,	FALSE,
	XV_SHOW,	TRUE,
	NULL);

    xv_set(am_ip->am_panel, PANEL_CARET_ITEM, am_ip->logid_item, NULL);

    am_mode = ADDING;
    xv_set(am_ip->am_frame,
	XV_LABEL,	"Message Board: Add Entry",
	XV_SHOW,	TRUE,
	NULL);
}

/*
 *  Prepare A/M window for modify mode
 */
void
prep_modify_mode()
{
    Mem_Rec *rec;

    if (admin)
	rec = &ulist[cur_sel];
    else {
	if (myentry == (struct mem_rec *)NULL) {
	    pr_msg(NULL, TRUE, "%s is not currently on the Message Board.",
		mylogid);
	    return;
	}
	rec = myentry;
    }
/*
 *@  Check to see if we're not already doing something with the window!
 */
    if (!am_window_created) create_am_window();

    xv_set(am_ip->am_panel, PANEL_CARET_ITEM, am_ip->until_item, NULL);
    xv_set(am_ip->logid_item,
        PANEL_VALUE,            rec->logid,
        PANEL_READ_ONLY,        TRUE,
        NULL);
    xv_set(am_ip->names_item, XV_SHOW, FALSE, NULL);
    xv_set(am_ip->fname_item,
	PANEL_VALUE,		rec->fname,
	PANEL_READ_ONLY,	!admin,
	NULL);
    xv_set(am_ip->status_item, PANEL_VALUE, rec->status, NULL);
    xv_set(am_ip->until_item, PANEL_VALUE, rec->until, NULL);
    xv_set(am_ip->comment_item, PANEL_VALUE, rec->comment, NULL);
    xv_set(am_ip->cat_item,
	PANEL_VALUE,	rec->cat,
	XV_SHOW,	admin,
	NULL);
    xv_set(am_ip->sortby_item,
	PANEL_VALUE,	rec->sortby,
	XV_SHOW,	admin,
	NULL);
    xv_set(am_ip->ext_item,
	PANEL_VALUE,	rec->ext,
	XV_SHOW,	admin,
	NULL);
    xv_set(am_ip->pagernum_item,
	PANEL_VALUE,	rec->pagernum,
	XV_SHOW,	admin,
	NULL);
    xv_set(am_ip->pagertype_item,
	PANEL_VALUE,	rec->pagertype,
	PANEL_INACTIVE,	rec->pagernum[0] == '\0',
	XV_SHOW,	admin,
	NULL);
    xv_set(am_ip->plan_item,
	PANEL_VALUE, (rec->flags & MB_CHG_PLAN) != 0,
	NULL);
    xv_set(am_ip->preserve_item,
	PANEL_VALUE, (rec->flags & MB_PRESERVE) != 0,
	NULL);
    xv_set(am_ip->via_mail_item,
	PANEL_VALUE, (rec->flags & MB_VIA_MAIL) != 0,
	NULL);
    xv_set(am_ip->via_mb_item,
	PANEL_VALUE, (rec->flags & MB_VIA_MB) != 0,
	NULL);
    xv_set(am_ip->admin_item,
	PANEL_VALUE,		(rec->flags & MB_ADMIN) != 0,
	XV_SHOW,		admin,
	NULL);

    am_mode = MODIFYING;
    xv_set(am_ip->am_frame,
        XV_LABEL,       "Message Board: Modify Entry",
        XV_SHOW,       TRUE,
        NULL);

    if (names_window_created && (int)xv_get(nameswin_p->names_frame, XV_SHOW))
	xv_set(nameswin_p->names_frame,
	    FRAME_CMD_PIN_STATE,	FRAME_CMD_PIN_OUT,
	    XV_SHOW,			FALSE,
	    NULL);
}

void
create_names_window()
{
    int i, x, y;
    struct passwd *pwbuf;
    Rect am_rect;

    nameswin_p = amwin_names_frame_objects_initialize(NULL, base_frame);

#ifdef SCALABLE
    xv_set(nameswin_p->list_item, XV_X, wgrid * 2, XV_Y, wgrid * 2, NULL);
    window_fit(nameswin_p->names_panel);
    window_fit(nameswin_p->names_frame);
#endif
 
    /*
     *  Get the list of possible users
     */    
    setpwent();

    i = 0;
    while ( (pwbuf = getpwent()) != (struct passwd *)NULL) {
	if (pwbuf->pw_uid >= MIN_UID && pwbuf->pw_uid <= MAX_UID) {
	    xv_set(nameswin_p->list_item,
		PANEL_LIST_STRING,	i, pwbuf->pw_name,
		NULL);
	    i++;
	}
    }

    xv_set(nameswin_p->list_item,
	PANEL_LIST_SORT,	PANEL_FORWARD,
	NULL);

    /*
     *  Position the window to initially appear just to the right of the
     *  "Names..." button
     */
    frame_get_rect(am_ip->am_frame, &am_rect);
    x = am_rect.r_left +
	(int)xv_get(am_ip->names_item, XV_X) +
	(int)xv_get(am_ip->names_item, XV_WIDTH) +
	(int)xv_get(am_ip->am_panel,	PANEL_ITEM_X_GAP);

    y = am_rect.r_top +
	(int)xv_get(am_ip->names_item, XV_Y);

/*
 *@  Check to make sure we're not moving it off the screen!
 */

    /*@ i = y + names-frame-height - root_win_height
	if ( i > 0 ) y -= i; */

    if ( y < 0 ) y = 0;

    /*@ i = x +names-frame-width - root_win_width
	if ( i > 0 ) x -= i; */

    if ( x < 0 ) x = 0;

    xv_set(nameswin_p->names_frame,
	FRAME_CMD_DEFAULT_PIN_STATE,	FRAME_CMD_PIN_IN,
	XV_X, x,
	XV_Y, y,
	NULL);

    xv_set(nameswin_p->names_panel,
	WIN_EVENT_PROC,	names_panel_event,
	NULL);

   names_window_created = TRUE;
}

/*
 *  Fills record pointed to by "r" with values from input window.
 */
void
get_am_data(r)
Mem_Rec *r;
{
    strcpy(r->logid, (char *)xv_get(am_ip->logid_item, PANEL_VALUE));
    strcpy(r->fname, (char *)xv_get(am_ip->fname_item, PANEL_VALUE));
    strcpy(r->until, (char *)xv_get(am_ip->until_item, PANEL_VALUE));
    strcpy(r->comment, (char *)xv_get(am_ip->comment_item, PANEL_VALUE));
    strcpy(r->cat, (char *)xv_get(am_ip->cat_item, PANEL_VALUE));
    strcpy(r->sortby, (char *)xv_get(am_ip->sortby_item, PANEL_VALUE));
    strcpy(r->ext, (char *)xv_get(am_ip->ext_item, PANEL_VALUE));
    strcpy(r->pagernum, (char *)xv_get(am_ip->pagernum_item, PANEL_VALUE));
    r->status = (int)xv_get(am_ip->status_item, PANEL_VALUE);
    r->flags = 0;
    if ((int)xv_get(am_ip->plan_item, PANEL_VALUE)) r->flags |= MB_CHG_PLAN;
    if ((int)xv_get(am_ip->admin_item, PANEL_VALUE)) r->flags |= MB_ADMIN;
    if ((int)xv_get(am_ip->preserve_item, PANEL_VALUE)) r->flags |= MB_PRESERVE;
    if ((int)xv_get(am_ip->via_mail_item, PANEL_VALUE)) r->flags |= MB_VIA_MAIL;
    if ((int)xv_get(am_ip->via_mb_item, PANEL_VALUE)) r->flags |= MB_VIA_MB;
    r->pagertype = (int)xv_get(am_ip->pagertype_item, PANEL_VALUE);
}

/*
 * Notify callback function for `names_item'.
 */
void
am_names_notify(item, event)
Panel_item	item;
Event		*event;
{	
    if (!names_window_created) create_names_window();

    xv_set(nameswin_p->names_frame,
	XV_SHOW,		TRUE,
	NULL);
    /*
     *  Keep A/M window from being dismissed by this event,
     *  even if it's unpinned:
     */
    xv_set(item, PANEL_NOTIFY_STATUS, XV_ERROR, NULL);
}

/*
 * Notify callback function for `logid_item'.
 */
Panel_setting
am_logid_item_notify(item, event)
Panel_item item;
Event *event;
{
    if (make_default_fname()) {
	clear_msg();
	if (event_shift_is_down(event))
	    return PANEL_PREVIOUS;
	else
	    return PANEL_NEXT;
    }
    else {
	pr_msg(NULL, TRUE, "Invalid login ID: %s",
	    (char *)xv_get(item, PANEL_VALUE));
	return PANEL_NONE;
    }
}

/*
 * Notify callback function for `fname_item'.
 */
Panel_setting
am_fname_item_notify(item, event)
Panel_item	item;
Event		*event;
{
    char sortby[SORTBY_LEN+1];

    strcpy(sortby, (char *)xv_get(am_ip->sortby_item, PANEL_VALUE));
    if (sortby[0] == '\0') {
	make_sort_string((char *)xv_get(am_ip->fname_item, PANEL_VALUE),
	    sortby);
	xv_set(am_ip->sortby_item, PANEL_VALUE, sortby, NULL);
    }
    if (event_shift_is_down(event))
	return PANEL_PREVIOUS;
    else
	return PANEL_NEXT;
}

/*
 * Notify callback function for `status_item'.
 */
void
am_inout_notify(item, value, event)
Panel_item	item;
int		value;
Event		*event;
{
    if (value == MB_OUT)
	xv_set(am_ip->am_panel, PANEL_CARET_ITEM, am_ip->until_item, NULL);
    else if (value == MB_IN) {
	xv_set(am_ip->until_item, PANEL_VALUE, "", NULL);
	if (!(int)xv_get(am_ip->preserve_item, PANEL_VALUE))
	    xv_set(am_ip->comment_item, PANEL_VALUE, "", NULL);
    }
}

/*
 * Notify callback function for `pagernum_item'.
 */
Panel_setting
am_pagernum_item_notify(item, event)
Panel_item	item;
Event		*event;
{
    xv_set(am_ip->pagertype_item,
	PANEL_INACTIVE, (strlen((char *)xv_get(item, PANEL_VALUE)) == 0),
	NULL);
	
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `reset_item'.
 */
void
am_reset_notify(item, event)
Panel_item	item;
Event		*event;
{
    if ((int)xv_get(am_ip->am_frame, FRAME_CMD_PIN_STATE) ==
	FRAME_CMD_PIN_IN) {
	if (am_mode == ADDING)
	    prep_add_mode();
	else if (am_mode == MODIFYING)
	    prep_modify_mode();
    }
    else {
	am_mode = IDLE;
	if (names_window_created)
	    xv_set(nameswin_p->names_frame,
		FRAME_CMD_PIN_STATE,	FRAME_CMD_PIN_OUT,
		XV_SHOW,		FALSE,
		NULL);
    }
}

/*
 * Notify callback function for `apply_item'.
 */
void
am_apply_notify(item, event)
Panel_item	item;
Event		*event;
{
    Mem_Rec new_rec;

    get_am_data(&new_rec);
    if (am_mode == ADDING) {
	if (add_user_end(&new_rec) == 0) {
	    clear_msg();
	    if ((int)xv_get(am_ip->am_frame, FRAME_CMD_PIN_STATE) ==
		FRAME_CMD_PIN_IN)
		prep_add_mode();
	    else
		am_mode = IDLE;
	}
	else
	    xv_set(item, PANEL_NOTIFY_STATUS, XV_ERROR, NULL);
    }
    else if (am_mode == MODIFYING) {
	if (modify_user_end(&new_rec) == 0) {
	    clear_msg();
	    if ((int)xv_get(am_ip->am_frame, FRAME_CMD_PIN_STATE) ==
		FRAME_CMD_PIN_IN)
		prep_modify_mode();
	    else
		am_mode = IDLE;
	}
	else
	    xv_set(item, PANEL_NOTIFY_STATUS, XV_ERROR, NULL);
    }

    if (am_mode == IDLE) {
	if (names_window_created)
	    xv_set(nameswin_p->names_frame,
		FRAME_CMD_PIN_STATE,	FRAME_CMD_PIN_OUT,
		XV_SHOW,		FALSE,
		NULL);
    }
}

/*
 *  Act on the choice they made from the name chooser.
 */
void
set_name_from_list(string)
char *string;
{
    xv_set(am_ip->logid_item,
	PANEL_VALUE, string,
	NULL);
    if (make_default_fname())
	xv_set(am_ip->am_panel, PANEL_CARET_ITEM, am_ip->fname_item,
	    NULL);
    else
	xv_set(am_ip->am_panel, PANEL_CARET_ITEM, am_ip->logid_item,
	    NULL);
    xv_set(am_ip->sortby_item, PANEL_VALUE, "", NULL);
}

/*
 * Notify callback function for `list_item'.
 */
int
amwin_list_notify(item, string, client_data, op, event, row)
Panel_item	item;
char		*string;
Xv_opaque	client_data;
Panel_list_op	op;
Event		*event;
int		row;
{
    if (op == PANEL_LIST_OP_SELECT)
	set_name_from_list(string);

    return XV_OK;
}

/*
 *  Uses lookup_user() to get a reasonable full name out of the GECOS field;
 *  also gets the user's group and makes it their default category.
 *
 *  If MB_G_LASTNM1ST (last name first) is set, make a stab at reversing the
 *  name order...
 */
int
make_default_fname()
{
    int len;
    char logid[LOGID_LEN+1], fname[FNAME_LEN+1], cat[CAT_LEN+1],
	 buf[FNAME_LEN + 1], *space, *ptr;

    strcpy(logid, (char *)xv_get(am_ip->logid_item, PANEL_VALUE));
    if (logid[0] == '\0' || lookup_user(logid, fname, cat) != 0)
	return FALSE;

    if ((globrec.flags & MB_G_LASTNM1ST) != 0 &&
	(space = strrchr(fname, ' ')) != CNULL) {
	strcpy(buf, space + 1);
	strncat(buf, ", ", FNAME_LEN - strlen(buf));
	for (ptr = fname, len = strlen(buf) ;
	    ptr != space && len < FNAME_LEN ; ptr++, len++)
	    buf[len] = *ptr;
	buf[len] = '\0';
	strcpy(fname, buf);
    }
    xv_set(am_ip->fname_item, PANEL_VALUE, fname, NULL);
    xv_set(am_ip->cat_item, PANEL_VALUE, cat, NULL);
	
    return TRUE;
}

/*
 *  Monitors resize events on the name chooser panel
 */
void
names_panel_event(window, event)
Xv_window window;
Event	*event;
{
    /*
     *  Check for a resize event
     */
    if (event_action(event) == WIN_RESIZE) {
	int rh, nrows, nlist_h, xtra;

	/*
	 *  "xtra" is the part of the scrolling list height that is not
	 *  contributed by the rows themselves
	 */
	rh = (int)xv_get(nameswin_p->list_item, PANEL_LIST_ROW_HEIGHT);
	xtra = (int)xv_get(nameswin_p->list_item, XV_HEIGHT) -
	    rh * (int)xv_get(nameswin_p->list_item, PANEL_LIST_DISPLAY_ROWS);

	/*
	 *  Figure out how long the scrolling list should be now
	 */
	nlist_h = (int)xv_get(nameswin_p->names_panel, XV_HEIGHT) -
	    (int)xv_get(nameswin_p->list_item, XV_Y) - wgrid * 2;

	nrows = (nlist_h - xtra) / rh;
	nlist_h = nrows * rh  + xtra;

	xv_set(nameswin_p->list_item,
	    XV_HEIGHT,			nlist_h,
	    PANEL_LIST_DISPLAY_ROWS,	nrows,
	    NULL);
    }
}

/*
 * Done callback function for `am_frame'.
 */
void
amwin_done(frame)
Frame		frame;
{
    if (names_window_created)
	xv_set(nameswin_p->names_frame,
	    FRAME_CMD_PIN_STATE,	FRAME_CMD_PIN_OUT,
	    XV_SHOW,			FALSE,
	    NULL);
    xv_set(frame, XV_SHOW, FALSE, NULL);
}

