/*
 * mainwin.c - support routines for the Message Board main window.
 */

#include <sys/param.h>
#include <sys/types.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <search.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/notice.h>
#include <xview/scrollbar.h>
#include <xview/font.h>
#include "mainwin_ui.h"
#include "msgboard.h"
#include "msgboard_xv.h"

/*
 *  Warning: the next defines must be changed if you change the order of
 *  items on the View menu.  Note that the pushpin is considered an item,
 *  which is why "Category", the first selectable item on the menu, is
 *  numbered 2.
 */
#define VIEW_CAT_POSITION	2
#define VIEW_STATUS_POSITION	3
#define VIEW_WATCH_POSITION	5

/*
 *  Maximum length of the "display" string for each scrolling list item,
 *  including the null character
 */
#define DISP_LEN	FNAME_LEN + EXT_LEN + UNTIL_LEN + COMMENT_LEN + 15

mainwin_frame_objects *main_ip;

void		(*msl_event_proc)();

Menu_item	ed_delete_mi = (Menu_item)NULL,
		ed_modify_mi = (Menu_item)NULL,
		view_watch_mi = (Menu_item)NULL,
		msl_mod_mi = (Menu_item)NULL,
		msl_watch_mi = (Menu_item)NULL,
		msl_send_mi = (Menu_item)NULL,
		msl_scan_mi = (Menu_item)NULL,
		msl_page_mi = (Menu_item)NULL,
		msl_outs_mi = (Menu_item)NULL;

Menu cat_menu = (Menu)NULL;

Scrollbar	main_list_bar;

int
    pending_sel = BOGUS,		/*  Holds last row touched on
    					    main scrolling list		    */
    sb_offset = 0,			/* width of panel list scrollbar, iff
					   it's located on the left.	    */
    display_mode = DISP_TEXT;		/* What type of info the list shows */

static char *status[3] = {
    "Out",
    "In",
    "???",
    };

Server_image	signs[3];

short out_bits[] = {
#include "out.icon"
};

short in_bits[] = {
#include "in.icon"
};

short unk_bits[] = {
#include "unk.icon"
};

void
create_signs()
{
    signs[0] = (Server_image)xv_create(NULL, SERVER_IMAGE,
	XV_WIDTH,		INOUT_WIDTH,
	XV_HEIGHT,		16,
	SERVER_IMAGE_BITS,	out_bits,
	NULL);

    signs[1] = (Server_image)xv_create(NULL, SERVER_IMAGE,
	XV_WIDTH,		INOUT_WIDTH,
	XV_HEIGHT,		16,
	SERVER_IMAGE_BITS,	in_bits,
	NULL);

    signs[2] = (Server_image)xv_create(NULL, SERVER_IMAGE,
	XV_WIDTH,		INOUT_WIDTH,
	XV_HEIGHT,		16,
	SERVER_IMAGE_BITS,	unk_bits,
	NULL);
}

/*
 *  Call this routine whenever the state of the program changes in some way
 *  to require the active/inactive status of main window objects to change.
 *
 *  The display status of the various objects is dependent on:
 *	* whether there's a current selection on the main list
 *      * whether the user is currently in admin mode
 *	* whether the connection to the server is active
 *  The scan button is also dependent on whether MB_G_TRUSTED
 *  ("trusted office" mode) is true for the current server.
 */
void
set_mainwin_items_status()
{
    int no_sel, server_down, state;

    no_sel = (cur_sel == BOGUS);
    server_down = !server_connected();

/*
 *  The edit button only depends on the current server status.
 */
    xv_set(main_ip->edit_item, PANEL_INACTIVE, server_down, NULL);

/*
 *  If the user is in admin mode, these items are inactive when there's
 *  no selection:
 */
    state = (admin && no_sel) || server_down;
    if ( (globrec.flags & MB_G_TRUSTED) == 0 )
        xv_set(main_ip->scanmsg_item, PANEL_INACTIVE, state, NULL);
    xv_set(main_ip->outs_item, PANEL_INACTIVE, state, NULL);
    if (ed_delete_mi != (Menu_item)NULL)
	xv_set(ed_delete_mi, MENU_INACTIVE, state, NULL);
    if (ed_modify_mi != (Menu_item)NULL)
	xv_set(ed_modify_mi, MENU_INACTIVE, state, NULL);
    if (msl_mod_mi != (Menu_item)NULL)
	xv_set(msl_mod_mi, MENU_INACTIVE, state, NULL);
    if (msl_scan_mi != (Menu_item)NULL)
	xv_set(msl_scan_mi, MENU_INACTIVE, state, NULL);
    if (msl_outs_mi != (Menu_item)NULL)
	xv_set(msl_outs_mi, MENU_INACTIVE, state, NULL);

/*
 *  These items are inactive when there's no selection in either mode:
 */
    state = no_sel || server_down;
    if ( view_watch_mi == (Menu_item)NULL) {
	Menu view_menu;

	view_menu = (Menu)xv_get(main_ip->view_item, PANEL_ITEM_MENU);
	view_watch_mi = (Menu_item)xv_get(view_menu, MENU_NTH_ITEM,
	    VIEW_WATCH_POSITION);
    }
    xv_set(view_watch_mi, MENU_INACTIVE, state, NULL);
    if ( (globrec.flags & MB_G_TRUSTED) != 0 )
	xv_set(main_ip->scanmsg_item,
	    PANEL_INACTIVE, state, NULL);
    xv_set(main_ip->sendmsg_item, PANEL_INACTIVE, state, NULL);
    xv_set(main_ip->page_item, PANEL_INACTIVE, state, NULL);
    if (msl_watch_mi != (Menu_item)NULL)
	xv_set(msl_watch_mi, MENU_INACTIVE, state, NULL);
    if (msl_send_mi != (Menu_item)NULL)
	xv_set(msl_send_mi, MENU_INACTIVE, state, NULL);
    if (msl_page_mi != (Menu_item)NULL)
	xv_set(msl_page_mi, MENU_INACTIVE, state, NULL);

/*
 *  Fix up the comment line:
 */
    if (no_sel)
        xv_set(main_ip->comment_item,
	    PANEL_INACTIVE,	TRUE,
	    PANEL_VALUE,	"",
	    NULL);
    else
        xv_set(main_ip->comment_item,
	    PANEL_INACTIVE,	FALSE,
	    PANEL_VALUE,	ulist[cur_sel].comment,
	    NULL);

/*
 *  Turn the "new messages" glyph on or off.
 */
    xv_set(main_ip->newmsg_item,
	XV_SHOW, myentry != NULL && (myentry->flags & MB_NEWMSGS) != 0,
	NULL);
}

/*
 *  Displays the site name and message of the day line
 */
void
update_global_display()
{
    int p_wid, m_wid, disconnected;
    char lbuf[SITE_LEN + 19];

    disconnected = !server_connected();
    
    if (disconnected)
	xv_set(main_ip->motd_item, PANEL_LABEL_STRING,
	    "Not connected to a server", NULL);
    else if (admin && globrec.name[0] == '\0' && globrec.motd[0] == '\0')
	xv_set(main_ip->motd_item, PANEL_LABEL_STRING, 
	    "Press 'Props...' button to set site name and/or message", NULL);
    else
	xv_set(main_ip->motd_item, PANEL_LABEL_STRING, globrec.motd, NULL);

    p_wid = (int)xv_get(main_ip->display_panel, XV_WIDTH);
    m_wid = (int)xv_get(main_ip->motd_item, XV_WIDTH);
    xv_set(main_ip->motd_item, XV_X, (p_wid - m_wid) / 2, NULL);

    /*
     *  Update main frame label:
     */
    if (disconnected)
	strcpy(lbuf, "Message Board");
    else {
	sprintf(lbuf, "Message Board for %s", globrec.name);
	if (admin) strcat(lbuf, " (admin mode)");
	xv_set(base_frame, XV_LABEL, lbuf, NULL);
    }
}

/*
 *  Wrapper to let the scrolling list menu bring up the modify window.
 *  (msl_* : Main Scrolling List)
 */
void
msl_modify_notify(menu, menu_item)
Menu menu;
Menu_item menu_item;
{
    clear_msg();
    prep_modify_mode();
}

/*
 *  Wrapper to let the scrolling list menu bring up the send window
 */
void
msl_send_notify(menu, menu_item)
Menu menu;
Menu_item menu_item;
{
    clear_msg();
    show_sendmsg_window();
}

/*
 *  Wrapper to let the scrolling list menu bring up the scan window
 */
void
msl_scan_notify(menu, menu_item)
Menu menu;
Menu_item menu_item;
{
    clear_msg();
    show_scan_window();
}

/*
 *  Wrapper to let the scrolling list menu bring up the page window
 */
void
msl_page_notify(menu, menu_item)
Menu menu;
Menu_item menu_item;
{
    clear_msg();
    show_page_window();
}

/*
 *  Monitor resize events on the main display panel.
 */
void
main_display_event(window, event)
Xv_window window;
Event	*event;
{
    static int
	list_w, list_h;			/* list width, height */
    int y, row_h, nrows;

    if (event_action(event) != WIN_RESIZE) return;

    list_h =
	(int)xv_get(main_ip->display_panel, XV_HEIGHT) -
	(int)xv_get(main_ip->list_item, XV_Y) -
	(int)xv_get(main_ip->comment_item, XV_HEIGHT) -
	wgrid * 3;

    /*
     *  20 may or may not be an appropriate constant here
     */
    row_h = (int)xv_get(main_ip->list_item, PANEL_LIST_ROW_HEIGHT);
    nrows = (list_h - 20) / row_h;
    list_h = nrows * row_h  + 20;
    list_w = (int)xv_get(main_ip->display_panel, XV_WIDTH) - wgrid * 4;

    xv_set(main_ip->list_item,
	XV_HEIGHT,			list_h,
	PANEL_LIST_WIDTH,		list_w,
	PANEL_LIST_DISPLAY_ROWS,	nrows,
	NULL);

    y = list_h + (int)xv_get(main_ip->list_item, XV_Y, NULL) + wgrid;
    xv_set(main_ip->comment_item, XV_Y, y, NULL);
    xv_set(main_ip->newmsg_item, XV_Y, y + wgrid, NULL);

    if (nrows >= users)
	xv_set(main_list_bar, SCROLLBAR_VIEW_START, 0, NULL); 
    update_global_display();
}

/*
 * Menu handler for `list_menu'.
 */
Menu
main_list_menu_handler(menu, op)
Menu		menu;
Menu_generate	op;
{	
    static int main_list_menu_updated = FALSE;

    if (op == MENU_DISPLAY && !main_list_menu_updated) {
	Menu pr_menu;

	xv_set(menu, MENU_TITLE_ITEM, "Board Entries", NULL);

	if (admin) {
	    msl_mod_mi = (Menu_item)xv_create(NULL, MENUITEM,
		MENU_STRING,		"Modify Entry...",
		MENU_NOTIFY_PROC,	msl_modify_notify,
		MENU_INACTIVE,		(cur_sel == BOGUS),
		NULL);
	    xv_set(menu, MENU_APPEND_ITEM, msl_mod_mi, NULL);
	}

	/*
	 *  Create a "Watch For" entry, using the same menu as the
	 *  similar entry on the View menu does.
	 */
	if ( view_watch_mi == (Menu_item)NULL) {
	    pr_menu = (Menu)xv_get(main_ip->view_item, PANEL_ITEM_MENU);
	    view_watch_mi = (Menu_item)xv_get(pr_menu, MENU_NTH_ITEM,
		VIEW_WATCH_POSITION);
	}
	pr_menu = (Menu)xv_get(view_watch_mi, MENU_PULLRIGHT);
	msl_watch_mi = (Menu_item)xv_create(NULL, MENUITEM,
	    MENU_STRING,	"Watch For",
	    MENU_PULLRIGHT,	pr_menu,
	    MENU_INACTIVE,	(cur_sel == BOGUS),
	    NULL);
	xv_set(menu, MENU_APPEND_ITEM, msl_watch_mi, NULL);

	msl_send_mi = (Menu_item)xv_create(NULL, MENUITEM,
	    MENU_STRING,	"Send Pmsg...",
	    MENU_NOTIFY_PROC,	msl_send_notify,
	    MENU_INACTIVE,	(cur_sel == BOGUS),
	    NULL);
	xv_set(menu, MENU_APPEND_ITEM, msl_send_mi, NULL);

	if (admin) {
	    msl_scan_mi = (Menu_item)xv_create(NULL, MENUITEM,
		MENU_STRING,		"Scan Pmsgs...",
		MENU_NOTIFY_PROC,	msl_scan_notify,
		MENU_INACTIVE,		(cur_sel == BOGUS),
		NULL);
	    xv_set(menu, MENU_APPEND_ITEM, msl_scan_mi, NULL);
	}

	msl_page_mi = (Menu_item)xv_create(NULL, MENUITEM,
	    MENU_STRING,	"Page...",
	    MENU_NOTIFY_PROC,	msl_page_notify,
	    MENU_INACTIVE,	(cur_sel == BOGUS),
	    NULL);
	xv_set(menu, MENU_APPEND_ITEM, msl_page_mi, NULL);
	
	if (admin) {
	    pr_menu = (Menu)xv_get(main_ip->outs_item, PANEL_ITEM_MENU);
	    msl_outs_mi = (Menu_item)xv_create(NULL, MENUITEM,
		MENU_STRING,	"Outs",
		MENU_PULLRIGHT,	pr_menu,
		MENU_INACTIVE,	(cur_sel == BOGUS),
		NULL);
	    xv_set(menu, MENU_APPEND_ITEM, msl_outs_mi, NULL);
	}

	main_list_menu_updated = TRUE;
    }
    return menu;
}

/*
 *  Handles destroy events off of the main window, including the
 *  dreaded WM_SAVE_YOURSELF.
 */
Notify_value
save_me(client, status)
Notify_client client;
Destroy_status status;
{
    Destroy_status retval = NOTIFY_DONE;

    switch (status) {
    case DESTROY_CHECKING:
	retval = notify_next_destroy_func(client, status);
    case DESTROY_SAVE_YOURSELF:
	save_params();
	break;
    case DESTROY_CLEANUP:
	retval = notify_next_destroy_func(client, status);
	break;
    case DESTROY_PROCESS_DEATH:
	base_frame = NULL;
	break;
    }
    return NOTIFY_DONE;
}

/*
 * Event callback function for `list_item'.
 *  Catches clicks on the "In/Out" column of the scrolling list.
 */
void
main_list_event(item, event)
Panel_item	item;
Event		*event;
{
    static int
	list_preview = FALSE;		/* Is list in preview mode?	*/

    /*
     *  First, call the main list's "real event proc:
     */
    (*msl_event_proc)(item, event);
    
    /*
     *  Now, do our own processing on the event.
     *  We're only interested in events from the SELECT mouse button.
     */
    if (event_action(event) != ACTION_SELECT) return;

    /*
     *  A down event in the scrolling list; enter preview mode.
     */
    if (event_is_down(event))
	list_preview = TRUE;
    /*
     *  A release event when we're in preview mode;
     *  Check whether to complete selection or abort preview.
     */
    else if (event_is_up(event) && list_preview) {	
	clear_msg(); /* For good housekeeping's sake */
	list_preview = FALSE;

	/*
	 *  If there's a good selection pending, act on it.
	 */
	if (pending_sel != BOGUS) {
	    /*
	     *  If the action took place on the glyph column and we have
	     *  permission, make this the current selection and toggle its
	     *  in/out value.
	     */
	    if (event_x(event) - (int)xv_get(main_ip->list_item, XV_X) -
		sb_offset < INOUT_WIDTH + 10 &&
		(admin || &(ulist[pending_sel]) == myentry)) {
		int new_status;
		char *new_until, *new_comment, temp_logid[LOGID_LEN + 1];
		Mem_Rec *cur_ent;

		cur_ent = &(ulist[pending_sel]);
		new_status = (cur_ent->status != MB_IN);
		/*
		 *  If moving to In: clear the until field;
		 *  if preserve isn't set, clear comment field too.
		 */
		if (new_status == MB_IN) {
		    new_until = strdup("");
		    if ((cur_ent->flags & MB_PRESERVE) != 0)
			new_comment = CNULL;
		    else
			new_comment = strdup("");
		}
		else
		    new_until = new_comment = CNULL;

		/*
		 *  Save the id so we can relocate this record
		 *  after we call get_modified_records()...
		 */
		strcpy(temp_logid, cur_ent->logid);
		
		if (modify_status(cur_ent->logid, new_status, new_until,
		    new_comment) == 0 &&
		    get_modified_records() == 0) {
		    /*
		     *  Find this record again.
		     */
		    cur_ent = find_urec(temp_logid);

		    if (cur_ent == (Mem_Rec *)NULL)
			cur_sel = BOGUS;
		    else {
			/*
			 *  If we moved to OUT and pop_mod is set, expose
			 *  the "Out modify" window.
			 */
			if (cur_ent->status == MB_OUT && pop_mod)
			    show_outmod_window(cur_ent);

			/*
			 *  Clicking on the glyph column of a row that
			 *  was selected will have deselected it; we want
			 *  to keep it selected.
		         */
			if (!(int)xv_get(main_ip->list_item,
			    PANEL_LIST_SELECTED, cur_ent->row))
			    xv_set(main_ip->list_item, PANEL_LIST_SELECT,
			    cur_ent->row, TRUE, NULL);
			cur_sel = cur_ent - ulist;
		    }
		}
	    }
	    /*
	     *  If it took place on the name column, or the glyph column of
	     *  a row we don't have permission to toggle: deselect if this
	     *  was already the current selection, else move to it.
	     */
	    else {
		if (cur_sel == pending_sel)
		    cur_sel = BOGUS;
		else
		    cur_sel = pending_sel;
	    }

	    set_mainwin_items_status();
	    pending_sel = BOGUS;
	} /* if (pending_sel != BOGUS) */

    } /* if ( up_event && list_preview ) */

    return;
}

/*
 *  Calls the devGuide-generated routine to create the various XView
 *  objects, makes the tweaks that devGuide itself doesn't support yet,
 *  including installing various event handlers.
 */
void
create_main_window()
{
    Xv_Font	font;
    Window_rescale_state scale;
    int xpos, ypos;

    main_ip = mainwin_frame_objects_initialize(NULL, NULL);

    base_frame = main_ip->frame;

    /*
     *  What scale is the window at?  We use "wgrid" to lay out panel
     *  items all through the rest of the application.
     */
    font = (Xv_Font)xv_get(base_frame, XV_FONT);
    scale = (Window_rescale_state)xv_get(font, FONT_SCALE);
    switch(scale) {
    case WIN_SCALE_SMALL:
	wgrid = 8;
	break;   
    case WIN_SCALE_LARGE:
	wgrid = 12;
	break;
    case WIN_SCALE_EXTRALARGE:
	wgrid = 16;
	break;
    default:
	wgrid = 10;
    }

    notify_interpose_destroy_func(base_frame, save_me);

    if (main_params.state != BOGUS)
	frame_set_rect(base_frame, &main_params.rect);

    xv_set(main_ip->display_panel,
	WIN_EVENT_PROC,		main_display_event,
	PANEL_ACCEPT_KEYSTROKE,	TRUE,
	NULL);

    xv_set((Menu)xv_get(main_ip->list_item, PANEL_ITEM_MENU),
	MENU_GEN_PROC, main_list_menu_handler, NULL);

    /*
     *  Save the main scrolling list's original event handler, then
     *  replace it with our own.
     */
    msl_event_proc = (void (*)())xv_get(main_ip->list_item, PANEL_EVENT_PROC);
    
    xv_set(main_ip->list_item, PANEL_EVENT_PROC, main_list_event, NULL);

    /*
     *  The routine that tracks select events on the panel list needs to
     *  know if the scrollbar is on the left side, and if so, how wide it
     *  is, so that it can figure out whether or not a mouse event is
     *  is occuring on the column the "In/Out/???" glyph appears in.
     */
    main_list_bar = (Scrollbar)xv_get(main_ip->list_item,
	PANEL_LIST_SCROLLBAR);
    if ((int)xv_get(main_list_bar, XV_X) ==
	(int)xv_get(main_ip->list_item, XV_X))
	sb_offset = (int)xv_get(main_list_bar, XV_WIDTH);

    /*
     *  Create the outs menu and attach it to the outs button
     */
    outs_menu_create(main_ip->outs_item);
    
#ifdef SCALABLE
    /*
     *  Position panel items in a scalable fashion:
     */
    ypos = wgrid * 0.8;
    xpos = wgrid * 3;
    xv_set(main_ip->view_item,
	XV_X,	xpos,
	XV_Y,	ypos,
	NULL);

    xpos += (int)xv_get(main_ip->view_item, XV_WIDTH) + wgrid;
    xv_set(main_ip->edit_item, XV_X, xpos, XV_Y, ypos, NULL);

    xpos += (int)xv_get(main_ip->edit_item, XV_WIDTH) + wgrid;
    xv_set(main_ip->props_item, XV_X, xpos, XV_Y, ypos, NULL);

    xpos += (int)xv_get(main_ip->props_item, XV_WIDTH) + wgrid * 2;
    xv_set(main_ip->sendmsg_item, XV_X, xpos, XV_Y, ypos, NULL);

    xpos += (int)xv_get(main_ip->sendmsg_item, XV_WIDTH) + wgrid;
    xv_set(main_ip->scanmsg_item, XV_X, xpos, XV_Y, ypos, NULL);

    xpos += (int)xv_get(main_ip->scanmsg_item, XV_WIDTH) + wgrid;
    xv_set(main_ip->page_item, XV_X, xpos, XV_Y, ypos, NULL);

    xpos += (int)xv_get(main_ip->page_item, XV_WIDTH) + wgrid * 2;
    xv_set(main_ip->outs_item, XV_X, xpos, XV_Y, ypos, NULL);

    ypos = wgrid * 1.6 + (int)xv_get(main_ip->view_item, XV_HEIGHT);
    xv_set(main_ip->control_panel, XV_HEIGHT, ypos, NULL);
    window_fit_width(main_ip->control_panel);

    xv_set(main_ip->display_panel,
	XV_Y,		ypos,
	XV_WIDTH,	(int)xv_get(main_ip->control_panel, XV_WIDTH),
	NULL);

    ypos = wgrid *.8;
    /*
     *  As we set the motd_item, set its value to something, so that it has
     *  a usable height.
     */
    xv_set(main_ip->motd_item,
	XV_Y,			ypos,
	PANEL_PAINT,		PANEL_NONE,
	PANEL_LABEL_STRING,	"FOOBAR",
	NULL);

    ypos += (int)xv_get(main_ip->motd_item, XV_HEIGHT) + wgrid *.8;
    xv_set(main_ip->list_item,
	XV_X, wgrid,
	XV_Y, ypos,
	PANEL_LIST_WIDTH,	100,
	NULL);
    /*
     *  We don't set the comment or newmsg item's Y position here; that's
     *  done in main_display_event().  16 is the width of the newmsg glyph;
     *  yet another possibly inappropriate thing to hardwire.
     */
    xv_set(main_ip->newmsg_item, XV_X, wgrid, NULL);
    xv_set(main_ip->comment_item, XV_X, wgrid * 2 + 16, NULL);
    window_fit(base_frame);
#endif
}

/*
 *  Creates in "sbuf" the display line for record pointed to by "rp"
 */
void
make_disp_string(sbuf, rp)
char *sbuf;
Mem_Rec *rp;
{
    strcpy(sbuf, rp->fname);
    if (admin && (rp->flags & MB_NEWMSGS) != 0)
	strcat(sbuf, "\266");
    if (display_mode == DISP_TEXT) {
        if (strlen(rp->ext) > 0) {
            strcat(sbuf, ", x");
            strcat(sbuf, rp->ext);
        }
	if (strlen(rp->until) > 0) {
	    strcat(sbuf, " - until ");
	    strcat(sbuf, rp->until);
	}
	if (strlen(rp->comment) > 0) {
	    strcat(sbuf, " - ");
	    strcat(sbuf, rp->comment);
	}
    }
    else {
	strcat(sbuf, " - ");
	strcat(sbuf, ctime(&(rp->timestamp)));
    }
}

/*
 *  Updates a currently existing line on the screen
 */
void
update_line(rp)
Mem_Rec *rp;
{
    char sbuf[DISP_LEN];

    make_disp_string(sbuf, rp);
    
    xv_set(main_ip->list_item,
	PANEL_LIST_STRING, rp->row,		sbuf,
	PANEL_LIST_GLYPH, rp->row,		signs[rp->status],
	NULL);

    if (rp->row == cur_sel)
	xv_set(main_ip->comment_item,
	    PANEL_VALUE,	ulist[cur_sel].comment,
	    PANEL_INACTIVE,	FALSE,
	    NULL);
}

/*
 *  Redoes everything on the main scrolling list.
 */
void
update_screen()
{
    int i, list_size, new_size;
    char fbuf[CAT_LEN + 24], sbuf[DISP_LEN];

    list_size = (int)xv_get(main_ip->list_item, PANEL_LIST_NROWS);

    xv_set(main_ip->list_item, XV_SHOW, FALSE, NULL);

    /*
     *  Turn off select indicator if it's on
     */
    if (cur_sel != BOGUS && ulist[cur_sel].row != BOGUS)
	xv_set(main_ip->list_item,
	    PANEL_LIST_SELECT, ulist[cur_sel].row, FALSE, NULL);

    new_size = 0;
    for ( i = 0; i < users; i++) {
	/*
	 *  A user is displayed if:
	 *	They're in the current category, or the current
	 *	    category is ALL, AND
	 *	Their status is the current one, or the current status
	 *	    is ALL.
	 */
	if ( (cur_cat == MB_ALL || strcmp(cats[cur_cat], ulist[i].cat) == 0) &&
	     (cur_status == MB_ALL || cur_status == ulist[i].status) ) {

	    /*
	     *  Create the display string
	     */
	    make_disp_string(sbuf, &(ulist[i]));

	    /*
	     *  If the list is large enough to accomodate this user, go
	     *  ahead, else create a new slot.  Set client data for each
	     *  row to the ulist index for this row.
	     */
	    if ( new_size < list_size)
		xv_set(main_ip->list_item,
		    PANEL_LIST_STRING, new_size,	sbuf,
		    PANEL_LIST_GLYPH, new_size,		signs[ulist[i].status],
		    PANEL_LIST_CLIENT_DATA, new_size,	i,
		    NULL);
	    else {
		xv_set(main_ip->list_item,
		    PANEL_LIST_INSERT,			new_size,
		    PANEL_LIST_STRING, new_size,	sbuf,
		    PANEL_LIST_GLYPH, new_size,		signs[ulist[i].status],
		    PANEL_LIST_CLIENT_DATA, new_size,	i,
		    NULL);
		++list_size;
	    }
	    ulist[i].row = new_size;
	    ++new_size;
	}
	else
	    ulist[i].row = BOGUS;
    }

    /*
     *  See if we have to destroy some list rows:
     */
    if (list_size > new_size) {
	xv_set(main_ip->list_item,
	    PANEL_LIST_DELETE_ROWS, new_size, list_size - new_size,
	    NULL);
	if (cur_sel >= users) cur_sel = BOGUS;
    }

    /*
     *  Turn indicator back on if appropriate; set comment line.
     */
    if (cur_sel != BOGUS && ulist[cur_sel].row != BOGUS) {
	xv_set(main_ip->list_item,
	    PANEL_LIST_SELECT, ulist[cur_sel].row, TRUE, NULL);
	xv_set(main_ip->comment_item,
	    PANEL_VALUE,	ulist[cur_sel].comment,
	    PANEL_INACTIVE,	FALSE,
	    NULL);
    }
    else
	xv_set(main_ip->comment_item,
	    PANEL_VALUE,	"",
	    PANEL_INACTIVE,	TRUE,
	    NULL);

    xv_set(main_ip->list_item, XV_SHOW, TRUE, NULL);

    /*
     *  Update the footer:
     */
    if (cur_cat != MB_ALL) {
	sprintf(fbuf, "Category: %s", cats[cur_cat]);
	if (cur_status != MB_ALL)
		strcat(fbuf, ", ");
    }
    else
	fbuf[0] = '\0';
    if (cur_status != MB_ALL) {
	strcat(fbuf, "Status: ");
	strcat(fbuf, status[cur_status]);
    }
    xv_set(base_frame, FRAME_RIGHT_FOOTER, fbuf, NULL);
}

/*
 *  Handles notify events from the category menu
 */
void
cat_menu_notify(menu, menu_item)
Menu menu;
Menu_item menu_item;
{
    int i, old_cat;

    char mstr[CAT_LEN+1];

    old_cat = cur_cat;
    strcpy(mstr, (char *)xv_get(menu_item, MENU_STRING));

    /*
     *  If the selected item is "(Next)", cycle the current category to the
     *  next higher item.  If we go above the highest available categories,
     *  we cycle back to the first item (MB_ALL).
     */
    if (strcmp(mstr, "(Next)") == 0) {
	cur_cat++;
	if (cur_cat >= numcats) cur_cat = MB_ALL;
    }
    else if ( strcmp(mstr, "All") == 0)
	cur_cat = MB_ALL;
    else
	for (i = 0; i < numcats; i++)
	    if ( strcmp(mstr, cats[i]) == 0) {
		cur_cat = i;
		break;
	    }

    if (cur_cat != old_cat) {
	update_screen();
	xv_set(main_list_bar, SCROLLBAR_VIEW_START, 0, NULL); 
    }
}

/*
 *  Handles notify events from the status menu.
 *  This code assumes the menu looks like:
 *	1.   o  -[]=|  (the pushpin)
 *	2.   (Next)
 *      3.   All
 *      4.   "In"
 *      5.   "Out"
 *      6.   ???
 */

#define NUM_STATUS_MENU_ITEMS	6

void
status_menu_notify(menu, menu_item)
Menu menu;
Menu_item menu_item;
{
    int pos, old_status;

    old_status = cur_status;

    for (pos = 2; pos <= NUM_STATUS_MENU_ITEMS; pos++)
	if (menu_item == (Menu_item)xv_get(menu, MENU_NTH_ITEM, pos)) break;

    /*
     *  This should never happen:
     */
    if (pos == NUM_STATUS_MENU_ITEMS + 1) {
	pr_msg(NULL, TRUE, "Couldn't match item on View->Status menu");
	return;
    }

    if (pos == 2) {
	cur_status++;
	if (cur_status > 2) cur_status = MB_ALL;
    }
    else if (pos == 3)
	cur_status = MB_ALL;
    else
	cur_status = pos - 4;

    if (cur_status != old_status) {
	update_screen();
	xv_set(main_list_bar, SCROLLBAR_VIEW_START, 0, NULL); 
    }
}

/*
 * Menu handler for `edit_menu (Modify Entry...)'.
 */
Menu_item
main_modify_handler(item, op)
Menu_item	item;
Menu_generate	op;
{
    switch (op) {
    case MENU_DISPLAY:
	if (ed_modify_mi == (Menu_item)NULL)
	    ed_modify_mi = item;
	if (admin) xv_set(item, MENU_INACTIVE, cur_sel == BOGUS, NULL);
	break;

    case MENU_NOTIFY:
	prep_modify_mode();
	break;
    }
    return item;
}

/*
 * Menu handler for `edit_menu (Add Entry...)'.
 */
Menu_item
main_add_handler(item, op)
Menu_item	item;
Menu_generate	op;
{
    switch (op) {
    case MENU_DISPLAY:
	xv_set(item, MENU_INACTIVE, !admin, NULL);
	break;

    case MENU_DISPLAY_DONE:
	break;

    case MENU_NOTIFY:
	prep_add_mode(base_frame);
	break;

    case MENU_NOTIFY_DONE:
	break;
    }
    return item;
}

/*
 * Menu handler for `edit_menu (Delete Entry)'.
 */
Menu_item
main_delete_handler(item, op)
Menu_item	item;
Menu_generate	op;
{
    char logid[LOGID_LEN + 1], del_string[LOGID_LEN + FNAME_LEN + 4];
    Xv_Notice notice;
    int notice_stat;
	
    switch (op) {
    case MENU_DISPLAY:
	if (ed_delete_mi == (Menu_item)NULL)
	    ed_delete_mi = item;
	xv_set(item, MENU_INACTIVE, !admin || (cur_sel == BOGUS), NULL);
	break;

    case MENU_DISPLAY_DONE:
	break;

    case MENU_NOTIFY:
	/*
	 *  Save the login ID, in case the user manages somehow to
	 *  move the current selection before we call delete_user.
	 */
	strcpy(logid, ulist[cur_sel].logid);

	/*
	 *  Generate "Did you really want to do this?" notice
	 */
	sprintf(del_string, "%s (%s)", logid, ulist[cur_sel].fname);
	notice = xv_create(base_frame, NOTICE,
	    NOTICE_MESSAGE_STRINGS,	"Confirm deletion of entry for",
					del_string,
					NULL,
	    NOTICE_BUTTON_YES,		"Delete",
	    NOTICE_BUTTON_NO,		"Cancel",
	    NOTICE_STATUS,		&notice_stat,
	    XV_SHOW,			TRUE,
	    NULL);

	if (notice_stat == NOTICE_YES) {
	    xv_set(main_ip->list_item,
		PANEL_LIST_SELECT, ulist[cur_sel].row, FALSE, NULL);
	    cur_sel = BOGUS;
	    set_mainwin_items_status();
	    delete_user(logid);
	}

	xv_destroy_safe(notice);
	break;

    case MENU_NOTIFY_DONE:
	break;
    }
    return item;
}

/*
 * Menu handler for `view_menu (About Message Board...)'.
 */
Menu_item
about_handler(item, op)
Menu_item	item;
Menu_generate	op;
{	
    if (op == MENU_NOTIFY) show_about_window();
    return item;
}

/*
 * Menu handler for `display_menu (Until/Comment)'.
 */
Menu_item
disp_text_notify(item, op)
Menu_item	item;
Menu_generate	op;
{
    if (op == MENU_NOTIFY && display_mode == DISP_DATES) {
	display_mode = DISP_TEXT;
	update_screen();
    }
    return item;
}

/*
 * Menu handler for `display_menu (Modification Times)'.
 */
Menu_item
disp_times_notify(item, op)
Menu_item	item;
Menu_generate	op;
{
    if (op == MENU_NOTIFY && display_mode == DISP_TEXT) {
	display_mode = DISP_DATES;
	update_screen();
    }
    return item;
}

/*
 * Menu handler for `watch_menu (Add to Watch List)'.
 */
Menu_item
watch_add_handler(item, op)
Menu_item	item;
Menu_generate	op;
{
    if (op == MENU_NOTIFY) {
	char *u;

	if (cur_sel == BOGUS) return item;
	u = ulist[cur_sel].logid;
	if (watchnum == 0) {
	    watchnum = 1;
	    watchlist = (char **)malloc(sizeof(char *));
	    watchlist[0] = (char *)malloc(strlen(u)+1);
	    strcpy(watchlist[0], u);
	    pr_msg(NULL, FALSE, "%s added to watch list.",
		get_fname(ulist[cur_sel].logid));
	}
	else {
	    if ( (char *)lfind((char *)&u, (char *)watchlist, &watchnum,
		sizeof(char *), comp_strings) != CNULL) {
		pr_msg(NULL, TRUE, "%s already on watch list.",
		    get_fname(ulist[cur_sel].logid));
	    }
	    else {
		watchnum++;
		watchlist = (char **)realloc((char *)watchlist,
		    sizeof(char *)*watchnum);
		watchlist[watchnum - 1] = (char *)malloc(strlen(u)+1);
		strcpy(watchlist[watchnum - 1], u);
		pr_msg(NULL, FALSE, "%s added to watch list.",
		    get_fname(ulist[cur_sel].logid));
	    }
	}
    }
    return item;
}

/*
 * Menu handler for `watch_menu (Remove from Watch List)'.
 */
Menu_item
watch_remove_handler(item, op)
Menu_item	item;
Menu_generate	op;
{
    if (op == MENU_NOTIFY) {
	if (cur_sel == BOGUS) return item;
	if (watch_del(ulist[cur_sel].logid))
	    pr_msg(NULL, FALSE, "%s deleted from watch list.",
		get_fname(ulist[cur_sel].logid));
	else
	    pr_msg(NULL, TRUE, "%s not on watch list.",
		get_fname(ulist[cur_sel].logid));
    }
    return item;
}

/*
 * Event callback function for `frame'.
 */
Notify_value
main_frame_event(frame, event, arg, type)
Frame		frame;
Event		*event;
Notify_arg	arg;
Notify_event_type type;
{
    int was_closed, is_closed;
    Notify_value value;
    
    was_closed = (int)xv_get(frame, FRAME_CLOSED);
    value = notify_next_event_func(frame, (Notify_event) event, arg, type);
    is_closed = (int)xv_get(frame, FRAME_CLOSED);

    if (was_closed != is_closed) {
	if (!is_closed)
	    set_timer(TRUE);
	else if (watchnum == 0)
	    set_timer(FALSE);
    }

    return value;
}

/*
 * Notify callback function for `props_item'.
 */
void
main_props_notify(item, event)
Panel_item	item;
Event		*event;
{
    clear_msg();	
    show_props_window();
}

/*
 * Notify callback function for `sendmsg_item'.
 */
void
main_sendmsg_notify(item, event)
Panel_item	item;
Event		*event;
{
    clear_msg();
    show_sendmsg_window();
}

/*
 * Notify callback function for `scanmsg_item'.
 */
void
main_scanmsg_notify(item, event)
Panel_item	item;
Event		*event;
{
    clear_msg();
    show_scan_window();
}

/*
 * Notify callback function for `list_item'.
 */
int
main_list_notify(item, string, client_data, op, event)
Panel_item	item;
char		*string;
Xv_opaque	client_data;
Panel_list_op	op;
Event		*event;
{
    switch (op) {
    /*
     *  For either SELECT or DESELECT, set pending_sel to the index of the
     *  ulist record for the row they just touched, which will be used in
     *  another routine when a mouse up-event occurs.
     */
    case PANEL_LIST_OP_DESELECT:
    case PANEL_LIST_OP_SELECT:
	pending_sel = (int)client_data;
	break;
    }
    return XV_OK;
}

/*
 *  Menu handler for `view_menu'.
 */

Menu
view_menu_handler(menu, op)
Menu		menu;
Menu_generate	op;
{
    static int status_menu_prepped = FALSE;

    /*
     *  cat_menu is NULL on program startup; it is also set to NULL
     *  whenever the cats[] array is modified.
     */	
    if (op == MENU_DISPLAY && cat_menu == (Menu)NULL ) {
	Menu_item mi;
	Menu status_menu;
	int i;

	cat_menu = xv_create(NULL, MENU_CHOICE_MENU,
	    MENU_STRINGS,		"(Next)", "All", NULL,
	    MENU_NOTIFY_PROC,		cat_menu_notify,
	    MENU_GEN_PIN_WINDOW,	base_frame, "Categories",
	    NULL);

	for (i = 0 ; i < numcats ; i++ ) {
	    mi = (Menu_item)xv_create(NULL, MENUITEM,
		MENU_STRING,	cats[i],
		MENU_RELEASE,
		NULL);
	    xv_set(cat_menu, MENU_APPEND_ITEM, mi, NULL);
	}

	/*
	 *  Get the handle to what better be the "Category" item on the
	 *  View menu and attach the menu we just created.
	 */
	mi = (Menu_item)xv_get(menu, MENU_NTH_ITEM, VIEW_CAT_POSITION);
	xv_set(mi, MENU_PULLRIGHT, cat_menu, NULL);

	/*
	 *  Get the handle to what we hope is the "Status" item and
	 *  establish its menu's notify proc.
	 */
	if ( !status_menu_prepped) {
	    mi = (Menu_item)xv_get(menu, MENU_NTH_ITEM, VIEW_STATUS_POSITION);
	    status_menu = (Menu)xv_get(mi, MENU_PULLRIGHT);
	    xv_set(status_menu, MENU_NOTIFY_PROC, status_menu_notify, NULL);
	    status_menu_prepped = TRUE;
	}
    }
    return menu;
}

/*
 * Menu handler for `view_menu (Page Log...)'.
 */
Menu_item
page_log_handler(item, op)
Menu_item	item;
Menu_generate	op;
{
    if (op == MENU_NOTIFY) show_log_window();
    return item;
}

/*
 *  Event callback function for `display_panel'.
 *  Traps for ASCII keystrokes, which we use to select items on the
 *  scrolling list by the first character of their last name.
 */
Notify_value
main_display_panel_event(win, event, arg, type)
Xv_window	win;
Event		*event;
Notify_arg	arg;
Notify_event_type type;
{
    if (event_is_down(event) && isascii(event_action(event)) &&
	isalpha(event_action(event))) {
	int c, ri, failed;

	c = tolower(event_action(event));
	failed = FALSE;
	for (ri = (cur_sel == BOGUS ? 0 : cur_sel + 1); ri < users; ri++)
	    if (ulist[ri].sortby[0] == c && ulist[ri].row != BOGUS)
		break;

	if (ri == users) {
	    if ( cur_sel != BOGUS ) {
		for (ri = 0; ri <= cur_sel; ri ++)
		    if (ulist[ri].sortby[0] == c && ulist[ri].row != BOGUS)
			break;
		if ( ri > cur_sel ) failed = TRUE;
	    }
	    else
		failed = TRUE;
	}
	
	if (failed)
	    window_bell(base_frame);
	else {
	    cur_sel = ri;
	    /*
	     *  Make sure list is showing the proper selection.
	     */
	    if (!(int)xv_get(main_ip->list_item, PANEL_LIST_SELECTED,
		ulist[cur_sel].row))
		xv_set(main_ip->list_item,
		    PANEL_LIST_SELECT, ulist[cur_sel].row, TRUE, NULL);
	}
	set_mainwin_items_status();
    }
    else
	return notify_next_event_func(win, (Notify_event) event, arg, type);
}

/*
 * Notify callback function for `page_item'.
 */
void
main_page_notify(item, event)
Panel_item	item;
Event		*event;
{	
    clear_msg();
    show_page_window();
}

/*
 *  We "update" the category list (actually the menu built from it) by
 *  destroying said menu; this will cause it to be re-created the next
 *  time its parent menu is accessed.
 */
void
update_cat_list()
{
    if (cat_menu != (Menu)NULL) {
	xv_destroy(cat_menu);
	cat_menu = (Menu)NULL;
    }
}

/*
 *  Gathers all the parameters for this window
 */
void
get_main_params(p)
struct winparams *p;
{
    p->state = FRAME_CMD_PIN_IN;	/* Main window is always created */
    frame_get_rect(main_ip->frame, &(p->rect));
}

/*
 *  Wrapper to allow routines outside of mainwin to redo the outs 
 *  menu
 */
void
redo_outs_menu()
{
    Menu omenu;
    
    if ( (omenu = (Menu)xv_get(main_ip->outs_item, PANEL_ITEM_MENU)) != NULL)
	xv_destroy(omenu);

    outs_menu_create(main_ip->outs_item);
    if (msl_outs_mi != (Menu_item)NULL) {
	omenu = (Menu)xv_get(main_ip->outs_item, PANEL_ITEM_MENU);
	xv_set(msl_outs_mi, MENU_PULLRIGHT, omenu, NULL);
    }
}
