/*
 *  Utility routines common to all versions of the Message Board
 */
#include <stdio.h>
#include <ctype.h>
#include <pwd.h>
#include <grp.h>
#include <string.h>
#include "msgboard.h"

/*
 *  Used by lsearch/lfind to do lookups in ulist by login ID
 */
int
comp_by_logid(i, j)
Mem_Rec *i, *j;
{
    return strcmp(i->logid, j->logid);
}

/*
 *  Returns pointer to a record in the list, keyed on login ID, or NULL if
 *  not found. 
 */
Mem_Rec *
find_urec(id)
char *id;
{
    Mem_Rec temp;

    if (strlen(id) == 0)
	return (Mem_Rec *)NULL;
    else {
	strncpy(temp.logid, id, LOGID_LEN);
	temp.logid[LOGID_LEN] = '\0';
	return (Mem_Rec *)lfind((char *)&temp,
	    (char *)ulist, &users, sizeof(Mem_Rec), comp_by_logid);
    }
}

/*
 *  Gets the user's full name.
 *
 *  If MB_G_LASTNM1ST (last name first) is set, make a stab at reversing the
 *  name order...
 */
char *
get_fname(id)
{
    Mem_Rec *mp;
    static char result[FNAME_LEN + 1];
    char *comma;

    if ( ( mp = find_urec(id) ) == (Mem_Rec *)NULL)
	strcpy(result, id);
    else
	strcpy(result, mp->fname);

    if ((globrec.flags & MB_G_LASTNM1ST) != 0 &&
	(comma = strrchr(result, ',')) != CNULL) {
	char buf[FNAME_LEN + 1], *ptr;
	int len;

	if (*(comma + 1) == ' ')
	    strcpy(buf, comma + 2);
	else
	    strcpy(buf, comma + 1);
	strncat(buf, " ", FNAME_LEN - strlen(buf));
	for (ptr = result, len = strlen(buf) ;
	    ptr != comma && len < FNAME_LEN ; ptr++, len++)
	    buf[len] = *ptr;
	buf[len] = '\0';
	strcpy(result, buf);
    }

    return result;
}

/*
 *  looks up "logname" in the passwd file/map; fills in the full name,
 *  and if passed a non-null pointer, the category name.
 *  Returned value is 0 if user lookup is successful, -1 if it fails.
 */
int
lookup_user(logname, fullname, cat)
char *logname, *fullname, *cat;
{
    struct passwd *pwbuf;
    struct group *grpbuf;
    int l;

    pwbuf = getpwnam(logname);
    if ( pwbuf == (struct passwd *)NULL) return -1;

    /*
     *  Read in the full name from the GECOS field, accepting a maximum of
     *  20 characters, up to but not including the first character that is
     *  not alpha, blank or a period.  Then, trim trailing blanks.
     *  WARNING: the constant "20" in the below format should be equal to
     *  FNAME_LEN; be sure to change it if you change FNAME_LEN's value!
     				  vv	*/
    if (sscanf(pwbuf->pw_gecos, "%20[A-Za-z .]%n", fullname, &l) == 1)
	while ( fullname[l-1] == ' ')
	    fullname[--l] = '\0';
    else
	fullname[0] = '\0';

    /*
     *  If they want it, get the category name.
     */
    if (cat != (char *)NULL) {
	if ( (grpbuf = getgrgid(pwbuf->pw_gid)) == (struct group *)NULL)
	    cat[0] = '\0';
	else
	    strncpy(cat, grpbuf->gr_name, CAT_LEN+1);
    }
    return 0;
}

/*
 *  We want to sort by last name, and a good guess on what their last name
 *  is might be "everything past the last space in the full name field."
 */
void
make_sort_string(fname, sortby)
char *fname, *sortby;
{
    char *pos;

    if ((globrec.flags & MB_G_LASTNM1ST) == 0) {
	if ( (pos = strrchr(fname, ' ')) == CNULL) 
	    pos = fname;
	else
	    ++pos;

	strcpy(sortby, pos);
    }
    else {
	strcpy(sortby, fname);
	if ( (pos = strchr(sortby, ',')) != CNULL) 
	    *pos = '\0';
    }

    for (pos = sortby; *pos != '\0'; pos++)
	if (isupper(*pos)) *pos = tolower(*pos);
}

/*
 *  Compare user list items by their sort field
 */
int
comp_by_sortby(i, j)
Mem_Rec *i, *j;
{
    return strcmp(i->sortby, j->sortby);
}

/*
 *  Compare strings called by pointer-to-pointer; used to sort category
 *  list, and names list in add/modify window.
 */
int
comp_strings(i, j)
char **i, **j;
{
    return(strcmp(*i, *j));
}
