/*
 *  Site-definable constants.
 */

/*
 *  Default interval between each database check.
 */
#define	CK_INT		30	

/*
 *  Default name of the machine running the rpc.msgd server.  User
 *  can override this with MB_SERVER envariable.
 */
/* #define MB_SERVER	"msghost" */
#define MB_SERVER	"baaa"

/*
 *  Where the server should look for the data file, error log and pmsg
 *  directory.
 */
#define MB_TOPDIR	"/var/spool/msgboard"

/*
 *  Uncomment this if you want everyone to be able to see everyone else's
 *  phone messages.  Used by the server; clients check the appropriate
 *  flag in the global record passed to them by the server.
 */
/* #define TRUSTED_OFFICE */

/*
 *  Minimum and maximum UID to consider as "real" users in passwd list
 */
#define MIN_UID		1001
#define MAX_UID		65533

/*
 *  Aging information, used by "nightly": how many days should a message
 *  stick around before being moved to the next lower category, and how
 *  long until it's deleted from trash?  Note that as delivered, things
 *  will never just disappear.  You might want to modify this by setting
 *  TO_COMPLETE_DAYS and TO_TRASH_DAYS to fairly high values.
 */
#define TO_PENDING_DAYS		14
#define TO_COMPLETE_DAYS	 0	/* Never auto-move from pending */
#define TO_TRASH_DAYS		 0	/* Never auto-move from complete*/
#define DEL_DAYS		21

/*
 *  Beeper defines, used by rpc.msgd.
 */
#define  DIALHOST	"suntan"	/* hostname of machine with */
					/* dialer program	    */
#define	 CU		"/usr/local/bin/cu"
#define  PAUSE		",,,,,"
#define  SKYPAGE_PHONE	"1-800-555-1212"
#define  MESSAGE	"555*1212"

/*
 *  Comment this out if you want to disable programmatic control of panel
 *  item positioning, defaulting to the DevGuide-defined coordinates; doing
 *  this is probably only useful for layout debugging.  I haven't checked
 *  to make sure this works, but code enabled by SCALABLE may also be
 *  obsoleted by devGuide 3.0.
 */
#define SCALABLE

/*
 *  System mail delivery program.  The code expects a Berkeley-type
 *  mail program that groks tilde escapes, so you probably don't want
 *  to change this.
 */
#define MAILER		"/usr/ucb/Mail"

/*
 *  Authorization levels required for the different RPC services
 *  provided by rpc.msgd.  Server currently supports AUTH_NONE,
 *  AUTH_UNIX and AUTH_DES.  Clients do not use these constants directly;
 *  authorization levels are placed in struct srvinfo by connect_to_server().
 */

/*
 *  Authentication for GET_REC, GET_MOD_REC
 */
#define MB_GET_AUTH	AUTH_NONE

/*
 *  Authentication for PUT_REC
 */
#define MB_PUT_AUTH	AUTH_DES

/*
 *  Authentication for SEND_PAGE
 */
#define MB_PAGE_AUTH	AUTH_DES

/*
 *  Authentication for SEND_PMSG
 */
#define MB_SNDPMSG_AUTH	AUTH_DES

/*
 *  Authentication for SCAN_PMSG and friends
 */
#define MB_GETPMSG_AUTH	AUTH_DES

/*
 *  Default parameter storage file
 */
#define PARAMFILE	".mb2rc"

/*
 * Nothing below here should need to be changed unless you're mucking
 * with the code.
 ***********************************************************************
 */

#define CLASS_NAME	"MsgBoard"

/*
 *  Choices for sendmsg window
 */
#define SM_CHOICE_DEFAULT	1  /*  Default choice for sendmsg (2^0)*/
#define SM_CHOICE_LV_MSG	16 /*  Choice that opens text subwin (2^4)*/
#define SM_MAX_CHOICE		4  /*  Highest choice available (0-based) */

/*
 *  Choices for scan message window
 */
#define SCM_CHOICE_DEFAULT	0
#define SCM_CHOICE_NEW		0
#define SCM_CHOICE_PEN		1
#define SCM_CHOICE_COM		2
#define SCM_CHOICE_TRA		3
#define SCM_MAX_CHOICE		3

/*
 *  Strings lengths for various database fields
 */
#define LOGID_LEN	8
#define FNAME_LEN	20
#define UNTIL_LEN	15
#define COMMENT_LEN	40
#define MOTD_LEN	64
#define SITE_LEN	40
#define CAT_LEN		10
#define SORTBY_LEN	FNAME_LEN
#define EXT_LEN		8
#define PAGERNUM_LEN	13

/*
 *  String lengths for pmsg fields
 */
#define TO_LEN		64
#define TIME_LEN	16
#define CALLER_LEN	50
#define ORG_LEN		50
#define PHNUM_LEN	50

/*
 *  Printing stuff
 */
#define PRINTCMD	"/usr/ucb/lpr"
#define PRINTERNAME_LEN	20
#define TO_PRINTER	0
#define TO_FILE		1
#define POSTSCRIPT	0
#define ASCII		1

/*
 *  Add/Modify window mode.
 */
#define IDLE		0
#define ADDING		1
#define MODIFYING	3

/*
 *  Display modes for the main window
 */
#define DISP_TEXT	0
#define DISP_DATES	1

/*
 *  Value representing bogosity
 */
#define BOGUS		-1

/*
 *  The three modes for an entry
 */
#define MB_OUT		0
#define MB_IN		1
#define MB_UNK		2

/*
 *  Value representing "All" -- used for categories and status
 */
#define MB_ALL		-1

/*
 *  Beeper types
 */
#define	BEEP_ALPHA	0
#define BEEP_NUM	1
#define BEEP_SKY	2
#define BEEP_UNK	3

/*
 *  Masks for Mem_Rec.flags:
 *	MB_CHG_PLAN:	Should msgboard automagically update .plan for you?
 *	MB_ADMIN:	Can you use msgboard's administrative functions?
 *			(only settable by users with this flag already set)
 *	MB_PRESERVE:	Should the comment field NOT be zeroed when moving
 *				from Out to In?
 *	MB_VIA_MAIL:	Should pmsgs be sent to you via e-mail?
 *	MB_VIA_MB:	Should pmsgs be saved/accessible within msgboard?
 *	MB_NEWMSGS:	Do you have unread messages? (read-only)
 */
#define MB_CHG_PLAN	0x01
#define MB_ADMIN	0x02
#define MB_PRESERVE	0x04
#define MB_VIA_MAIL	0x08
#define MB_VIA_MB	0x10
#define MB_NEWMSGS	0x20

/*
 *  Masks for global flags:
 *	MB_G_CHG_PLAN:	Default for MB_CHG_PLAN field; see above
 *	MB_G_VIA_MAIL:	Default for MB_VIA_MAIL
 *	MB_G_PRESERVE:	Default for MB_PRESERVE
 *	MB_G_VIA_MB:	Default for MB_VIA_MB
 *	MB_G_LASTNM1ST:	Format for full names: if TRUE, last name first;
 *			 if FALSE, first name first.
 *	MB_G_TRUSTED:	If TRUE, server is running in TRUSTED_OFFICE mode.
 *			Note that this is a READ-ONLY variable as far as
 *			clients are concerned.
 */
#define MB_G_CHG_PLAN	0x01
#define MB_G_PRESERVE	0x04
#define MB_G_VIA_MAIL	0x08
#define MB_G_VIA_MB	0x10
#define MB_G_LASTNM1ST	0x40
#define MB_G_TRUSTED	0x80

/*
 *  Status flags returned by the server.
 */

#define MSGD_OK			0
#define MSGD_DISCONNECTED	1	/* Not connected to server */
#define MSGD_RPC_FAIL		2	/*  Pretty vague, eh? */
#define MSGD_VALID_FAIL		3	/*  Security validation error */
#define MSGD_NOT_FOUND		4	/*  Record not found */
#define MSGD_ALREADY_EXISTS	5	/*  Record already exists */
#define MSGD_ERROR		6	/*  General failure */

/*
 *  Width of the In/Out/??? glyph
 */
#define INOUT_WIDTH	64

#ifndef	TRUE
#define TRUE	(1)
#define FALSE	(0)
#endif

#define CNULL	((char *)0)
#define FNULL	((FILE *)NULL)
#define ITIMER_NULL ((struct itimerval *)0)

/*
 *  Structure for user records
 */
typedef struct mem_rec {
    char	logid[LOGID_LEN+1],		/* UNIX login name	*/
		fname[FNAME_LEN+1],		/* Full name		*/
		until[UNTIL_LEN+1],		/* "Out until" field	*/
		comment[COMMENT_LEN+1],		/* Message field	*/
		cat[CAT_LEN+1],			/* Category		*/
		sortby[SORTBY_LEN+1],		/* Sort key		*/
		ext[EXT_LEN+1],			/* Phone extension	*/
		pagernum[PAGERNUM_LEN+1];	/* Pager phone number	*/
    int		row,				/* on-screen row # 	*/
		status,				/* In, Out or ???	*/
		flags,				/* Boolean flags:
							MB_CHG_PLAN
							MB_ADMIN
							MB_PRESERVE
							MB_VIA_MAIL
							MB_VIA_MB	*/
		pagertype;			/* Pager type		*/
    time_t	timestamp;			/* Record mod time	*/
} Mem_Rec;

int comp_by_logid(), comp_by_sortby(), comp_strings();

char *getenv(), *get_fname();

Mem_Rec *find_urec();

typedef struct global_rec {
    char	name[SITE_LEN + 1],		/* Name of the server's site */
		motd[MOTD_LEN + 1];		/* Message of the day	     */
    int		flags,				/* Global flags		     */
		pagertype;			/* Default pager type	     */
} Global_Rec;

extern char		**cats, mylogid[], mb_server[];
extern int		users, admin, delay, numcats, cur_cat,
			cur_status, admin_enable;
extern Mem_Rec		*ulist, *myentry;
extern Global_Rec	globrec;
