/*
 *  nightly -- housekeeping program for the Message Board
 *	Performs the following tasks:
 *		1.  Finds anyone still marked as "In" and changes their
 *		    status to "Unknown"
 *		2.  Ages phone messages
 */

#include <sys/param.h>
#include <stdio.h>
#include <time.h>
#include "msgboard.h"
#include "pmsg_client.h"

#define DAYSEC	86400L		/*  Number of seconds in a day		*/

Mem_Rec
    *ulist,			/*  Pointer to list of users		*/
    *myentry,			/*  Pointer to current user's record in
				    ulist array				*/
    tmp_record;  		/*  Used when creating/modifying records */

Global_Rec
   globrec;

char
    mb_server[MAXHOSTNAMELEN+1],
    motd[MOTD_LEN+1],		/* Message of the day			*/
    mylogid[L_cuserid],		/* Current user's login ID		*/
    sitename[SITE_LEN+1],	/* Name of this site 			*/
    **cats;			/* Pointer to array of category names	*/

int 
    global_flags,			/* global flags	*/
    users = 0,				/* Number of users in datalist	*/
    numcats = 0,			/* Number of categories		*/
    cur_cat = MB_ALL,			/* Category to restrict display to */
    cur_status = MB_ALL,		/* Category to restrict display to */
    admin = FALSE,			/* Is user in admin mode?	*/
    admin_enable = TRUE;

void
kaboom()
{
    exit(1);
}

void
pr_msg(ignore, bell, format, a1, a2, a3)
int bell;
char *ignore, *format, *a1, *a2, *a3;
{
    char bellchar;

    if (bell) fputc('\007', stderr);
    fprintf(stderr, format, a1, a2, a3);
}

/*
 *  Moves messages older than "date" from from_cat to to_cat.  If to_cat is
 *  BOGUS, mv_msg_call will just throw the messages out.
 */
void
age_msgs(user, date, from_cat, to_cat)
char *user;
int from_cat, to_cat;
time_t date;
{
    char buf[BUFSIZ], **m_list;
    int dp, size, r_count, m_count, i;
    struct tm tmbuf;
    time_t msgdate;
    struct pmsg_hdr *ph_ptr;

    ph_ptr = get_pmsg_headers(user, from_cat, &size);
    if (size == 0) return;

    r_count = 0;
    for (i = 0; i < size; i++) {
	sscanf(ph_ptr[i].id, "%*s %*s %n", &dp);
	strptime(ph_ptr[i].id + dp, "%a %b %d %T %Y", &tmbuf);
	msgdate = timelocal(&tmbuf);
	if (msgdate < date) {
	    r_count++;
	    if (r_count == 1)
		m_list = (char **)malloc(sizeof(char *));
	    else
		m_list = (char **)realloc(m_list, sizeof(char *) * r_count);
	    m_list[r_count - 1] = ph_ptr[i].id;
	}
	else
	    free(ph_ptr[i].id);
	free(ph_ptr[i].desc);
    }

    if (r_count > 0) {
	m_count = mv_msg_call(user, from_cat, to_cat, r_count, m_list);
	for ( i = 0; i < r_count; i++)
	    free(m_list[i]);
	free(m_list);
	if (m_count != r_count)
	    fprintf(stderr,
	"Warning: tried to move %d msgs from %s.%d; actually moved %d.\n", 
		r_count, user, from_cat, m_count);
    }
}

main(argc, argv)
int argc;
char **argv;
{
    int u;
    time_t now;

    if (cuserid(mylogid) == CNULL) {
	pr_msg(NULL, TRUE, "System doesn't know who %s is! (cuserid failed)",
	    mylogid);
	kaboom();
    }

    if (!connect_to_server()) {
	pr_msg(NULL, TRUE, "Could not connect to server %s.", mb_server);
	exit(1);
    }

    (void)get_modified_records();

    time(&now);

    for ( u = 0; u < users; u++) {
	if ( ulist[u].status == MB_IN)
	    modify_status(ulist[u].logid, MB_UNK, CNULL, CNULL);

	/*
	 *  delete old msgs from trash:
	 */
	if (DEL_DAYS > 0)
	    age_msgs(ulist[u].logid, now - DAYSEC * DEL_DAYS,
		SCM_CHOICE_TRA, BOGUS);

	/*
	 *  Move complete to trash:
	 */
	if (TO_TRASH_DAYS > 0)
	    age_msgs(ulist[u].logid, now - DAYSEC * TO_TRASH_DAYS,
		SCM_CHOICE_COM, SCM_CHOICE_TRA);

	/*
	 *  Move pending to complete:
	 */
	if (TO_COMPLETE_DAYS > 0)
	    age_msgs(ulist[u].logid, now - DAYSEC * TO_COMPLETE_DAYS,
		SCM_CHOICE_PEN, SCM_CHOICE_COM);

	/*
	 *  Move new to pending:
	 */
	if (TO_PENDING_DAYS > 0)
	    age_msgs(ulist[u].logid, now - DAYSEC * TO_PENDING_DAYS,
		SCM_CHOICE_NEW, SCM_CHOICE_PEN);
    }
}

void
update_screen()
{
}

void
set_mainwin_items_status()
{
}

void
update_line(rp)
Mem_Rec *rp;
{
}

void
update_global_display()
{
}

void
update_cat_list()
{
}

void
watch_del()
{
}

void
watch_alert()
{
}

void
server_was_dropped()
{
}
