/*
 * outmodwin.c - Notify and event callback function stubs for the abbreviated
 *  modify window that's popped up when a user is marked Out.
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include "outmodwin_ui.h"
#include "msgboard.h"
#include "msgboard_xv.h"


/*
 * Global object definitions.
 */
outmodwin_frame_objects	*om_ip;

int
    outmod_window_created = FALSE,
    om_warped = FALSE;

void
create_out_window()
{
#ifdef SCALABLE
    int valx, ypos;
#endif

    om_ip = outmodwin_frame_objects_initialize(NULL, base_frame);

    if (outmod_params.state != BOGUS) {
        frame_set_rect(om_ip->frame, &outmod_params.rect);
        if (outmod_params.state == FRAME_CMD_PIN_IN)
	    xv_set(om_ip->frame, FRAME_CMD_PIN_STATE, FRAME_CMD_PIN_IN, NULL);
    }

    xv_set(om_ip->panel,
	PANEL_DEFAULT_ITEM,     om_ip->apply_item,
	PANEL_CLIENT_DATA,	CNULL,
	NULL);

#ifdef SCALABLE
    /*
     *  comment has the longest label, so we position it properly from the
     *  left edge of the panel.
     */
    xv_set(om_ip->comment_item, XV_X, wgrid * 2, NULL);
    valx = (int)xv_get(om_ip->comment_item, PANEL_VALUE_X);

    ypos = wgrid * 2;
    align_values(om_ip->status_item, valx, ypos);

    ypos += wgrid + (int)xv_get(om_ip->status_item, XV_HEIGHT);
    align_values(om_ip->until_item, valx, ypos);

    ypos += wgrid + (int)xv_get(om_ip->until_item, XV_HEIGHT);
    xv_set(om_ip->comment_item, XV_Y, ypos, NULL);

    ypos += wgrid + (int)xv_get(om_ip->comment_item, XV_HEIGHT);
    xv_set(om_ip->apply_item, XV_Y, ypos, NULL);
    xv_set(om_ip->reset_item, XV_Y, ypos, NULL);

    window_fit(om_ip->panel);    
    window_fit(om_ip->frame);

    position_bot_buttons(om_ip->panel, om_ip->apply_item, om_ip->reset_item);
#endif
    outmod_window_created = TRUE;
}

void
show_outmod_window(user)
Mem_Rec *user;
{
    char buf[FNAME_LEN + 25], *old_logid;
    int no_rec_passed;
    Panel_item focus_item;

    if (!outmod_window_created) create_out_window();

    no_rec_passed = (user == (Mem_Rec *)NULL);
    if (no_rec_passed) {
	user = (Mem_Rec *)malloc(sizeof(Mem_Rec));
	user->status = MB_UNK;
	user->logid[0] = user->until[0] = user->comment[0] = '\0';
    }
	
    xv_set(om_ip->status_item,
	PANEL_VALUE,	user->status,
	PANEL_INACTIVE, no_rec_passed,
	NULL);
    xv_set(om_ip->until_item,
	PANEL_VALUE,	user->until,
	PANEL_INACTIVE, no_rec_passed,
	NULL);
    xv_set(om_ip->comment_item,
	PANEL_VALUE, user->comment,
	PANEL_INACTIVE, no_rec_passed,
	NULL);

    xv_set(om_ip->apply_item, PANEL_INACTIVE, no_rec_passed, NULL);
    xv_set(om_ip->reset_item, PANEL_INACTIVE, no_rec_passed, NULL);

    old_logid = (char *)xv_get(om_ip->panel, PANEL_CLIENT_DATA);
    if (old_logid != CNULL) {
	free(old_logid);
	xv_set(om_ip->panel, PANEL_CLIENT_DATA, NULL, NULL);
    }

    sprintf(buf, "Message Board: Mark \"%s\" Out", user->fname);

    if (no_rec_passed)
	free(user);
    else {
	xv_set(om_ip->comment_item,
	    PANEL_CLIENT_DATA, (user->flags & MB_PRESERVE) != 0, NULL);

	focus_item = (user->until[0] != '\0' && user->comment[0] == '\0') ?
	    om_ip->comment_item :
	    om_ip->until_item ;

	xv_set(om_ip->panel,
	    PANEL_CLIENT_DATA,	strdup(user->logid),
	    PANEL_CARET_ITEM,	focus_item,
	    NULL);
    }
    om_warped = FALSE;
    xv_set(om_ip->frame,
	FRAME_LABEL,	buf,
	XV_SHOW,	TRUE,
	NULL);
}

void
outmod_done(flag)
int flag;
{
    char *logid;

    logid = (char *)xv_get(om_ip->panel, PANEL_CLIENT_DATA);
    if (flag) {
	modify_status(
	    logid,
	    (int)xv_get(om_ip->status_item, PANEL_VALUE),
	    (char *)xv_get(om_ip->until_item, PANEL_VALUE),
	    (char *)xv_get(om_ip->comment_item, PANEL_VALUE)
	);
	(void)get_modified_records();
    }

    if ((int)xv_get(om_ip->frame, FRAME_CMD_PIN_STATE) == FRAME_CMD_PIN_IN)
	show_outmod_window((Mem_Rec *)NULL);
    else
	clear_msg();
}

/*
 * Notify callback function for `status_item'.
 */
void
out_inout_notify(item, value, event)
Panel_item	item;
int		value;
Event		*event;
{
    if (value == MB_IN) {
	xv_set(om_ip->until_item, PANEL_VALUE, "", NULL);
	if (!(int)xv_get(om_ip->comment_item, PANEL_CLIENT_DATA))
	    xv_set(om_ip->comment_item, PANEL_VALUE, "", NULL);
    }
}

/*
 * Notify callback function for `comment_item'.
 */
Panel_setting
out_comment_notify(item, event)
Panel_item      item;
Event           *event;
{
    if (event_shift_is_down(event))
        return PANEL_PREVIOUS;
    else {
        outmod_done(TRUE);
        if ((int)xv_get(om_ip->frame, FRAME_CMD_PIN_STATE) == FRAME_CMD_PIN_OUT)
	    xv_set(om_ip->frame, XV_SHOW, FALSE, NULL);
	return PANEL_NONE;
    }
}

/*
 * Notify callback function for `apply_item' and `reset_item'.
 */
void
out_apply_reset_notify(item, event)
Panel_item	item;
Event		*event;
{
    char *logid;  

    outmod_done(item == om_ip->apply_item);
}

/*
 * Event callback function for `panel'.
 */
Notify_value
om_panel_event(win, event, arg, type)
Xv_window	win;
Event		*event;
Notify_arg	arg;
Notify_event_type type;
{
    if (!om_warped && event_action(event) == WIN_REPAINT) {
        /*
         *  Conditionally warp the mouse to the apply button
         */
        warpme(om_ip->frame, om_ip->panel, om_ip->apply_item);
        om_warped = TRUE;
    }
    else if (event_action(event) == LOC_WINEXIT)
	/*
	 *  If we ever leave the window, cancel any pending dewarp
	 */
	no_dewarp();

    return notify_next_event_func(win, (Notify_event) event, arg, type);
}

/*
 * Done callback function for `frame'.
 */
void
om_done(frame)
Frame		frame;
{
    dewarp();
    xv_set(frame, XV_SHOW, FALSE, NULL);
}

/*
 *  Gathers all the parameters for this window
 */
void
get_outmod_params(p)
struct winparams *p;
{
    if (!outmod_window_created)
	*p = outmod_params;
    else {
	p->state = (int)xv_get(om_ip->frame, FRAME_CMD_PIN_STATE);
	frame_get_rect(om_ip->frame, &(p->rect));
    }
}
