/*
 * outs_menu.c - Code to create "Outs" menu.  Used by msgboard_x,
 * msgboard_sv, msglock_x, msglock_sv
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/time.h>
#include <strings.h>
#include <ctype.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include "outs_menu.h"
#include "msgboard.h"

struct outent *outlist;

int num_outs = 0;

/*
 *  Accepts a line and adds it to the array of entries for the "Outs" menu.
 */
void
read_outs_line(l)
char *l;
{
    char *p;
    
    if (num_outs == 0) {
	num_outs = 1;
	outlist = (struct outent *)malloc(sizeof(struct outent));
    }
    else {
        num_outs++;
        outlist = (struct outent *)realloc((char *)outlist,
	    sizeof(struct outent)*num_outs);
    }

    /*
     *  Scan past the keyword
     */
    (void)strtok(l, ": ");

    p = strtok(NULL, "|\n");
    while ( *p == ' ' || *p == '\t' ) p++;
    outlist[num_outs-1].label = strdup(p);
    outlist[num_outs-1].until = strdup(strtok(NULL, "|\n"));
    if ( strcmp(outlist[num_outs-1].until, " ") == 0)
	outlist[num_outs-1].until[0] = '\0';
    outlist[num_outs-1].message = strdup(strtok(NULL, "|\n"));
    if ( strcmp(outlist[num_outs-1].message, " ") == 0)
	outlist[num_outs-1].message[0] = '\0';
    p = strtok(NULL, "|\n");
    outlist[num_outs-1].pop = (p != (char *)NULL) ? atoi(p) : 0;
}

/*
 *  Called when user selects an item from the "Outs" menu
 */
void
outs_handler(menu, menu_item)
Menu menu;
Menu_item menu_item;
{
    char *menu_choice = (char *)xv_get(menu_item, MENU_STRING),
    	until[UNTIL_LEN+1], comment[COMMENT_LEN+1], ubuf[20], *p;
    struct outent *outp;
    int i;

    /*
     *  Find this menu choice in the list of outs options
     */
    outp = NULL;
    for ( i = 0 ; i < num_outs ; i++ )
        if ( strcmp(menu_choice, outlist[i].label) == 0 ) outp = &(outlist[i]);

    /*
     *  This should never happen
     */
    if ( outp == NULL ) return;

    strcpy(ubuf, outp->until);
    if (ubuf[0] == '+') {
	struct timeval	tp;
	struct tm	*tm;
	char tmp[4], *merid;
	int hr, mn;

	for ( i = 1, p = tmp ; isdigit(ubuf[i]) &&
		      i < strlen(ubuf) &&
		      p < tmp + 3 ; i++, p++)
	    *p = ubuf[i];

	*(p) = '\0';
	hr = atoi(tmp);
    
	if ( (p = strchr(ubuf, ':')) != CNULL ) {
	    for ( i = p - ubuf + 1, p = tmp ; isdigit(ubuf[i]) &&
			i < strlen(ubuf) &&
			p < tmp + 3 ; i++, p++)
		*p = ubuf[i];

	    *(p) = '\0';
	    mn = atoi(tmp);
	}
	else
	    mn = 0;

	gettimeofday(&tp, (struct timezone *)NULL);
	tp.tv_sec += hr * 3600 + mn * 60;
	
	tm = localtime(&tp.tv_sec);
	hr = tm->tm_hour;
	mn = tm->tm_min;

	if ((mn % 10) > 0) mn += (10 - (mn % 10)); /* Round up to nearest 10 */
	if (mn >= 60) {
	    mn -= 60;
	    ++hr;
	}

	if (hr > 23) hr -= 24;
    
	if (hr > 11) {
	    merid = "PM";
	    if (hr > 12) hr -= 12;
	}
	else {
	    merid = "AM";
	    if (hr == 0) hr = 12;
	}

	sprintf(ubuf, "%d:%02d %s", hr, mn, merid);
    }

    if ( (p = strchr(ubuf, '%')) != CNULL) {
	if ( *(p + 1) == 'N') {
	    struct timeval tp;
	    struct tm *tm;
	    int nextday, preserve;
	    char *p2;
	    
	    static char *weekday[7] = {
		"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
	    };

	    gettimeofday(&tp, (struct timezone *)NULL);
	    tm = localtime(&tp.tv_sec);
	    
	    nextday = tm->tm_wday + 1;
	    if (nextday > 5) nextday = 1;
	    
	    p2 = strlen(ubuf) == 19 ? ubuf + 18 : ubuf + strlen(ubuf) + 1;
	    while ( p2 > p + 1 ) {
	        *p2 = *(p2 - 1);
	        p2--;
	    }
	    *p = weekday[nextday][0];
	    *(++p) = weekday[nextday][1];
	    *(++p) = weekday[nextday][2];
	}
    }

    strncpy(until, ubuf, UNTIL_LEN);
    until[UNTIL_LEN] = '\0';
    strncpy(comment, outp->message, COMMENT_LEN+1);
    comment[COMMENT_LEN] = '\0';
    set_from_out_button(until, comment, outp->pop);
}

/*
 * Create object `outs_menu' 
 */
void
outs_menu_create(button)
Panel_item button;
{
    Menu menu;
    Menu_item mi;
    int i;

    if (num_outs == 0) {
        outlist = (struct outent *)malloc((unsigned)(sizeof(struct outent)*2));
        outlist[0].label = strdup("Lunch");
        outlist[0].until = strdup("+1:10");
        outlist[0].message = strdup("Lunch");
        outlist[0].pop = FALSE;
        outlist[1].label = strdup("For the Day");
        outlist[1].until = strdup("9 AM %N");
        outlist[1].message = strdup("");
        outlist[1].pop = FALSE;
        num_outs = 2;
    }

    menu = xv_create(XV_NULL, MENU_COMMAND_MENU,
	MENU_NOTIFY_PROC, outs_handler,
	NULL);

    for ( i = 0; i < num_outs; i++) {
        mi = xv_create(NULL, MENUITEM,
            MENU_STRING,	outlist[i].label,
            MENU_RELEASE,
            NULL);
	xv_set(menu, MENU_APPEND_ITEM, mi, NULL);
    }
    xv_set(button, PANEL_ITEM_MENU, menu, NULL);
}
