/*
 * pmsgwin.c - Routines to support the phone message display window.
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <X11/Xlib.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include "pmsgwin_ui.h"
#include "scanwin_ui.h"
#include "msgboard.h"
#include "msgboard_xv.h"
#include "pmsg_client.h"

/*
 * Global object definitions.
 */
pmsgwin_frame_objects	*smsg_ip;

extern char cur_msg_user[];
extern scanwin_frame_objects *scan_ip;
extern int msg_cat, cur_msg;
int
    smsg_window_created = FALSE;	/* is message window created?	*/

void
create_smsg_window()
{
#ifdef SCALABLE
    int valx, ypos;
#endif

    Rect scan_rect, smsg_rect;
    int disp_ht;

    smsg_ip = pmsgwin_frame_objects_initialize(NULL, base_frame);
    
#ifdef SCALABLE
    /*
     *  Use "signed" to set the position for all the other text items
     */
    xv_set(smsg_ip->signed_item, XV_X, wgrid * 2, NULL);
    valx = (int)xv_get(smsg_ip->signed_item, PANEL_VALUE_X);

    ypos = wgrid * 2;
    align_values(smsg_ip->caller_item, valx, ypos);

    ypos += wgrid + (int)xv_get(smsg_ip->caller_item, XV_HEIGHT);
    align_values(smsg_ip->of_item, valx, ypos);

    ypos += wgrid + (int)xv_get(smsg_ip->of_item, XV_HEIGHT);
    align_values(smsg_ip->phone_item, valx, ypos);

    xv_set(smsg_ip->flags_item,
	XV_X,	(int)xv_get(smsg_ip->phone_item, XV_X) +
		(int)xv_get(smsg_ip->phone_item, XV_WIDTH) + wgrid,
	XV_Y,	ypos,
	NULL);

    ypos += wgrid + (int)xv_get(smsg_ip->phone_item, XV_HEIGHT);
    align_values(smsg_ip->time_item, valx, ypos);
    
    ypos += wgrid * 2 + (int)xv_get(smsg_ip->time_item, XV_HEIGHT);
    xv_set(smsg_ip->signed_item, XV_Y, ypos, NULL);

    window_fit(smsg_ip->panel);

    xv_set(smsg_ip->textsw,
	XV_Y,		(int)xv_get(smsg_ip->panel, XV_HEIGHT),
	XV_WIDTH,	(int)xv_get(smsg_ip->panel,XV_WIDTH),
	NULL);

    xv_set(smsg_ip->frame,
	XV_HEIGHT, (int)xv_get(smsg_ip->panel, XV_HEIGHT) + wgrid * 6,
	NULL);

    window_fit_width(smsg_ip->frame);
#endif

    xv_set(smsg_ip->textsw,
	TEXTSW_BROWSING,	TRUE,
	TEXTSW_IGNORE_LIMIT,	TEXTSW_INFINITY,   /* Turn off quit warning  */
	TEXTSW_LOWER_CONTEXT,	-1,		   /* Turn off autoscroll    */
	TEXTSW_UPPER_CONTEXT, -1,
	TEXTSW_MEMORY_MAXIMUM,	100000,
	NULL);

    xv_set(smsg_ip->frame,
	FRAME_CMD_DEFAULT_PIN_STATE, FRAME_CMD_PIN_IN,
	NULL);

    /*
     *  Position frame immediately below scan window
     */
    disp_ht = DisplayHeight(dpy, DefaultScreen(dpy));
		
    frame_get_rect(scan_ip->frame, &scan_rect);
    frame_get_rect(smsg_ip->frame, &smsg_rect);
    smsg_rect.r_top = scan_rect.r_top + scan_rect.r_height;
    if ((smsg_rect.r_top + smsg_rect.r_height) > disp_ht)
	smsg_rect.r_top = disp_ht - smsg_rect.r_height;
    smsg_rect.r_left = scan_rect.r_left;
    frame_set_rect(smsg_ip->frame, &smsg_rect);
    
    smsg_window_created = TRUE;
}

/*
 *  Bring the message window up, showing the current message.
 *  Returns 0 if successful, error number if not.
 */
int
show_smsg_window()
{
    struct pmsg msg;
    int retval;
    char label[FNAME_LEN + 40];

    retval = get_pmsg(cur_msg_user, msg_cat,
	(char *)xv_get(scan_ip->list_item, PANEL_LIST_CLIENT_DATA,
	cur_msg), &msg);
    if (retval != MSGD_OK) return retval;

    if (!smsg_window_created) create_smsg_window();

    textsw_reset(smsg_ip->textsw, 0, 0);
    if ((msg.flags & SM_CHOICE_LV_MSG) != 0) {
	textsw_insert(smsg_ip->textsw, msg.text, strlen(msg.text));
	textsw_normalize_view(smsg_ip->textsw, 0);
    }

    xv_set(smsg_ip->signed_item, PANEL_VALUE, msg.id, NULL);
    xv_set(smsg_ip->caller_item, PANEL_VALUE, msg.caller, NULL);
    xv_set(smsg_ip->time_item, PANEL_VALUE, msg.time, NULL);
    xv_set(smsg_ip->of_item, PANEL_VALUE, msg.org, NULL);
    xv_set(smsg_ip->phone_item, PANEL_VALUE, msg.phnum, NULL);
    xv_set(smsg_ip->flags_item, PANEL_VALUE, msg.flags >> 1, NULL);
    free_pmsg(&msg);

    sprintf(label, "Message Board: Message %d for %s", cur_msg + 1,
	get_fname(cur_msg_user));
    xv_set(smsg_ip->frame,
	XV_SHOW,	TRUE,
	XV_LABEL,	label,
	NULL);
    return retval;
}

/*
 *  Returns true if showmsg window is created and currently visible
 */
int
smsg_is_up()
{
    return (smsg_window_created && (int)xv_get(smsg_ip->frame, XV_SHOW));
}

/*
 *  Clears the message display window
 */
void
clear_smsg_window()
{
    if (!smsg_window_created) return;
    
    xv_set(smsg_ip->caller_item, PANEL_VALUE, "", NULL);
    xv_set(smsg_ip->of_item, PANEL_VALUE, "", NULL);
    xv_set(smsg_ip->phone_item, PANEL_VALUE, "", NULL);
    xv_set(smsg_ip->time_item, PANEL_VALUE, "", NULL);
    xv_set(smsg_ip->signed_item, PANEL_VALUE, "", NULL);
    xv_set(smsg_ip->flags_item, PANEL_VALUE, 0, NULL);
    textsw_reset(smsg_ip->textsw, 0, 0);
    xv_set(smsg_ip->frame, FRAME_LABEL, "Message Board: No Message Selected",
	NULL);
}

void
dismiss_smsg_window()
{
    if (smsg_window_created)
	xv_set(smsg_ip->frame,
	    FRAME_CMD_PIN_STATE,	FRAME_CMD_PIN_OUT,
	    XV_SHOW,			FALSE,
	    NULL);
}

/*
 * Event callback function for `flags_item'.
 *  This function's purpose in life is to silently consume SELECT actions
 *  on the checkboxes, causing them to effectively be read-only.
 */
void
scm_flags_event(item, event)
Panel_item	item;
Event		*event;
{
	if (event_action(event) != ACTION_SELECT && event_action(event) !=
	    LOC_DRAG)
	    panel_default_handle_event(item, event);
}

/*
 * Done callback function for `frame'.
 */
void
pmsg_done_proc(frame)
Frame frame;
{
    pr_msg(scan_ip->frame, FALSE, "");
    xv_set(frame, XV_SHOW, FALSE, NULL);
}
