/*
 * propswin.c - Support for Message Board properties window.
 *   There are currently three property sheets defined: user, outs and
 *   global.  User contains general properties, outs allows definition of
 *   entries on the "Outs" button, and global defines global
 *   properties, which may only be altered by users in admin mode.
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/notice.h>
#include "propswin_ui.h"
#include "msgboard.h"
#include "msgboard_xv.h"
#include "outs_menu.h"

#define PROPS_CAT_GENERAL		0
#define PROPS_CAT_OUTS		1
#define PROPS_CAT_GLOBAL	2

#define ADD_BEFORE		43
#define	ADD_AFTER		44

propswin_frame_objects	*props_ip;

int
    props_window_created = FALSE,	/* is props window created?	*/
    outs_changes_pending = FALSE,
    temp_num_outs = 0;

struct outent *temp_outlist = NULL;

/*
 *  Configures the print-related items, based on current panel settings.
 */
void
set_prt_items(dest, type)
int dest, type;
{
    if (dest == TO_PRINTER) {
	xv_set(props_ip->prt_dir_item,
	    PANEL_LABEL_STRING,	"Printer:",
	    PANEL_VALUE,	prt_name,
	    NULL);
	xv_set(props_ip->prt_file_item,
	    XV_SHOW,		FALSE,
	    NULL);
    }
    else {
	xv_set(props_ip->prt_dir_item,
	    PANEL_LABEL_STRING,	"Directory:",
	    PANEL_VALUE,	prt_dir,
	    NULL);

	if ((type == POSTSCRIPT) && (strcmp(prt_file, "msgs.txt") == 0))
	    xv_set(props_ip->prt_file_item,
		PANEL_VALUE,	"msgs.ps",
		XV_SHOW,	TRUE,
		NULL);
	else if ((type == ASCII) && (strcmp(prt_file, "msgs.ps") == 0))
	    xv_set(props_ip->prt_file_item,
		PANEL_VALUE,	"msgs.txt",
		XV_SHOW,	TRUE,
		NULL);
	else
	    xv_set(props_ip->prt_file_item,
		PANEL_VALUE,	prt_file,
		XV_SHOW,	TRUE,
		NULL);
    }
}

/*
 *  Checks to see whether cat_item should be modified to have the "global"
 *  entry added or removed.
 */
void
modify_cat_item(needs_global)
int needs_global;
{
    static int has_global = BOGUS;  /* Initialized to neither true nor false */
        
    if (needs_global == has_global) return;
    
    if (has_global == BOGUS) {
	has_global = ((int)xv_get(props_ip->cat_item, PANEL_NCHOICES) >
	    PROPS_CAT_GLOBAL);
	if (needs_global == has_global) return;
    }

    if (needs_global)
	xv_set(props_ip->cat_item,
	    PANEL_CHOICE_STRING,	PROPS_CAT_GLOBAL, "Global",
	    NULL);
    else {
	xv_destroy(props_ip->cat_item);
	props_ip->cat_item =
	    propswin_frame_cat_item_create(props_ip, props_ip->cat_panel);
    }
    has_global = needs_global;
}

/*
 *  Exposes the proper panel for the properties category they've selected.
 */
void
show_props_cat(cat)
int cat;
{
    xv_set(props_ip->general_panel, XV_SHOW, cat == PROPS_CAT_GENERAL, NULL);
    xv_set(props_ip->outs_panel, XV_SHOW, cat == PROPS_CAT_OUTS, NULL);
    xv_set(props_ip->global_panel, XV_SHOW, cat == PROPS_CAT_GLOBAL, NULL);
}

/*
 *  Makes a temporary duplicate of the list of "outs" options for us to
 *  work with.
 */
void
make_temp_outlist()
{
    int i;

    temp_outlist =
	(struct outent *)malloc((unsigned)(sizeof(struct outent)*num_outs));
    
    for ( i = 0 ; i < num_outs ; i++ ) {
	temp_outlist[i].label = strdup(outlist[i].label);
	temp_outlist[i].until = strdup(outlist[i].until);
	temp_outlist[i].message = strdup(outlist[i].message);
	temp_outlist[i].pop = outlist[i].pop;
    }
    temp_num_outs = num_outs;
}

/*
 *  Destroys the list of "outs" options passed to it
 */
void
destroy_outlist(lp, n)
struct outent **lp;
int *n;
{
    int i;
    struct outent *l;

    l = *lp;
    for ( i = 0 ; i < *n ; i++ ) {
	free(l[i].label);
	free(l[i].until);
	free(l[i].message);
    }
    *n = 0;
    free(l);
    *lp = (struct outent *)NULL;
}

void
prep_props_window()
{
    int i, flags;

    if (!admin &&
	(int)xv_get(props_ip->cat_item, PANEL_VALUE) == PROPS_CAT_GLOBAL)
	xv_set(props_ip->cat_item, PANEL_VALUE, PROPS_CAT_GENERAL, NULL);
    modify_cat_item(admin);	
    xv_set(props_ip->intvl_item, PANEL_VALUE, delay, NULL);
    xv_set(props_ip->server_item, PANEL_VALUE, mb_server, NULL);
    xv_set(props_ip->prt_type_item, PANEL_VALUE, prt_type, NULL);
    xv_set(props_ip->prt_dest_item, PANEL_VALUE, prt_dest, NULL);
    set_prt_items(prt_dest, prt_type);
    xv_set(props_ip->dragstyle_item, PANEL_VALUE, dragstyle - 2, NULL);
    xv_set(props_ip->pop_mod_item, PANEL_VALUE, pop_mod, NULL);
    xv_set(props_ip->fmt_item, PANEL_VALUE, fmt_msgs, NULL);
    xv_set(props_ip->admin_item,
	PANEL_VALUE,	admin_enable,
	XV_SHOW,	myentry != (struct mem_rec *)NULL &&
			    (myentry->flags & MB_ADMIN) != 0,
	NULL);
    xv_set(props_ip->site_item,
	PANEL_VALUE,	globrec.name,
	NULL);
    xv_set(props_ip->motd_item,
	PANEL_VALUE,	globrec.motd,
	NULL);
    xv_set(props_ip->fname_fmt_item,
	PANEL_VALUE,	(globrec.flags & MB_G_LASTNM1ST) != 0,
	NULL);

    flags =	((globrec.flags & MB_G_CHG_PLAN) != 0) +
		((globrec.flags & MB_G_PRESERVE) != 0) * 2 +
		((globrec.flags & MB_G_VIA_MAIL) != 0) * 4 +
		((globrec.flags & MB_G_VIA_MB) != 0) * 8;
    xv_set(props_ip->flags_item,
	PANEL_VALUE,	flags,
	NULL);

    xv_set(props_ip->pgr_type_item,
	PANEL_VALUE,	globrec.pagertype,
	NULL);

    /*
     *  Prepare the "outs" panel
     */
    i = (int)xv_get(props_ip->outs_list, PANEL_LIST_NROWS);
    if ( i > 0 )
        xv_set(props_ip->outs_list, PANEL_LIST_DELETE_ROWS, 0, i, NULL);
    for ( i = 0 ; i < num_outs ; i++ )
	xv_set(props_ip->outs_list,
	    PANEL_LIST_INSERT,	i,
	    PANEL_LIST_STRING,	i, outlist[i].label,
	    NULL);
    xv_set(props_ip->outs_label_item, PANEL_VALUE, "", NULL);
    xv_set(props_ip->outs_until_item,
	PANEL_VALUE, "",
	PANEL_INACTIVE, TRUE,
	NULL);
    xv_set(props_ip->outs_msg_item,
	PANEL_VALUE, "",
	PANEL_INACTIVE, TRUE,
	NULL);
    xv_set(props_ip->outs_pop_item,
	PANEL_VALUE, FALSE,
	PANEL_INACTIVE, TRUE,
	NULL);
    xv_set(props_ip->outs_add_item, PANEL_INACTIVE, TRUE, NULL);
    xv_set(props_ip->outs_change_item, PANEL_INACTIVE, TRUE, NULL);
    xv_set(props_ip->outs_delete_item, PANEL_INACTIVE, TRUE, NULL);
    if (temp_outlist != NULL) {
	destroy_outlist(&temp_outlist, &temp_num_outs);
	temp_outlist = NULL;
    }
    outs_changes_pending = FALSE;

    show_props_cat((int)xv_get(props_ip->cat_item, PANEL_VALUE));
}

void outs_add_menu_notify();

void
create_props_window()
{
#ifdef SCALABLE
    int valx, ypos;
#endif

    props_ip = propswin_frame_objects_initialize(NULL, base_frame);

    xv_set((Menu)xv_get(props_ip->outs_add_item, PANEL_ITEM_MENU),
	MENU_NOTIFY_PROC, outs_add_menu_notify, NULL);

    xv_set(props_ip->outs_label_item, PANEL_NOTIFY_LEVEL, PANEL_ALL, NULL);

#ifdef SCALABLE
    /*
     *  Position the category item on the category panel...
     */
    xv_set(props_ip->cat_item,
	XV_X,	wgrid * 2,
	XV_Y,	wgrid * 2,
	NULL);
    window_fit_height(props_ip->cat_panel);

    /*
     *  Position the general, outs and global panels just under the category
     *  panel...
     */
    ypos = xv_get(props_ip->cat_panel, XV_HEIGHT);
    xv_set(props_ip->general_panel, XV_Y, ypos, NULL);
    xv_set(props_ip->outs_panel, XV_Y, ypos, NULL);
    xv_set(props_ip->global_panel, XV_Y, ypos, NULL);

    /*
     *  Position items on the general panel.
     *  message1 is the word "seconds", which is a label to be placed
     *  immediately after the interval item.
     */
    ypos = wgrid * 2;
    xv_set(props_ip->intvl_item,
	XV_X,	wgrid * 2,
	XV_Y,	ypos,
	NULL);
    valx = (int)xv_get(props_ip->intvl_item, PANEL_VALUE_X);
    xv_set(props_ip->message1,
	XV_X,	wgrid * 3 + (int)xv_get(props_ip->intvl_item, XV_WIDTH),
	XV_Y,	wgrid * 2,
	NULL);

    ypos += wgrid + (int)xv_get(props_ip->intvl_item, XV_HEIGHT);
    align_values(props_ip->server_item, valx, ypos);

    ypos += wgrid * 2 + (int)xv_get(props_ip->server_item, XV_HEIGHT);
    align_values(props_ip->prt_type_item, valx, ypos);

    ypos += wgrid + (int)xv_get(props_ip->prt_type_item, XV_HEIGHT);
    align_values(props_ip->prt_dest_item, valx, ypos);

    ypos += wgrid + (int)xv_get(props_ip->prt_dest_item, XV_HEIGHT);
    align_values(props_ip->prt_dir_item, valx, ypos);

    ypos += wgrid + (int)xv_get(props_ip->prt_dir_item, XV_HEIGHT);
    align_values(props_ip->prt_file_item, valx, ypos);

    ypos += wgrid * 2 + (int)xv_get(props_ip->prt_file_item, XV_HEIGHT);
    align_values(props_ip->dragstyle_item, valx, ypos);

    ypos += wgrid * 2 + (int)xv_get(props_ip->dragstyle_item, XV_HEIGHT);
    xv_set(props_ip->pop_mod_item, XV_X, wgrid * 2, XV_Y, ypos, NULL);

    ypos += wgrid / 2 + (int)xv_get(props_ip->pop_mod_item, XV_HEIGHT);
    xv_set(props_ip->fmt_item, XV_X, wgrid * 2, XV_Y, ypos, NULL);

    ypos += wgrid / 2 + (int)xv_get(props_ip->fmt_item, XV_HEIGHT);
    xv_set(props_ip->admin_item, XV_X, wgrid * 2, XV_Y, ypos, NULL);

    window_fit(props_ip->general_panel);

    /*
     *  Position items on the outs panel.
     *  "Outs List" is the widest text item label on this list...
     */
    xv_set(props_ip->outs_list,
	XV_X, wgrid * 2,
	XV_Y, wgrid * 2,
	NULL);
    valx = (int)xv_get(props_ip->outs_list, PANEL_VALUE_X);

    ypos = wgrid * 3 + (int)xv_get(props_ip->outs_list, XV_HEIGHT);
    align_values(props_ip->outs_label_item, valx, ypos);

    ypos += wgrid + (int)xv_get(props_ip->outs_label_item, XV_HEIGHT);
    align_values(props_ip->outs_until_item, valx, ypos);

    ypos += wgrid + (int)xv_get(props_ip->outs_until_item, XV_HEIGHT);
    align_values(props_ip->outs_msg_item, valx, ypos);

    ypos += wgrid + (int)xv_get(props_ip->outs_msg_item, XV_HEIGHT);
    align_values(props_ip->outs_pop_item, valx, ypos);

    valx = (int)xv_get(props_ip->outs_change_item, XV_WIDTH);
    xv_set(props_ip->outs_add_item, PANEL_LABEL_WIDTH,
	(int)xv_get(props_ip->outs_add_item, PANEL_LABEL_WIDTH) + valx -
	(int)xv_get(props_ip->outs_add_item, XV_WIDTH), NULL);
    xv_set(props_ip->outs_delete_item, PANEL_LABEL_WIDTH,
	(int)xv_get(props_ip->outs_delete_item, PANEL_LABEL_WIDTH) + valx -
	(int)xv_get(props_ip->outs_delete_item, XV_WIDTH), NULL);
    
    valx = wgrid * 3 + (int)xv_get(props_ip->outs_list, XV_WIDTH);
    ypos = wgrid * 2 + (int)xv_get(props_ip->outs_list, XV_HEIGHT) / 2 -
	(int)xv_get(props_ip->outs_delete_item, XV_HEIGHT) / 2;
    xv_set(props_ip->outs_delete_item,
	XV_X, valx,
	XV_Y, ypos,
	NULL);

    xv_set(props_ip->outs_add_item,
	XV_X, valx,
	XV_Y, ypos - wgrid - (int)xv_get(props_ip->outs_add_item, XV_HEIGHT),
	NULL);

    xv_set(props_ip->outs_change_item,
	XV_X, valx,
	XV_Y, ypos + wgrid + (int)xv_get(props_ip->outs_delete_item, XV_HEIGHT),
	NULL);
    window_fit(props_ip->outs_panel);

    /*
     *  Position items on the global panel.
     *  First group is sitename and motd:
     */
    xv_set(props_ip->site_item,
	XV_X, wgrid * 2,
	XV_Y, wgrid * 2,
	NULL);

    ypos = wgrid * 3 + (int)xv_get(props_ip->site_item, XV_HEIGHT);
    valx = (int)xv_get(props_ip->site_item, PANEL_VALUE_X);
    align_values(props_ip->motd_item, valx, ypos);

    /*
     *  Remainder are grouped together.  "Defaults Pager Type" is longest
     *  label:
     */
    xv_set(props_ip->pgr_type_item, XV_X, wgrid * 2, NULL);
    valx = (int)xv_get(props_ip->pgr_type_item, PANEL_VALUE_X);

    ypos += wgrid * 2 + (int)xv_get(props_ip->motd_item, XV_HEIGHT);
    align_values(props_ip->fname_fmt_item, valx, ypos);

    ypos += wgrid + (int)xv_get(props_ip->fname_fmt_item, XV_HEIGHT);
    align_values(props_ip->flags_item, valx, ypos);

    ypos += wgrid + (int)xv_get(props_ip->flags_item, XV_HEIGHT);
    align_values(props_ip->pgr_type_item, valx, ypos);

    window_fit(props_ip->global_panel);

    /*
     *  Final sizing of the middle panels.  Assumes global panel
     *  is the widest, general panel is the tallest.
     */
    valx = (int)xv_get(props_ip->global_panel, XV_WIDTH, NULL);
    ypos = (int)xv_get(props_ip->general_panel, XV_HEIGHT, NULL);
    xv_set(props_ip->general_panel,
	XV_WIDTH, valx,
	NULL);
    xv_set(props_ip->outs_panel,
	XV_WIDTH, valx,
	XV_HEIGHT, ypos,
	NULL);
    xv_set(props_ip->global_panel,
	XV_HEIGHT, ypos,
	NULL);
    xv_set(props_ip->apply_panel,
	XV_Y,	(int)xv_get(props_ip->general_panel, XV_Y) + ypos,
	NULL);

    /*
     *  Position items in the apply panel
     */
    xv_set(props_ip->apply_item, XV_Y, wgrid, NULL);
    xv_set(props_ip->reset_item, XV_Y, wgrid, NULL);
    window_fit_height(props_ip->apply_panel);

    window_fit(props_ip->frame);
    position_bot_buttons(props_ip->apply_panel, props_ip->apply_item,
	props_ip->reset_item);
#endif

    props_window_created = TRUE;
}

void
show_props_window()
{
    if (!props_window_created) create_props_window();

    prep_props_window();
    xv_set(props_ip->frame, XV_SHOW, TRUE, NULL);
}

void
props_apply_changes()
{
    int has_admin_caps;

    /*
     *  If they changed the delay value, process accordingly
     */
    if (delay != (int)xv_get(props_ip->intvl_item, PANEL_VALUE)) {
	delay = (int)xv_get(props_ip->intvl_item, PANEL_VALUE);
	set_timer(TRUE);
    }

    prt_type = (int)xv_get(props_ip->prt_type_item, PANEL_VALUE);
    prt_dest = (int)xv_get(props_ip->prt_dest_item, PANEL_VALUE);
    if (prt_dest == TO_PRINTER) {
	strncpy(prt_name, (char *)xv_get(props_ip->prt_dir_item, PANEL_VALUE),
	    PRINTERNAME_LEN);
	prt_name[PRINTERNAME_LEN] = '\0';
    }
    else {
	strncpy(prt_dir, (char *)xv_get(props_ip->prt_dir_item, PANEL_VALUE),
	    MAXPATHLEN);
	prt_dir[MAXPATHLEN - 1] = '\0';
	strncpy(prt_file, (char *)xv_get(props_ip->prt_file_item,
	    PANEL_VALUE), 81);
	prt_file[80] = '\0';
    }

    /*
     *  Did they change the outs list?
     */
    if ( temp_outlist != NULL ) {
	destroy_outlist(&outlist, &num_outs);
	outlist = temp_outlist;
	temp_outlist = NULL;
	num_outs = temp_num_outs;
	redo_outs_menu();
    }

    if ((int)xv_get(props_ip->dragstyle_item, PANEL_VALUE) != dragstyle - 2 ) {
	if ( (int)xv_get(props_ip->dragstyle_item, PANEL_VALUE) == 0 )
	    dragstyle = 2;
	else
	    dragstyle = 3;
	set_scanwin_items_status();
    }

   pop_mod = (int)xv_get(props_ip->pop_mod_item, PANEL_VALUE);
    
    /*
     *  Did they change message formatting?
     */
    if (fmt_msgs != (int)xv_get(props_ip->fmt_item, PANEL_VALUE)) {
	fmt_msgs = !fmt_msgs;
	set_sendwin_items_status();
    }

    admin_enable = (int)xv_get(props_ip->admin_item, PANEL_VALUE);

    /*
     *  Are they allowed to go into admin mode?
     */
    has_admin_caps = myentry != (Mem_Rec *)NULL &&
		     (myentry->flags & MB_ADMIN) != 0;

    /*
     *  Did they ask to go into or out of admin mode?
     */
    if ( admin != (has_admin_caps && admin_enable) ) {
	admin = (has_admin_caps && admin_enable);
	update_global_display();
	set_mainwin_items_status();
    }

    /*
     *  saves global variables
     */
    if (has_admin_caps) {
	Global_Rec new_glob;
	int flags;

	strcpy(new_glob.name, (char *)xv_get(props_ip->site_item, PANEL_VALUE));
	strcpy(new_glob.motd, (char *)xv_get(props_ip->motd_item, PANEL_VALUE));
	new_glob.flags = 0;
	if ((int)xv_get(props_ip->fname_fmt_item, PANEL_VALUE))
	    new_glob.flags |= MB_G_LASTNM1ST;
	flags = (int)xv_get(props_ip->flags_item, PANEL_VALUE);
	if ((flags & 1) != 0) new_glob.flags |= MB_G_CHG_PLAN;
	if ((flags & 2) != 0) new_glob.flags |= MB_G_PRESERVE;
	if ((flags & 4) != 0) new_glob.flags |= MB_G_VIA_MAIL;
	if ((flags & 8) != 0) new_glob.flags |= MB_G_VIA_MB;
	new_glob.pagertype = (int)xv_get(props_ip->pgr_type_item, PANEL_VALUE);
	modify_global_values(&new_glob);
    }

    /*
     *  Save user-specific program parameters to PARAMFILE:
     */
    save_params();

    /*
     *  If they asked to change servers, do it last.
     */

    if ( strcmp(mb_server, (char *)xv_get(props_ip->server_item,
	PANEL_VALUE)) != 0 ) {
	strcpy(mb_server, (char *)xv_get(props_ip->server_item, PANEL_VALUE));
	if(connect_to_server()) (void)get_modified_records();
    }

    prep_props_window();
}

/*
 *  Called whenever we change categories or dismiss the window; checks
 *  whether there are pending changes for this category.
 */
int
props_ok_to_leave()
{
    int cat = (int)xv_get(props_ip->cat_item, PANEL_VALUE);
    int retval = TRUE;
    Xv_Notice notice;

    if ( cat == PROPS_CAT_GENERAL ) {
	/*
	 *  This is a real knot.  Apologies to anyone who has to figure it
	 *  out...all we're trying to do is see if anything was changed.
	 */
	if ( (int)xv_get(props_ip->intvl_item, PANEL_VALUE) != delay ||
	    strcmp((char *)xv_get(props_ip->server_item, PANEL_VALUE),
		mb_server) != 0 ||
	    (int)xv_get(props_ip->prt_type_item, PANEL_VALUE) != prt_type ||
	    (int)xv_get(props_ip->prt_dest_item, PANEL_VALUE) != prt_dest ||
	    (int)xv_get(props_ip->dragstyle_item, PANEL_VALUE) !=
		(dragstyle - 2) ||
	    (int)xv_get(props_ip->pop_mod_item, PANEL_VALUE) != pop_mod ||
	    (int)xv_get(props_ip->fmt_item, PANEL_VALUE) != fmt_msgs ||
	    (int)xv_get(props_ip->admin_item, PANEL_VALUE) != admin_enable )
	    retval = FALSE;
	else {
	    if ( (int)xv_get(props_ip->prt_dest_item, PANEL_VALUE ) ==
	        TO_PRINTER ) {
	        if ( strcmp((char *)xv_get(props_ip->prt_dir_item, PANEL_VALUE), prt_name)
		    != 0 ) 
		    retval = FALSE;
	    }
	    else {
		if ( strcmp((char *)xv_get(props_ip->prt_dir_item, PANEL_VALUE), prt_dir)
		    != 0 || strcmp((char *)xv_get(props_ip->prt_file_item, PANEL_VALUE),
		    prt_file) != 0 )
		    retval = FALSE;
	    }
	}
    }
    else if (cat == PROPS_CAT_OUTS ) {
	if ( outs_changes_pending ||
	    strlen((char *)xv_get(props_ip->outs_label_item, PANEL_VALUE)) > 0 )
	    retval = FALSE;
    }
    else if (cat == PROPS_CAT_GLOBAL ) {
	int flags =	((globrec.flags & MB_G_CHG_PLAN) != 0) +
			((globrec.flags & MB_G_PRESERVE) != 0) * 2 +
			((globrec.flags & MB_G_VIA_MAIL) != 0) * 4 +
			((globrec.flags & MB_G_VIA_MB) != 0) * 8;

	if ( strcmp((char *)xv_get(props_ip->site_item, PANEL_VALUE),
	        globrec.name) != 0 ||
	    strcmp((char *)xv_get(props_ip->motd_item, PANEL_VALUE),
	        globrec.motd) != 0 ||
	    (int)xv_get(props_ip->fname_fmt_item, PANEL_VALUE) !=
	        ((globrec.flags & MB_G_LASTNM1ST) != 0) ||
	    (int)xv_get(props_ip->flags_item, PANEL_VALUE) != flags )
	    retval = FALSE;
    }

    if ( retval == FALSE ) {
	notice = xv_create(props_ip->frame, NOTICE,
	    NOTICE_MESSAGE_STRINGS,	"Changes to this category have not been applied.",
					NULL,
	    NOTICE_BUTTON,		"Apply Changes", 42,
	    NOTICE_BUTTON,		"Discard Changes", TRUE,
	    NOTICE_BUTTON,		"Cancel", FALSE,
	    NOTICE_STATUS,		&retval,
	    XV_SHOW,			TRUE,
	    NULL);

	xv_destroy_safe(notice);
    }

    if (retval == 42) {
	props_apply_changes();
	retval = TRUE;
    }

    return retval;
}

void
props_out_add(where)
int where;
{
    int i, new_row;

    if (temp_outlist == NULL) make_temp_outlist();

    new_row = (int)xv_get(props_ip->outs_list, PANEL_LIST_FIRST_SELECTED);
    if (where == ADD_AFTER) {
	if (new_row == -1)
	    new_row = (int)xv_get(props_ip->outs_list, PANEL_LIST_NROWS);
	else
	    new_row++;
    }
    else if (new_row == -1)
	new_row = 0;

    temp_num_outs++;
    temp_outlist = (struct outent *)realloc((char *)temp_outlist,
			(unsigned)(sizeof(struct outent)*temp_num_outs));
    for ( i = temp_num_outs - 1 ; i > new_row ; i--) {
	temp_outlist[i].label = temp_outlist[i-1].label;
	temp_outlist[i].until = temp_outlist[i-1].until;
	temp_outlist[i].message = temp_outlist[i-1].message;
	temp_outlist[i].pop = temp_outlist[i-1].pop;
    }
    temp_outlist[new_row].label =
	strdup((char *)xv_get(props_ip->outs_label_item, PANEL_VALUE));
    temp_outlist[new_row].until =
	strdup((char *)xv_get(props_ip->outs_until_item, PANEL_VALUE));
    temp_outlist[new_row].message =
	strdup((char *)xv_get(props_ip->outs_msg_item, PANEL_VALUE));
    temp_outlist[new_row].pop =
	(int)xv_get(props_ip->outs_pop_item, PANEL_VALUE);

    xv_set(props_ip->outs_list,
	PANEL_LIST_INSERT,	new_row,
	PANEL_LIST_STRING,	new_row, temp_outlist[new_row].label,
	PANEL_LIST_SELECT,	new_row, TRUE,
	NULL);

    xv_set(props_ip->outs_change_item, PANEL_INACTIVE, FALSE, NULL);
    xv_set(props_ip->outs_delete_item, PANEL_INACTIVE, FALSE, NULL);

    outs_changes_pending = TRUE;
}

void
props_out_change()
{
    int drow;

    drow = (int)xv_get(props_ip->outs_list, PANEL_LIST_FIRST_SELECTED);

    if (temp_outlist == NULL) make_temp_outlist();

    free(temp_outlist[drow].label);
    free(temp_outlist[drow].until);
    free(temp_outlist[drow].message);
    temp_outlist[drow].label =
	strdup((char *)xv_get(props_ip->outs_label_item, PANEL_VALUE));
    temp_outlist[drow].until =
	strdup((char *)xv_get(props_ip->outs_until_item, PANEL_VALUE));
    temp_outlist[drow].message =
	strdup((char *)xv_get(props_ip->outs_msg_item, PANEL_VALUE));
    temp_outlist[drow].pop =
	(int)xv_get(props_ip->outs_pop_item, PANEL_VALUE);
    xv_set(props_ip->outs_list,
	PANEL_LIST_STRING,	drow, temp_outlist[drow].label,
	NULL);

    outs_changes_pending = TRUE;
}


/*
 * Notify callback function for `outs_label_item'.
 */
Panel_setting
props_outs_label_notify(item, event)
Panel_item	item;
Event		*event;
{
    int flag;

    flag = strlen((char *)xv_get(item, PANEL_VALUE)) == 0;

    xv_set(props_ip->outs_until_item, PANEL_INACTIVE, flag, NULL);
    xv_set(props_ip->outs_msg_item, PANEL_INACTIVE, flag, NULL);
    xv_set(props_ip->outs_pop_item, PANEL_INACTIVE, flag, NULL);
    xv_set(props_ip->outs_add_item, PANEL_INACTIVE, flag, NULL);
	
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `reset_item'.
 */
void
props_reset_notify(item, event)
Panel_item	item;
Event		*event;
{	
    prep_props_window();
}

/*
 * Notify callback function for `cat_item'.
 */
void
props_cat_notify(item, value, event)
Panel_item	item;
int		value;
Event		*event;
{
    show_props_cat(value);
}

/*
 * Notify callback function for `prt_dest_item' and `prt_type_item'.
 */
void
props_prt_settings_notify(item, value, event)
Panel_item	item;
int		value;
Event		*event;
{
    int dest, type;

    if (item == props_ip->prt_dest_item) {
	dest = value;
	type = (int)xv_get(props_ip->prt_type_item, PANEL_VALUE);
    }
    else {
	type = value;
	dest = (int)xv_get(props_ip->prt_dest_item, PANEL_VALUE);
    }
    set_prt_items(dest, type);
}
	
/*
 * Notify callback function for `apply_item'.
 */
void
props_apply_notify(item, event)
Panel_item	item;
Event		*event;
{
    int s, retval = TRUE;
    Xv_Notice notice;

    /*
     *  The Outs category requires special treatment, as it is possible to
     *  have started to make changes but not have completed them.
     */
    if ( (int)xv_get(props_ip->cat_item, PANEL_VALUE) != PROPS_CAT_OUTS)
	props_apply_changes();

    if ( ( s = (int)xv_get(props_ip->outs_list, PANEL_LIST_FIRST_SELECTED) ) 
	== -1 ) {
	if ( strlen((char *)xv_get(props_ip->outs_label_item, PANEL_VALUE)) > 0 ) {
	    notice = xv_create(props_ip->frame, NOTICE,
		NOTICE_MESSAGE_STRINGS,	"You have not finished adding this item.",
					"You may add it to the list, discard it, or",
					"cancel the Apply:",
					NULL,
		NOTICE_BUTTON,		"Add to Top",
					ADD_BEFORE,
		NOTICE_BUTTON,		"Add to Bottom",
					ADD_AFTER,
		NOTICE_BUTTON,		"Discard", TRUE,
		NOTICE_BUTTON,		"Cancel", FALSE,
		NOTICE_STATUS,		&retval,
		XV_SHOW,			TRUE,
		NULL);
	    xv_destroy_safe(notice);
	}
    }
    else {
	if (temp_outlist == NULL) make_temp_outlist();

	if ( strcmp(temp_outlist[s].label,
	    (char *)xv_get(props_ip->outs_label_item, PANEL_VALUE)) != 0 ||
	    strcmp(temp_outlist[s].until,
	    (char *)xv_get(props_ip->outs_until_item, PANEL_VALUE)) != 0 ||
	    strcmp(temp_outlist[s].message,
	    (char *)xv_get(props_ip->outs_msg_item, PANEL_VALUE)) != 0 ||
	    temp_outlist[s].pop !=
	    (int)xv_get(props_ip->outs_pop_item, PANEL_VALUE)) {
	    notice = xv_create(props_ip->frame, NOTICE,
		NOTICE_MESSAGE_STRINGS,	"You have not finished adding or changing this",
					"item.  You may change the selected item on the",
					"list, add a new item, discard your changes, or",
					"cancel the Apply:",
					NULL,
		NOTICE_BUTTON,		"Change Selected Item", 42,
		NOTICE_BUTTON,		"Add Before Selected Item", ADD_BEFORE,
		NOTICE_BUTTON,		"Add After Selected Item", ADD_AFTER,
		NOTICE_BUTTON,		"Discard", TRUE,
		NOTICE_BUTTON,		"Cancel", FALSE,
		NOTICE_STATUS,		&retval,
		XV_SHOW,			TRUE,
		NULL);
	    xv_destroy_safe(notice);
	}
    }

    if (retval == TRUE)
	props_apply_changes();
    else if (retval == FALSE)
	xv_set(item, PANEL_NOTIFY_STATUS, XV_ERROR, NULL);
    else if (retval == 42) {
	props_out_change();
	props_apply_changes();
    }
    else if (retval == ADD_BEFORE || retval == ADD_AFTER) {
	props_out_add(retval);
	props_apply_changes();
    }
}

/*
 *  Notify callback from the outs panel's Add button menu
 */
void
outs_add_menu_notify(menu, menu_item)
Menu menu;
Menu_item menu_item;
{
    if (strcmp((char *)xv_get(menu_item, MENU_STRING), "After") == 0)
	props_out_add(ADD_AFTER);
    else
	props_out_add(ADD_BEFORE);

    /*
     *  Keep window from being dismissed.
     */
    xv_set(menu, MENU_NOTIFY_STATUS, XV_ERROR, NULL);
}

/*
 * Notify callback function for `outs_delete_item'.
 */
void
outs_delete_notify(item, event)
Panel_item	item;
Event		*event;
{
    int drow, i;

    drow = (int)xv_get(props_ip->outs_list, PANEL_LIST_FIRST_SELECTED);

    if (temp_outlist == NULL) make_temp_outlist();
    xv_set(props_ip->outs_list, PANEL_LIST_DELETE, drow, NULL);
    free(temp_outlist[drow].label);
    free(temp_outlist[drow].until);
    free(temp_outlist[drow].message);
    for ( i = drow ; i < temp_num_outs - 1 ; i++ ) {
	temp_outlist[i].label = temp_outlist[i+1].label;
	temp_outlist[i].until = temp_outlist[i+1].until;
	temp_outlist[i].message = temp_outlist[i+1].message;
	temp_outlist[i].pop = temp_outlist[i+1].pop;
    }
    temp_num_outs--;

    xv_set(props_ip->outs_change_item, PANEL_INACTIVE, TRUE, NULL);
    xv_set(props_ip->outs_delete_item, PANEL_INACTIVE, TRUE, NULL);

    outs_changes_pending = TRUE;

    /*
     *  Keep the window from being dismissed
     */
    xv_set(item, PANEL_NOTIFY_STATUS, XV_ERROR, NULL);
}

/*
 * Notify callback function for `outs_change_item'.
 */
void
outs_change_notify(item, event)
Panel_item	item;
Event		*event;
{
    props_out_change();

    /*
     *  Keep the window from being dismissed
     */
    xv_set(item, PANEL_NOTIFY_STATUS, XV_ERROR, NULL);
}

/*
 * Notify callback function for `outs_list'.
 */
int
outs_list_notify(item, string, client_data, op, event, row)
Panel_item	item;
char		*string;
Xv_opaque	client_data;
Panel_list_op	op;
Event		*event;
int		row;
{
    int selected;
    
    selected = (op == PANEL_LIST_OP_SELECT);
    if (selected) {
	if (temp_outlist == NULL) make_temp_outlist();
	xv_set(props_ip->outs_label_item,
	    PANEL_VALUE, temp_outlist[row].label, NULL);
	xv_set(props_ip->outs_until_item,
	    PANEL_VALUE, temp_outlist[row].until,
	    PANEL_INACTIVE, FALSE,
	    NULL);
	xv_set(props_ip->outs_msg_item,
	    PANEL_VALUE, temp_outlist[row].message,
	    PANEL_INACTIVE, FALSE,
	    NULL);
	xv_set(props_ip->outs_pop_item,
	    PANEL_VALUE, temp_outlist[row].pop,
	    PANEL_INACTIVE, FALSE,
	    NULL);
    }
    else {
	xv_set(props_ip->outs_label_item,
	    PANEL_VALUE,	"",
	    NULL);
	xv_set(props_ip->outs_until_item,
	    PANEL_VALUE,	"",
	    PANEL_INACTIVE,	TRUE,
	    NULL);
	xv_set(props_ip->outs_msg_item,
	    PANEL_VALUE,	"",
	    PANEL_INACTIVE,	TRUE,
	    NULL);
	xv_set(props_ip->outs_pop_item,
	    PANEL_VALUE,	FALSE,
	    PANEL_INACTIVE,	TRUE,
	    NULL);
    }

    xv_set(props_ip->outs_add_item, PANEL_INACTIVE, !selected, NULL);    
    xv_set(props_ip->outs_change_item, PANEL_INACTIVE, !selected, NULL);
    xv_set(props_ip->outs_delete_item, PANEL_INACTIVE, !selected, NULL);
    return XV_OK;
}

/*
 * Done callback function for `frame'.
 */
void
props_done_proc(frame)
Frame frame;
{
    if (props_ok_to_leave()) {
	if (temp_outlist != NULL) destroy_outlist(temp_outlist, &temp_num_outs);
	xv_set(frame, XV_SHOW, FALSE, NULL);
    }
}
