/*
 *  sendwin.c -- Routines to support the msgboard "Send Pmsg" window.
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include "sendwin_ui.h"
#include "msgboard.h"
#include "msgboard_xv.h"
#include "pmsg_client.h"

sendwin_frame_objects	*send_ip;

int
    send_window_created = FALSE,	/* is sendmsg window created?	*/
    to_is_wide = FALSE,			/* is the to item in "wide mode"? */
    old_choice,				/* Previous state of choice item */
    bot_panel_ht;

char	default_to[LOGID_LEN+1];	/* "To" value that we go back to
					   if the Reset button is pressed */

void
create_sendmsg_window()
{
#ifdef SCALABLE
    int valx, ypos;
#endif

    send_ip = sendwin_frame_objects_initialize(NULL, base_frame);

#ifdef SCALABLE
    /*
     *  Phone number text item has the widest label on this panel, so use
     *  it to set the X positioning for all the other ones.
     */
    xv_set(send_ip->phnum_item, XV_X, wgrid * 2, NULL);
    valx = (int)xv_get(send_ip->phnum_item, PANEL_VALUE_X);

    ypos = wgrid * 2;
    align_values(send_ip->to_item, valx, ypos);
    xv_set(send_ip->fname_item,
	XV_Y,	ypos,
	XV_X,	(int)xv_get(send_ip->to_item, XV_X) +
		(int)xv_get(send_ip->to_item, XV_WIDTH) + wgrid,
	NULL);

    ypos += wgrid + (int)xv_get(send_ip->to_item, XV_HEIGHT);
    align_values(send_ip->time_item, valx, ypos);

    ypos += wgrid + (int)xv_get(send_ip->time_item, XV_HEIGHT);
    align_values(send_ip->caller_item, valx, ypos);

    ypos += wgrid + (int)xv_get(send_ip->caller_item, XV_HEIGHT);
    align_values(send_ip->org_item, valx, ypos);

    ypos += wgrid + (int)xv_get(send_ip->org_item, XV_HEIGHT);
    align_values(send_ip->phnum_item, valx, ypos);

    ypos += wgrid + (int)xv_get(send_ip->phnum_item, XV_HEIGHT);
    align_values(send_ip->choice_item, valx, ypos);

    window_fit(send_ip->panel);

    xv_set(send_ip->textsw,
	XV_Y,		(int)xv_get(send_ip->panel, XV_HEIGHT),
	XV_WIDTH,	(int)xv_get(send_ip->panel, XV_WIDTH),
	XV_HEIGHT,	wgrid * 6,
	NULL);
    xv_set(send_ip->fmt_item,
	XV_X,	wgrid * 2,
	XV_Y,	wgrid,
	NULL);
    xv_set(send_ip->send_item, XV_Y, wgrid * 2, NULL);
    xv_set(send_ip->reset_item, XV_Y, wgrid * 2, NULL);
    window_fit_height(send_ip->bot_panel);
    xv_set(send_ip->bot_panel,
	XV_Y,		(int)xv_get(send_ip->textsw, XV_Y) +
			(int)xv_get(send_ip->textsw, XV_HEIGHT),
	XV_WIDTH,	(int)xv_get(send_ip->panel, XV_WIDTH),
	NULL);
    window_fit(send_ip->frame);
#endif
    bot_panel_ht = (int)xv_get(send_ip->bot_panel, XV_HEIGHT);

    if (send_params.state != BOGUS) {
	frame_set_rect(send_ip->frame, &send_params.rect);
	if (send_params.state == TRUE)
	    xv_set(send_ip->frame, FRAME_CMD_PIN_STATE, FRAME_CMD_PIN_IN, NULL);
    }

    xv_set(send_ip->textsw,
	TEXTSW_IGNORE_LIMIT,	TEXTSW_INFINITY,
	NULL);

    xv_set(send_ip->to_item,
	PANEL_NOTIFY_STRING,	"\n\r\t@, !",
	NULL);

    send_window_created = TRUE;
}

/*
 *  Sets the fullname string for the sendmsg popup
 */
void
set_sm_fname_item()
{
    char cbuf[FNAME_LEN+3];

    sprintf(cbuf, "(%s)",
	get_fname((char *)xv_get(send_ip->to_item, PANEL_VALUE)));
    xv_set(send_ip->fname_item, PANEL_LABEL_STRING, cbuf, NULL);
}

void
set_sm_time()
{
    time_t now;
    char timebuf[17];

    if (strlen((char *)xv_get(send_ip->to_item, PANEL_VALUE)) == 0)
	timebuf[0] = '\0';
    else {
	time(&now);
	strftime(timebuf, 17, "%k:%M %a %b %e", localtime(&now));
    }
    xv_set(send_ip->time_item, PANEL_VALUE, timebuf, NULL);
}

/*
 *  Change width of "to" display to extra-wide if flag is TRUE, or
 *  normal if FALSE.
 */
void
set_to_wide(flag)
int flag;
{
    if (to_is_wide == flag) return;

    to_is_wide = flag;
    if (to_is_wide)
	xv_set(send_ip->to_item,
	    PANEL_VALUE_DISPLAY_LENGTH,	40,
	    NULL);
    else
	xv_set(send_ip->to_item,
	    PANEL_VALUE_DISPLAY_LENGTH,	LOGID_LEN,
	    NULL);

    xv_set(send_ip->fname_item,
	    XV_X,	(int)xv_get(send_ip->to_item, XV_X) +
			(int)xv_get(send_ip->to_item, XV_WIDTH) + wgrid,
	    NULL);
}

/*
 *  Makes sure those panel items whose active or visible status can vary
 *  are set properly.
 */
void
set_sendwin_items_status()
{
    int no_text;

    if (!send_window_created) return;

    no_text = ((int)xv_get(send_ip->choice_item, PANEL_VALUE) &
	SM_CHOICE_LV_MSG) == 0;

    if (fmt_msgs)
	xv_set(send_ip->fmt_item, XV_SHOW, FALSE, NULL);
    else
	xv_set(send_ip->fmt_item,
	    PANEL_INACTIVE,	no_text,
	    XV_SHOW,		TRUE,
	    NULL);

    xv_set(send_ip->textsw,
	TEXTSW_BROWSING,	no_text,
	TEXTSW_READ_ONLY,	no_text,
	NULL);
}

void
prep_sendmsg_window()
{
    xv_set(send_ip->to_item, PANEL_VALUE, default_to, NULL);
    set_to_wide(FALSE);
    set_sm_fname_item();
    set_sm_time();    
    xv_set(send_ip->caller_item, PANEL_VALUE, "", NULL);
    xv_set(send_ip->org_item, PANEL_VALUE, "", NULL);
    xv_set(send_ip->phnum_item, PANEL_VALUE, "", NULL);
    xv_set(send_ip->choice_item, PANEL_VALUE, SM_CHOICE_DEFAULT, NULL);
    old_choice = SM_CHOICE_DEFAULT;
    textsw_reset(send_ip->textsw, 0, 0);
    if (default_to[0] == '\0')
	xv_set(send_ip->panel, PANEL_CARET_ITEM, send_ip->to_item, NULL);
    else
	xv_set(send_ip->panel,
	    PANEL_CARET_ITEM, send_ip->caller_item,
	    NULL);
    set_sendwin_items_status();
}

void
show_sendmsg_window()
{
    if (!send_window_created) create_sendmsg_window();

    if (cur_sel != BOGUS)
	strcpy(default_to, ulist[cur_sel].logid);
    prep_sendmsg_window();
    xv_set(send_ip->frame, XV_SHOW, TRUE, NULL);
}

/*
 *  Formats the text in the send text subwindow
 */
void
fmt_sendwin()
{
    char buf[100], tmpname[20];
    FILE *pipe;

    sprintf(tmpname, "/tmp/msgb_%d", getpid());
    textsw_store_file(send_ip->textsw, tmpname, 0, 0);
    sprintf(buf, "fmt %s", tmpname);
    if ((pipe = popen(buf, "r")) == FNULL) {
	pr_msg(NULL, TRUE, "Couldn't open pipe!");
	return;
    }
    textsw_reset(send_ip->textsw, 0, 0);

    while (fgets(buf, 100, pipe) != CNULL)
	textsw_insert(send_ip->textsw, buf, strlen(buf));
    pclose(pipe);
    textsw_normalize_view(send_ip->textsw, 0);
    unlink(tmpname);
}

/*
 *  Does the actual sending stuff.  Returns TRUE if successful, FALSE if not.
 */
int
send_pmsg()
{
    int buf_len, retval;
    struct pmsg msg;
    char caller[CALLER_LEN + 1];

    msg.id = strdup((char *)xv_get(send_ip->to_item, PANEL_VALUE));
    if (msg.id[0] == '\0') {
	pr_msg(NULL, TRUE, "No recipient specified.");
	free(msg.id);
	return FALSE;
    }

    msg.caller = strdup((char *)xv_get(send_ip->caller_item, PANEL_VALUE));
    if (msg.caller[0] == '\0') {
	pr_msg(NULL, TRUE, "No caller specified.");
	free(msg.id);
	free(msg.caller);
	return FALSE;
    }

    msg.flags = (int)xv_get(send_ip->choice_item, PANEL_VALUE);
    if ( (msg.flags & SM_CHOICE_LV_MSG) != 0 ) {
	buf_len = (int)xv_get(send_ip->textsw, TEXTSW_LENGTH);
	if ( buf_len == 0) {
	    pr_msg(NULL, TRUE, "No text in message.");
	    free(msg.id);
	    free(msg.caller);
	    return FALSE;
	}
	if (fmt_msgs) {
	    fmt_sendwin();
	    buf_len = (int)xv_get(send_ip->textsw, TEXTSW_LENGTH);
	}
	msg.text = malloc(buf_len + 1);
	(void)xv_get(send_ip->textsw, TEXTSW_CONTENTS, 0, msg.text, buf_len);
	msg.text[buf_len] = '\0';
    }
    else
	msg.text = strdup("");
	
    msg.org = strdup((char *)xv_get(send_ip->org_item, PANEL_VALUE));
    msg.phnum = strdup((char *)xv_get(send_ip->phnum_item, PANEL_VALUE));
    msg.time = strdup((char *)xv_get(send_ip->time_item, PANEL_VALUE));

    retval = send_msg_to_server(&msg);
    free_pmsg(msg);
    return retval;
}

/*
 *  Gathers all the parameters for this window
 */
void
get_send_params(p)
struct winparams *p;
{
    if (!send_window_created)
	*p = send_params;
    else {
	p->state = (int)xv_get(send_ip->frame, FRAME_CMD_PIN_STATE);
	frame_get_rect(send_ip->frame, &(p->rect));
    }
}

/*
 * Event callback function for `frame'.
 */
Notify_value
sm_frame_event(win, event, arg, type)
Xv_window	win;
Event		*event;
Notify_arg	arg;
Notify_event_type type;
{
    /*
     *  Check for a resize event
     */
    if (event_action(event) == WIN_RESIZE) {
	int text_ht, top_panel_ht;

	/*
	 *  Make the textsw the pane that consumes any additional height
	 *  on a resize.  Textsw will always be at least wgrid * 6
	 *  pixels high; this means the bottom panel may disappear if
	 *  the window is made too short.  C'est la vie, caveat resizer
	 *  and all that.
	 */
	top_panel_ht = (int)xv_get(send_ip->panel, XV_HEIGHT);
	text_ht = (int)xv_get(send_ip->frame, XV_HEIGHT) - top_panel_ht -
	    bot_panel_ht;
	if (text_ht < wgrid * 6) text_ht = wgrid * 6;
	xv_set(send_ip->textsw, XV_HEIGHT, text_ht, NULL);
	xv_set(send_ip->bot_panel,
	    XV_Y,	top_panel_ht + text_ht,
	    XV_HEIGHT,	bot_panel_ht,
	    NULL);
	position_bot_buttons(send_ip->bot_panel, send_ip->send_item,
	    send_ip->reset_item);
    }
    return notify_next_event_func(win, (Notify_event) event, arg, type);
}

/*
 * Notify callback function for `to_item'.
 */
Panel_setting
sm_to_notify(item, event)
Panel_item	item;
Event		*event;
{
    /*
     *  Process special chars
     */
    if (event_action(event) == '@' || event_action(event) == '!') {
	set_to_wide(TRUE);
	xv_set(send_ip->fname_item, PANEL_LABEL_STRING, "(OFFSITE)", NULL);
	return PANEL_INSERT;
    }
    else if (event_action(event) == ' ' || event_action(event) == ',') {
	set_to_wide(TRUE);
	xv_set(send_ip->fname_item, PANEL_LABEL_STRING, "(MULTIPLE)", NULL);
	return PANEL_INSERT;
    }
    /*
     *  Normal completion chars (NL, TAB, etc.)
     */
    else {
	if (to_is_wide) {
	    char tmpbuf[TO_LEN + 1];

	    strcpy(tmpbuf, (char *)xv_get(send_ip->to_item, PANEL_VALUE));
	    if (strchr(tmpbuf, '@') == 0 && strchr(tmpbuf, '!') == 0 &&
		strchr(tmpbuf, ' ') == 0 && strchr(tmpbuf, ',') == 0)
		set_to_wide(FALSE);
	}
	if (!to_is_wide)
	    set_sm_fname_item();
	set_sm_time();
	if (event_shift_is_down(event))
	    return PANEL_PREVIOUS;
	else
	    return PANEL_NEXT;
    }
}

/*
 * Notify callback function for `phnum_item'.
 */
Panel_setting
sm_phnum_notify(item, event)
Panel_item	item;
Event		*event;
{
    if (event_shift_is_down(event))
	return PANEL_PREVIOUS;
    else {
	int msg_flags;

	xv_set(send_ip->panel, PANEL_CARET_ITEM, send_ip->caller_item, NULL);
	msg_flags= (int)xv_get(send_ip->choice_item, PANEL_VALUE);
	if ( (msg_flags & SM_CHOICE_LV_MSG) !=0 )
	    win_set_kbd_focus(textsw_first(send_ip->textsw),
	    xv_get(textsw_first(send_ip->textsw), XV_XID));
	return PANEL_NONE;
    }
}

/*
 * Notify callback function for `choice_item'.
 */
void
sm_choice_notify(item, value, event)
Panel_item	item;
int		value;
Event		*event;
{
    if ( (value & SM_CHOICE_DEFAULT) == SM_CHOICE_DEFAULT ) {
	if ( (old_choice & SM_CHOICE_DEFAULT) != SM_CHOICE_DEFAULT )
	    value = SM_CHOICE_DEFAULT;
	else
	    value -= SM_CHOICE_DEFAULT;
	xv_set(send_ip->choice_item, PANEL_VALUE, value, NULL);
    }
    if ( value == 0 ) {
	value = SM_CHOICE_DEFAULT;
	xv_set(send_ip->choice_item, PANEL_VALUE, value, NULL);
    }
    if ( (value & SM_CHOICE_LV_MSG) != 0 )
	win_set_kbd_focus(textsw_first(send_ip->textsw),
	    xv_get(textsw_first(send_ip->textsw), XV_XID));
    else
	xv_set(send_ip->panel,
	    PANEL_CARET_ITEM, send_ip->caller_item,
	    NULL);

    set_sendwin_items_status();
    old_choice = value;
}

/*
 * Notify callback function for `fmt_item'.
 */
void
sm_fmt_notify(item, event)
Panel_item	item;
Event		*event;
{
    fmt_sendwin();
    xv_set(item, PANEL_NOTIFY_STATUS, XV_ERROR, NULL);
}

/*
 * Notify callback function for `send_item'.
 */
void
sm_send_notify(item, event)
Panel_item	item;
Event		*event;
{	
    clear_msg();
    if(send_pmsg()) {
	default_to[0] = '\0';
	prep_sendmsg_window();
    }
    else
	xv_set(item, PANEL_NOTIFY_STATUS, XV_ERROR, NULL);
}

/*
 * Notify callback function for `reset_item'.
 */
void
sm_reset_notify(item, event)
Panel_item	item;
Event		*event;
{
    prep_sendmsg_window();
}
