#include <stdio.h>
#include <sys/time.h>
#include <sys/stat.h>
#include "msgboard.h"
#include "mb_client.h"

/*
 *  Determined by the max size of a record; needs to be recomputed each
 *  time we change what's stored in the datafile
 */
#define DFINBUFSIZE	161

#define DF_COOKIE       "msgdata"
#define DF_VERSION      3
#define SEP		'\034'

int
    initialized = FALSE;		/* Have we read the data file yet? */

unsigned int
    globals_seq_no,			/* Header sequence number	*/
    highest_seq_no = 0,			/* last sequence number issued	*/
    *seq_no;

#ifdef FIXITINMIX

#include <sys/param.h>
#include <sys/limits.h>
#include <string.h>
#include <pwd.h>
#include <signal.h>
#include "msgd.h"
#include "mb_server.h"

#ifdef DEBUG
#include <errno.h>
extern int sys_nerr;
extern char *sys_errlist[];
#endif


#define DFNAME          "msgdata"
#define PMSGDIRNAME	"msgs"
#define LOGNAME		"errlog"

/*
 *  Number of seconds that we wait before flushing modifications
 *  to the checkpoint file.
 */
#define CACHE_TIME	30

/*
 *  OFF =     No timer running
 *  RUNNING = timer running, not blocked
 *  BLOCKED = function that modifies data in progress
 *  EXPIRED = alarm went off while blocked
 *  KILLED  = terminate signal received while blocked
 */
#define TIMER_OFF	0
#define TIMER_RUNNING	1
#define TIMER_BLOCKED	2
#define TIMER_EXPIRED	3
#define TIMER_KILLED	4

char
#ifdef DEBUG
    remhost[MAXHOSTNAMELEN + 1],	/* Remote host, for debugging	*/
#endif
    *mb_datafile = CNULL,		/* Name of the data file	*/
    *pmsg_dir = CNULL,			/* Directory phone messages are in */
    *logfilename = CNULL;		/* Name of logfile. */

int
    users = 0,				/* Number of users in datalist	*/
    timer_status = TIMER_OFF,
    init_client_cnt,			/* counters for usage stats	*/
    get_rec_cnt,
    put_rec_cnt,
    get_mod_rec_cnt, gmr_null_cnt,
    send_page_cnt,
    send_pmsg_cnt,
    scan_pmsg_cnt,
    read_pmsg_cnt,
    mv_pmsgs_cnt;

Server_Data	*s_list;		/* The server's list of users	*/

Global_Rec	globrec;		/*  Global data			*/

init_data srvinfo;			/*  Initial server info, such as
					    required authorization levels
					    and start time */

#endif
/*
 *  "cstok" stands for "Character-Separator TOKen" -- it works the same way
 *  "strtok" does except that the second argument is a single character rather
 *  than a string, and the token separator in the input string is always
 *  assumed to be exactly one character long, rather than one or more
 *  characters long.
 */
char *
cstok(start, sep)
char *start, sep;
{
    static char *p = CNULL;

    if (start == CNULL)
	start = p;
    else
	p = start;
    if (p == CNULL || *p == '\0') return CNULL;

    while (*p != sep && *p != '\0') ++p;

    if (*p != '\0')
	*p++ = '\0';

    return start;
}
#ifdef FIXITINTHEMIX

/*
 *  Look up a user on this machine's UID.  If "logid" is not a valid
 *  UNIX login ID on this machine, return BOGUS.
 */
int
get_uid_by_logid(logid)
char *logid;
{
    struct passwd *pwbuf;

    pwbuf = getpwnam(logid);
    if ( pwbuf != (struct passwd *)NULL)
	return pwbuf->pw_uid;
    else 
	return BOGUS;
}

/*
 *  Makes sure a string does not contain the field separator character.
 */
void
strclean(s)
char *s;
{
    char *p;

    while( (p = strchr(s, SEP)) != CNULL )
	*p = '!';
}

void
logmsg(fmt, s1, s2, s3)
char *fmt, *s1, *s2, *s3;
{
    struct stat statbuf;
    time_t now;
    FILE *logfile;
    char timestr[26];

    if (logfilename == CNULL) find_files();
    if (stat(logfilename, &statbuf) != 0 ||
	(logfile = fopen(logfilename, "a")) == FNULL) return;

    now = time(NULL);
    strcpy(timestr, ctime(&now));
    timestr[19] = '\0';
    fprintf(logfile, "%s ", timestr + 4);
    fprintf(logfile, fmt, s1, s2, s3);
    fputc('\n', logfile);
    fclose(logfile);
}

/*
 *  Flushes the data to a checkpoint file.
 */
void
write_cpfile()
{
    int i;
    FILE *dbfile;
    
    if ( (dbfile = fopen(mb_datafile, "w")) == FNULL) {
#ifdef DEBUG
	logmsg("Checkpoint failed!");
#endif
	return;
    }

    strclean(globrec.name);
    strclean(globrec.motd);
    fprintf(dbfile, "%s\034%d\034%s\034%s\034%d\034%d\n",
	DF_COOKIE, DF_VERSION, globrec.name, globrec.motd, globrec.flags,
	globrec.pagertype);
    for (i = 0; i < users; i++) {
	/*
	 *  Don't write deleted records to the checkpoint file.
	 */
	if (s_list[i].status != BOGUS) {
	    strclean(s_list[i].logid);
	    strclean(s_list[i].fname);
	    strclean(s_list[i].until);
	    strclean(s_list[i].comment);
	    strclean(s_list[i].cat);
	    strclean(s_list[i].sortby);
	    strclean(s_list[i].ext);
	    strclean(s_list[i].pagernum);
	    fprintf(dbfile, "%s\034%s\034%s\034%s\034%s\034%s\034%s\034%s",
		s_list[i].logid,
		s_list[i].fname,
		s_list[i].until,
		s_list[i].comment,
		s_list[i].cat,
		s_list[i].sortby,
		s_list[i].ext,
		s_list[i].pagernum);
	    fprintf(dbfile, "\034%d\034%d\034%d\034%ld\n",
		s_list[i].status,
		s_list[i].flags,
		s_list[i].pagertype,
		s_list[i].timestamp);
	}
    }

    fclose(dbfile);
#ifdef DEBUG
    logmsg("Checkpointed.");
#endif
}
#endif

void
init_server()
{
    char inline[DFINBUFSIZE];
    Mem_Rec *cur_rec;
    FILE *dbfile;
    struct stat statbuf;
    int i;
    struct timeval now;

    if ( (dbfile = fopen("msgdata", "r")) == FNULL) {
	exit(1);
    }

    /*
     *  Read header
     */
    if ( fgets(inline, DFINBUFSIZE, dbfile) == CNULL) exit(1);
    if (inline[strlen(inline)] == '\n') inline[strlen(inline)] = '\0';
    if (strcmp(cstok(inline, SEP), DF_COOKIE) != 0) {
	exit(1);
    }
    if (atoi(cstok(NULL, SEP)) != DF_VERSION) {
	exit(1);
    }

    strncpy(globrec.name, cstok(NULL, SEP), SITE_LEN+1);
    strncpy(globrec.motd, cstok(NULL, SEP), MOTD_LEN+1);
    globrec.flags = atoi(cstok(NULL, SEP));
    globrec.pagertype = atoi(cstok(NULL, SEP));
    globals_seq_no = ++highest_seq_no;

    /*
     *  Loop to read in remaining lines
     */
    users = 0;
    while ( fgets(inline, DFINBUFSIZE, dbfile) != CNULL) {
	if (inline[strlen(inline)] == '\n') inline[strlen(inline)] = '\0';
	/*
	 * Allocate a slot
	 */
	if (++users == 1) {
	    ulist = (Mem_Rec *)malloc(
		(unsigned)sizeof(Mem_Rec) );
	    seq_no = (unsigned int *)malloc(sizeof(unsigned int));
	}
	else {
	    ulist = (Mem_Rec *)realloc( (char *)ulist,
		(unsigned)( sizeof(Mem_Rec) * users ) );
	    seq_no = (unsigned int *)realloc( (char *)seq_no, sizeof(unsigned int));
	}

	cur_rec = &ulist[users-1];

	strncpy(cur_rec->logid, cstok(inline, SEP), LOGID_LEN+1);
	strncpy(cur_rec->fname, cstok(NULL, SEP), FNAME_LEN+1);
	strncpy(cur_rec->until, cstok(NULL, SEP), UNTIL_LEN+1);
	strncpy(cur_rec->comment, cstok(NULL, SEP), COMMENT_LEN+1);
	strncpy(cur_rec->cat, cstok(NULL, SEP), CAT_LEN+1);
	strncpy(cur_rec->sortby, cstok(NULL, SEP), SORTBY_LEN+1);
	strncpy(cur_rec->ext, cstok(NULL, SEP), EXT_LEN+1);
	strncpy(cur_rec->pagernum, cstok(NULL, SEP), PAGERNUM_LEN+1);
	cur_rec->status = atoi(cstok(NULL, SEP));
	cur_rec->flags = atoi(cstok(NULL, SEP));
	cur_rec->pagertype = atoi(cstok(NULL, SEP));
	cur_rec->timestamp = atol(cstok(NULL, SEP));
	seq_no[users-1] = ++highest_seq_no;
    }

    fclose(dbfile);

    initialized = TRUE;
}
#ifdef FIXITINMIX

/*
 *  Used by lsearch/lfind to do lookups in s_list by login ID
 */
int
comp_by_logid(i, j)
Server_Data *i, *j;
{
    return strcmp(i->logid, j->logid);
}

/*
 *  Used by lsearch/lfind to do lookups in s_list by user ID
 */
int
comp_by_uid(i, j)
Server_Data *i, *j;
{
    return (i->uid == j->uid) ? 0 : 1;
}

/*
 *  Validate the calling host; if the user ID pointer uidp is non-NULL,
 *  return the validated user ID.
 *
 *  Currently, the host is only validated
 *  if the authorization level is AUTH_DES, and then only to the extent of
 *  "is this host in our NIS domain?"  User validation works for both
 *  AUTH_UNIX and AUTH_DES, but AUTH_UNIX is trivially easy to spoof.
 *
 *  TO DO: read a file of additional trusted hosts beyond the current NIS
 *  domain.  These hosts should also be notified via init_client() that their allowed validation
 *  level is AUTH_UNIX instead of AUTH_DES.
 */
int
ok_user(rqstp, uidp, level)
struct svc_req *rqstp;
int *uidp, level;
{
    struct sockaddr_in *sockaddr;
    u_long addr;
    int retval;

    /*
     *@  Get the caller's host address.  (This is where we should verify it
     *@  against list of allowed/disallowed hosts)
     */
    sockaddr = svc_getcaller(rqstp->rq_xprt);
    addr = sockaddr->sin_addr.s_addr;
    /*
     *  If no authentication required for this service, return immediately.
     *  If they asked for UID authentication, they're out of luck.
     */
    if (level == AUTH_NONE) {
	if (uidp != (int *)NULL) *uidp = BOGUS;
	return TRUE;
    }

    /*
     *  For UNIX and DES authentication, use the intrinisic checks for
     *  validation.  Also, return FALSE if they're using the wrong level
     *  of authentication.
     */
    if (uidp != (int *)NULL) {
	switch (rqstp->rq_cred.oa_flavor) {
	case AUTH_UNIX:
	    {
		struct authunix_parms *aup;

		aup = (struct authunix_parms *)rqstp->rq_clntcred;
		*uidp = aup->aup_uid;
		retval = (level == AUTH_UNIX);
	    }
	    break;
	case AUTH_DES:
	    {
		struct authdes_cred *des_cred;
		int gid, gidlen, gidlist[10];
		char host[MAXHOSTNAMELEN];

		des_cred = (struct authdes_cred *)rqstp->rq_clntcred;
		if ( ! netname2user(des_cred->adc_fullname.name,
		    uidp, &gid, &gidlen, gidlist))
		    return FALSE;
		retval = (level == AUTH_DES);
	    }
	    break;
	}
    }
    return retval;
}

/*
 *  gets record based on UID.
 */
Server_Data *
get_rec_by_uid(uid)
int uid;
{
    Server_Data tmp_record;

    tmp_record.uid = uid;
    return (Server_Data *)lfind((char *)&tmp_record,
	(char *)s_list, &users, sizeof(Server_Data), comp_by_uid);
}

/*
 *  Does this UID have admin privs?
 */
int
is_admin(uid)
int uid;
{
    Server_Data *recp;

    recp = get_rec_by_uid(uid);
    if (recp == (Server_Data *)NULL)
	return FALSE;

    return (recp->flags & MB_ADMIN) != 0;
}

/*
 *  Part of the routine to caches writes.  After we update any of the server
 *  data, we start a timer; actual flush to disk is deferred until the
 *  timer expires.  If the timer is already running, this call does
 *  nothing.
 */
void
start_timer()
{
    struct itimerval ntime, otime;

    switch (timer_status) {
    case TIMER_EXPIRED:
	/*
	 *  Timer went off in between the time we set the block and now.
	 *  Perform the flush now.
	 */
	write_cpfile();
	timer_status == TIMER_OFF;
	break;
    case TIMER_OFF:
	/*
	 *  No timer in effect -- start it.
	 */
	timerclear(&(ntime.it_interval));
	ntime.it_value.tv_sec = CACHE_TIME;
	ntime.it_value.tv_usec = 0;

	setitimer(ITIMER_REAL, &ntime, &otime);
	timer_status = TIMER_RUNNING;
	break;
    case TIMER_BLOCKED:
	/*
	 *  Timer was running when we started our update, so we blocked it;
	 *  unblock it.
	 */
	timer_status = TIMER_RUNNING;
	break;
    case TIMER_KILLED:
	/*
	 *  Got a quit signal while we were doing our update.  Flush and exit.
	 */
	write_cpfile();
	exit(0);
    }
}

/*
 *  Copies incoming data to server record pointed to by "rec"; increments
 *  sequence number; updates timestamp.
 *
 *  Does not copy logid; also does not assign value to uid.
 */
void
copy_data_to_rec(rec_p, data_p)
Server_Data *rec_p;
user_data *data_p;
{
    struct timeval now;

    strcpy(rec_p->fname, data_p->fname);
    strcpy(rec_p->until, data_p->until);
    strcpy(rec_p->comment, data_p->comment);
    strcpy(rec_p->cat, data_p->cat);
    strcpy(rec_p->sortby, data_p->sortby);
    strcpy(rec_p->ext, data_p->ext);
    strcpy(rec_p->pagernum, data_p->pagernum);
    rec_p->status = data_p->status;
    rec_p->flags = data_p->flags;
    rec_p->pagertype = data_p->pagertype;
    rec_p->seq_no = ++highest_seq_no;
    gettimeofday(&now, (struct timezone *)NULL);
    rec_p->timestamp = now.tv_sec;
}

/*
 *  Copies a server record to the RPC data struct
 */
void
copy_rec_to_data(data_p, rec_p)
user_data *data_p;
Server_Data *rec_p;
{	
	data_p->logid = rec_p->logid;
	data_p->fname = rec_p->fname;
	data_p->until = rec_p->until;
	data_p->comment = rec_p->comment;
	data_p->cat = rec_p->cat;
	data_p->sortby = rec_p->sortby;
	data_p->ext = rec_p->ext;
	data_p->pagernum = rec_p->pagernum;
	data_p->status = rec_p->status;
	data_p->flags = rec_p->flags;
	data_p->pagertype = rec_p->pagertype;
	data_p->seq_no = rec_p->seq_no;
	data_p->timestamp = rec_p->timestamp;
}

/***************************************
 *  The actual remotely-called procedures:
 ***************************************/

/*
 *  Returns initialization information to a client.  Since the clients need to
 *  call this to find out what auth levels they need, no authorization is
 *  needed to make this call.
 */
init_data *
init_client_1()
{

    init_client_cnt++;

    if (!initialized) init_server();
    
    return &srvinfo;
}

/*
 *  Returns either a user record (keyed on login ID), or the global record.
 */
get_rec_data *
get_rec_1(req, rqstp)
get_rec_req *req;
struct svc_req *rqstp;
{
    static get_rec_data result;
    Server_Data *rec_ptr, tmp_record;


    get_rec_cnt++;

    if (!ok_user(rqstp, (int *)NULL, srvinfo.get_auth)) {
	result.retval = MSGD_VOID;
	return &result;
    }

    if (!initialized) init_server();

    if ( req->req_type == MSGD_GREC ) {
	result.retval = MSGD_GREC;
	result.get_rec_data_u.g_dat.site = globrec.name;
	result.get_rec_data_u.g_dat.motd = globrec.motd;
	result.get_rec_data_u.g_dat.flags = globrec.flags;
	result.get_rec_data_u.g_dat.pagertype = globrec.pagertype;
	result.get_rec_data_u.g_dat.seq_no = globals_seq_no;
    }
    else {
	strcpy(tmp_record.logid, req->get_rec_req_u.u_rec.logid);
	rec_ptr = (Server_Data *)lfind((char *)&tmp_record,
	    (char *)s_list, &users, sizeof(Server_Data), comp_by_logid);
	if (rec_ptr == (Server_Data *)NULL || rec_ptr->status == BOGUS) {
	    result.retval = MSGD_VOID;
	}
	else {
	    result.retval = MSGD_UREC;
	    copy_rec_to_data(&(result.get_rec_data_u.u_dat), rec_ptr);
	}
    }
    return &result;
}

/*
 *  Modifies, adds or deletes a record.  Returns MSGD_OK on success, error
 *  code on fail.  If srvinfo.put_auth is AUTH_NONE,
 *  then anyone can perform this operation on any record, which is probably
 *  not the greatest idea.
 */
unsigned int *
put_rec_1(data, rqstp)
put_rec_data *data;
struct svc_req *rqstp;
{
    static unsigned int result;
    Server_Data *rec_ptr, tmp_record;
    int r_uid, no_auth_reqd;
    user_data *dptr;


    put_rec_cnt++;

    if (!ok_user(rqstp, &r_uid, srvinfo.put_auth)) {
	result = MSGD_VALID_FAIL;
	return &result;
    }
    no_auth_reqd = (srvinfo.put_auth == AUTH_NONE);

    if (!initialized) init_server();

    if (timer_status == TIMER_RUNNING) timer_status = TIMER_BLOCKED;

    /*
     *  Handle a global record
     */
    if ( data->req_type == MSGD_GREC ) {
	global_data *g_rec = &(data->put_rec_data_u.g_rec);

	strcpy(globrec.name, g_rec->site);
	strcpy(globrec.motd, g_rec->motd);
	/*
	 *  We don't want MB_G_TRUSTED to be set from clients.  Mask it
	 *  out and add it back in from the current setting.
	 */
	globrec.flags = (g_rec->flags & ~MB_G_TRUSTED) |
			(globrec.flags & MB_G_TRUSTED);
	globrec.pagertype = g_rec->pagertype;
	globals_seq_no = ++highest_seq_no;

	result = MSGD_OK;
	start_timer();
	return &result;
    }

    /*
     *  For remaining operations, try to find a record matching the request
     */
    if ( data->req_type == MSGD_DREC )
	strcpy(tmp_record.logid, data->put_rec_data_u.d_rec.logid);
    else if ( data->req_type == MSGD_AREC )    
	strcpy(tmp_record.logid, data->put_rec_data_u.a_rec.logid);
    else if ( data->req_type == MSGD_UREC )    
	strcpy(tmp_record.logid, data->put_rec_data_u.u_rec.logid);
    rec_ptr = (Server_Data *)lfind((char *)&tmp_record,
	(char *)s_list, &users, sizeof(Server_Data), comp_by_logid);

    /*
     *  Handle a delete request
     */
    if ( data->req_type == MSGD_DREC ) {
	if (rec_ptr == (Server_Data *)NULL || rec_ptr->status == BOGUS)
	    result = MSGD_NOT_FOUND;
	else if (no_auth_reqd || is_admin(r_uid)) {
	    rec_ptr->status = BOGUS;
	    rec_ptr->seq_no = ++highest_seq_no;
	    result = MSGD_OK;
	}
	else
    	    result = MSGD_VALID_FAIL;
    }

    /*
     *  Handle an ADD request
     */
    else if ( data->req_type == MSGD_AREC ) {
	if (rec_ptr == (Server_Data *)NULL || rec_ptr->status == BOGUS) {
	    if (no_auth_reqd || is_admin(r_uid)) {
		dptr = &(data->put_rec_data_u.a_rec);
		if ( rec_ptr == (Server_Data *)NULL ) {
		    if (++users == 1)
			s_list = (Server_Data *)malloc( sizeof(Server_Data) );
		    else
			s_list = (Server_Data *)realloc( (char *)s_list,
			    (unsigned)( sizeof(Server_Data) * users ) );
		    rec_ptr = &s_list[users-1];
		    strcpy(rec_ptr->logid, dptr->logid);
		}
		rec_ptr->uid = get_uid_by_logid(rec_ptr->logid);
		copy_data_to_rec(rec_ptr, dptr);
		result = MSGD_OK;
	    }
	    else
		result = MSGD_VALID_FAIL;
	}
	else
	    result = MSGD_ALREADY_EXISTS;
    }

    /*
     *  Handle a modify request
     */
    else {
	if (rec_ptr == (Server_Data *)NULL || rec_ptr->status == BOGUS)
	    result = MSGD_NOT_FOUND;
	else if (no_auth_reqd || (r_uid == rec_ptr->uid || is_admin(r_uid))) {
	    dptr = &(data->put_rec_data_u.u_rec);
	    /*
	     *  If this user does not have admin caps, strip out any attempt
	     *  to set the admin bit.
	     */
	    if (!no_auth_reqd && !is_admin(r_uid))
		dptr->flags &= (~MB_ADMIN | (rec_ptr->flags & MB_ADMIN));
	    copy_data_to_rec(rec_ptr, dptr);
	    result = MSGD_OK;
	}
	else
	    result = MSGD_VALID_FAIL;
    }
    if (result == MSGD_OK) start_timer();
    return &result;
}
#endif

/*
 *  Looks for a record with a sequence number higher than the one passed
 *  and returns it if found.  Returns error number + 1000 on errors.
 */
struct bogosity {
    enum msgd_types ret_type;
    unsigned int seq_no;
    Mem_Rec *rec;
};

struct bogosity *
get_mod_rec_1()
{
    int i;
    unsigned int found_seq_no;
    Mem_Rec *p, *found_rec;
    static struct bogosity result;

    result.ret_type = MSGD_VOID;
    if ( last_seq_no < globals_seq_no ) {
	result.ret_type = MSGD_GREC;
	result.seq_no = globals_seq_no;
    }
    else
	for ( i = 0 ; i < users ; i++ )
	    if ( seq_no[i] > last_seq_no ) {
		result.ret_type = MSGD_UREC;
		result.seq_no = seq_no[i];
		result.rec = &ulist[i];
		break;
	    }

    return &result;
}
